@@ -27,6 +27,7 @@
 
 #include <ags/widget/ags_dial.h>
 
+#include <ags/thread/ags_mutex_manager.h>
 #include <ags/thread/ags_audio_loop.h>
 #include <ags/thread/ags_task_thread.h>
 
@@ -62,6 +63,8 @@
 
 #include <ladspa.h>
 
+extern pthread_mutex_t ags_application_mutex;
+
 void
 ags_line_member_editor_add_callback(GtkWidget *button,
 				    AgsLineMemberEditor *line_member_editor)
@@ -77,8 +80,7 @@ ags_line_member_editor_ladspa_browser_response_callback(GtkDialog *dialog,
   switch(response){
   case GTK_RESPONSE_ACCEPT:
     {
-      AgsAudioLoop *audio_loop;
-      AgsTaskThread *task_thread;
+      AgsWindow *window;
       AgsLine *line;
       AgsLineMember *line_member;
       AgsMachineEditor *machine_editor;
@@ -100,6 +102,12 @@ ags_line_member_editor_ladspa_browser_response_callback(GtkDialog *dialog,
 
       AgsLadspaPlugin *ladspa_plugin;
 
+      AgsMutexManager *mutex_manager;
+      AgsAudioLoop *audio_loop;
+      AgsTaskThread *task_thread;
+
+      AgsMain *ags_main;
+    
       GList *plugin;
       GList *task;
       GList *port;
@@ -117,6 +125,8 @@ ags_line_member_editor_ladspa_browser_response_callback(GtkDialog *dialog,
       LADSPA_Data lower_bound, upper_bound;
       long i;
 
+      pthread_mutex_t *audio_loop_mutex;
+
       machine_editor = (AgsMachineEditor *) gtk_widget_get_ancestor((GtkWidget *) line_member_editor,
 								    AGS_TYPE_MACHINE_EDITOR);
       line_editor = (AgsLineEditor *) gtk_widget_get_ancestor((GtkWidget *) line_member_editor,
@@ -158,9 +168,22 @@ ags_line_member_editor_ladspa_browser_response_callback(GtkDialog *dialog,
       if(line == NULL){
 	return;
       }
-      
-      audio_loop = (AgsAudioLoop *) AGS_MAIN(AGS_DEVOUT(AGS_MACHINE(machine_editor->machine)->audio->devout)->ags_main)->main_loop;
-      task_thread = (AgsTaskThread *) audio_loop->task_thread;
+
+      /* get window and ags_main  */
+      window = (AgsWindow *) gtk_widget_get_toplevel(machine_editor->machine);
+  
+      ags_main = window->ags_main;
+
+      /* get audio loop */
+      pthread_mutex_lock(&(ags_application_mutex));
+
+      audio_loop = ags_main->main_loop;
+
+      pthread_mutex_unlock(&(ags_application_mutex));
+
+      /* get task thread */
+      task_thread = (AgsTaskThread *) ags_thread_find_type(audio_loop,
+							   AGS_TYPE_TASK_THREAD);
 
       /* retrieve plugin */
       filename = ags_ladspa_browser_get_plugin_filename(line_member_editor->ladspa_browser);
@@ -396,8 +419,8 @@ ags_line_member_editor_ladspa_browser_response_callback(GtkDialog *dialog,
 		    
 		    dial = (AgsDial *) child_widget;
 		    gtk_widget_set_size_request((GtkWidget *) dial,
-						2 * dial->radius + 2 * dial->outline_strength + dial->button_width + 1,
-						2 * dial->radius + 2 * dial->outline_strength + 1);
+						2 * dial->radius + 2 * (dial->outline_strength + dial->button_width + 4),
+						2 * dial->radius + 2 * (dial->outline_strength + 1));
 		    g_object_set(dial,
 				 "adjustment", adjustment,
 				 NULL);
@@ -460,13 +483,19 @@ void
 ags_line_member_editor_remove_callback(GtkWidget *button,
 				       AgsLineMemberEditor *line_member_editor)
 {
-  AgsAudioLoop *audio_loop;
-  AgsTaskThread *task_thread;
+  AgsWindow *window;
   AgsLine *line;
   AgsMachineEditor *machine_editor;
   AgsLineEditor *line_editor;
+
   AgsRemoveRecall *remove_recall;
   AgsRemoveRecallContainer *remove_recall_container;
+  
+  AgsAudioLoop *audio_loop;
+  AgsTaskThread *task_thread;
+
+  AgsMain *ags_main;
+  
   GList *control;
   GList *line_member;
   GList *children;
@@ -485,9 +514,21 @@ ags_line_member_editor_remove_callback(GtkWidget *button,
 								AGS_TYPE_MACHINE_EDITOR);
   line_editor = (AgsLineEditor *) gtk_widget_get_ancestor((GtkWidget *) line_member_editor,
 							  AGS_TYPE_LINE_EDITOR);
+  window = gtk_widget_get_ancestor(machine_editor->machine,
+				   AGS_TYPE_WINDOW);
+  
+  ags_main = window->ags_main;
 
-  audio_loop = (AgsAudioLoop *) AGS_MAIN(AGS_DEVOUT(AGS_MACHINE(machine_editor->machine)->audio->devout)->ags_main)->main_loop;
-  task_thread = (AgsTaskThread *) audio_loop->task_thread;
+  /* get audio loop */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  audio_loop = (AgsAudioLoop *) ags_main->main_loop;
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* get task thread */
+  task_thread = (AgsTaskThread *) ags_thread_find_type(audio_loop,
+						       AGS_TYPE_TASK_THREAD);
 
   line_member = gtk_container_get_children(GTK_CONTAINER(line_member_editor->line_member));
   task = NULL;
