@@ -27,8 +27,10 @@
 #include <ags/object/ags_marshal.h>
 #include <ags/object/ags_plugin.h>
 
+#include <ags/thread/ags_mutex_manager.h>
 #include <ags/thread/ags_audio_loop.h>
 #include <ags/thread/ags_task_thread.h>
+#include <ags/thread/ags_task_completion.h>
 #include <ags/thread/ags_gui_thread.h>
 
 #include <ags/file/ags_file.h>
@@ -37,6 +39,7 @@
 
 #include <ags/audio/ags_output.h>
 #include <ags/audio/ags_input.h>
+#include <ags/audio/ags_pattern.h>
 
 #include <ags/audio/file/ags_audio_file.h>
 
@@ -101,6 +104,8 @@ enum{
 static gpointer ags_machine_parent_class = NULL;
 static guint machine_signals[LAST_SIGNAL];
 
+extern pthread_mutex_t ags_application_mutex;
+
 GType
 ags_machine_get_type(void)
 {
@@ -251,7 +256,9 @@ ags_machine_plugin_interface_init(AgsPluginInterface *plugin)
 void
 ags_machine_init(AgsMachine *machine)
 {
+  GtkVBox *vbox;
   GtkFrame *frame;
+  GtkMenuToolButton *menu_tool_button;
 
   machine->flags = 0;
   machine->file_input_flags = 0;
@@ -267,6 +274,8 @@ ags_machine_init(AgsMachine *machine)
 
   machine->bank_0 = 0;
   machine->bank_1 = 0;
+
+  vbox = gtk_vbox_new(FALSE, 0);
   
   frame = (GtkFrame *) gtk_frame_new(NULL);
   gtk_container_add((GtkContainer *) machine, (GtkWidget *) frame);
@@ -285,11 +294,21 @@ ags_machine_init(AgsMachine *machine)
   machine->play = NULL;
 
   machine->output = NULL;
+  machine->selected_output_pad = NULL;
+
   machine->input = NULL;
+  machine->selected_input_pad = NULL;
 
   machine->port = NULL;
 
   machine->popup = ags_machine_popup_new(machine);
+
+  menu_tool_button = g_object_new(GTK_TYPE_MENU_TOOL_BUTTON,
+				  "label\0", "machine\0",
+				  "menu\0", machine->popup,
+				  NULL);
+  gtk_frame_set_label_widget(frame,
+			     menu_tool_button);
   machine->properties = NULL;
   machine->rename = NULL;
 }
@@ -558,8 +577,8 @@ ags_machine_connect(AgsConnectable *connectable)
   }
 
   /* GtkWidget */
-  g_signal_connect(G_OBJECT (machine), "button_press_event\0",
-		   G_CALLBACK(ags_machine_button_press_callback), (gpointer) machine);
+  //  g_signal_connect(G_OBJECT (machine), "button_press_event\0",
+  //		   G_CALLBACK(ags_machine_button_press_callback), (gpointer) machine);
 
   /* AgsPad - input */
   if(machine->input != NULL){
@@ -745,7 +764,7 @@ ags_machine_get_possible_links(AgsMachine *machine)
       list = list->next;
     }
   }
-
+  
   return(model);
 }
 
@@ -846,8 +865,37 @@ ags_machine_set_run_extended(AgsMachine *machine,
   AgsWindow *window;
   AgsThread *task_thread;
 
+  AgsMutexManager *mutex_manager;
+  AgsAudioLoop *audio_loop;
+
+  AgsMain *ags_main;
+  
+  pthread_mutex_t *audio_loop_mutex;
+  
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) machine);
-  task_thread = (AgsThread *) AGS_AUDIO_LOOP(AGS_MAIN(window->ags_main)->main_loop)->task_thread;
+
+  ags_main = (AgsMain *) window->ags_main;
+
+  /* get threads */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  audio_loop = (AgsAudioLoop *) ags_main->main_loop;
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* lookup audio loop mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_loop_mutex = ags_mutex_manager_lookup(mutex_manager,
+					      (GObject *) audio_loop);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* get task thread */
+  task_thread = (AgsTaskThread *) ags_thread_find_type(audio_loop,
+						       AGS_TYPE_TASK_THREAD);
 
   if(run){
     AgsInitAudio *init_audio;
@@ -863,7 +911,7 @@ ags_machine_set_run_extended(AgsMachine *machine,
     list = g_list_prepend(list, init_audio);
     
     /* create append task */
-    append_audio = ags_append_audio_new(G_OBJECT(AGS_MAIN(window->ags_main)->main_loop),
+    append_audio = ags_append_audio_new(audio_loop,
 					(GObject *) machine->audio);
 
     list = g_list_prepend(list, append_audio);
@@ -874,7 +922,7 @@ ags_machine_set_run_extended(AgsMachine *machine,
     list = g_list_prepend(list, init_audio);
 
     /* create append task */
-    append_audio = ags_append_audio_new(G_OBJECT(AGS_MAIN(window->ags_main)->main_loop),
+    append_audio = ags_append_audio_new(audio_loop,
 					(GObject *) machine->audio);
 
     list = g_list_prepend(list, append_audio);
@@ -884,8 +932,9 @@ ags_machine_set_run_extended(AgsMachine *machine,
       AgsGuiThread *gui_thread;
       AgsTaskCompletion *task_completion;
 
-      gui_thread = (AgsGuiThread *) AGS_AUDIO_LOOP(AGS_MAIN(window->ags_main)->main_loop)->gui_thread;
-	
+      gui_thread = (AgsGuiThread *) ags_thread_find_type(audio_loop,
+							 AGS_TYPE_GUI_THREAD);
+
       start_devout = ags_start_devout_new(window->devout);
       list = g_list_prepend(list, start_devout);
 
@@ -893,9 +942,10 @@ ags_machine_set_run_extended(AgsMachine *machine,
 						NULL);
       g_signal_connect_after(G_OBJECT(task_completion), "complete\0",
 			     G_CALLBACK(ags_machine_start_complete_callback), machine);
+      ags_connectable_connect(AGS_CONNECTABLE(task_completion));
+      
       gui_thread->task_completion = g_list_prepend(gui_thread->task_completion,
 						   task_completion);
-      ags_connectable_connect(AGS_CONNECTABLE(task_completion));
       
       /* append AgsStartDevout */
       list = g_list_reverse(list);
@@ -922,26 +972,66 @@ ags_machine_set_audio_channels(AgsAudio *audio,
 			       AgsMachine *machine)
 {
   AgsChannel *channel;
+
+  AgsMutexManager *mutex_manager;
+
   GList *list_output_pad, *list_output_pad_start;
   GList *list_input_pad, *list_input_pad_start;
   GList *list_output_pad_next, *list_output_pad_next_start;
   GList *list_input_pad_next, *list_input_pad_next_start;
+
   guint i, j;
+
+  pthread_mutex_t *audio_mutex;
+  pthread_mutex_t *channel_mutex;
+
+  /* lookup audio mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
   
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					 (GObject *) audio);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
   if(audio_channels > audio_channels_old){
     /* grow lines */
     AgsPad *pad;
+
     GList *list, *list_start;
 
+    guint input_pads, output_pads;
+
+    pthread_mutex_lock(audio_mutex);
+
+    input_pads = audio->input_pads;
+    output_pads = audio->output_pads;
+    
+    pthread_mutex_unlock(audio_mutex);
+
     /* AgsInput */
     if(machine->input != NULL){
       list_input_pad_start = 
 	list_input_pad = g_list_reverse(gtk_container_get_children((GtkContainer *) machine->input));
+      /* get input */
+      pthread_mutex_lock(audio_mutex);
+
       channel = audio->input;
 
-      for(i = 0; i < audio->input_pads; i++){
+      pthread_mutex_unlock(audio_mutex);
+
+      for(i = 0; i < input_pads; i++){
+	/* lookup channel mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+
+	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+						 (GObject *) channel);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* create AgsPad's if necessary or resize */
 	if(audio_channels_old == 0){
-	  /* create AgsPad's if necessary */
 	  pad = g_object_new(machine->input_pad_type,
 			     "channel\0", channel,
 			     NULL);
@@ -951,7 +1041,7 @@ ags_machine_set_audio_channels(AgsAudio *audio,
 			     0);
 
 	  ags_pad_resize_lines((AgsPad *) pad, machine->input_line_type,
-			       audio->audio_channels, 0);
+			       audio_channels, 0);
 	}else{
 	  pad = AGS_PAD(list_input_pad->data);
 
@@ -959,8 +1049,13 @@ ags_machine_set_audio_channels(AgsAudio *audio,
 			       audio_channels, audio_channels_old);
 	}
 
+	/* iterate */
+	pthread_mutex_lock(channel_mutex);
+	
 	channel = channel->next_pad;
-
+	
+	pthread_mutex_unlock(channel_mutex);
+	
 	if(audio_channels_old != 0){
 	  list_input_pad = list_input_pad->next;
 	}
@@ -973,11 +1068,24 @@ ags_machine_set_audio_channels(AgsAudio *audio,
     if(machine->output != NULL){
       list_output_pad_start = 
 	list_output_pad = g_list_reverse(gtk_container_get_children((GtkContainer *) machine->output));
+      /* get output */
+      pthread_mutex_lock(audio_mutex);
+
       channel = audio->output;
 
-      for(i = 0; i < audio->output_pads; i++){
+      pthread_mutex_unlock(audio_mutex);
+
+      for(i = 0; i < output_pads; i++){
+	/* lookup channel mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+
+	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+						 (GObject *) channel);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* create AgsPad's if necessary or resize */
 	if(audio_channels_old == 0){
-	  /* create AgsPad's if necessary */
 	  pad = g_object_new(machine->output_pad_type,
 			     "channel\0", channel,
 			     NULL);
@@ -986,7 +1094,7 @@ ags_machine_set_audio_channels(AgsAudio *audio,
 			     FALSE, FALSE,
 			     0);
 	  ags_pad_resize_lines((AgsPad *) pad, machine->output_line_type,
-			       AGS_AUDIO(channel->audio)->audio_channels, 0);
+			       audio_channels, 0);
 	}else{
 	  pad = AGS_PAD(list_output_pad->data);
 
@@ -994,7 +1102,12 @@ ags_machine_set_audio_channels(AgsAudio *audio,
 			       audio_channels, audio_channels_old);
 	}
 
+	/* iterate */
+	pthread_mutex_lock(channel_mutex);
+	
 	channel = channel->next_pad;
+	
+	pthread_mutex_unlock(channel_mutex);
 
 	if(audio_channels_old != 0){
 	  list_output_pad = list_output_pad->next;
@@ -1048,11 +1161,13 @@ ags_machine_set_audio_channels(AgsAudio *audio,
       }
     }
 
-    if(list_output_pad_start)
+    if(list_output_pad_start){
       g_list_free(list_output_pad_start);
-
-    if(list_input_pad_start)
+    }
+    
+    if(list_input_pad_start){
       g_list_free(list_input_pad_start);
+    }
   }  
 }
 
@@ -1062,17 +1177,51 @@ ags_machine_set_pads(AgsAudio *audio, GType type,
 		     AgsMachine *machine)
 {
   AgsPad *pad;
+
   AgsChannel *channel;
+  AgsChannel *input, *output;
+  
+  AgsMutexManager *mutex_manager;
+
   guint i, j;
 
+  pthread_mutex_t *audio_mutex;
+  pthread_mutex_t *channel_mutex;
+
+  /* lookup audio mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					 (GObject *) audio);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
   if(pads_old < pads){
-    /* input  */
+    pthread_mutex_lock(audio_mutex);
+
+    input = audio->input;
+    output = audio->output;
+    
+    pthread_mutex_unlock(audio_mutex);
+
+    /* grow input */
     if(machine->input != NULL){
       if(type == AGS_TYPE_INPUT){
-	channel = ags_channel_nth(audio->input,
+	channel = ags_channel_nth(input,
 				  pads_old * audio->audio_channels);
       
 	for(i = pads_old; i < pads; i++){
+	  /* lookup channel mutex */
+	  pthread_mutex_lock(&(ags_application_mutex));
+
+	  channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+						   (GObject *) channel);
+  
+	  pthread_mutex_unlock(&(ags_application_mutex));
+
+	  /* instantiate pad */
 	  pad = g_object_new(machine->input_pad_type,
 			     "channel\0", channel,
 			     NULL);
@@ -1080,25 +1229,46 @@ ags_machine_set_pads(AgsAudio *audio, GType type,
 			     (GtkWidget *) pad, FALSE, FALSE, 0);
 	  ags_pad_resize_lines((AgsPad *) pad, machine->input_line_type,
 			       audio->audio_channels, 0);
+	  
+	  /* iterate */
+	  pthread_mutex_lock(channel_mutex);
+	
 	  channel = channel->next_pad;
+	
+	  pthread_mutex_unlock(channel_mutex);
 	}
       }
-    }  
-    /* output */
+    }
+    
+    /* grow output */
     if(machine->output != NULL){
       if(type == AGS_TYPE_OUTPUT){
-	channel = ags_channel_nth(audio->output,
+	channel = ags_channel_nth(output,
 				  pads_old * audio->audio_channels);
     
 	for(i = pads_old; i < pads; i++){
+	  /* lookup channel mutex */
+	  pthread_mutex_lock(&(ags_application_mutex));
+
+	  channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+						   (GObject *) channel);
+  
+	  pthread_mutex_unlock(&(ags_application_mutex));
+
+	  /* instantiate pad */
 	  pad = g_object_new(machine->output_pad_type,
 			     "channel\0", channel,
 			     NULL);
 	  gtk_box_pack_start((GtkBox *) machine->output, (GtkWidget *) pad, FALSE, FALSE, 0);
 	  ags_pad_resize_lines((AgsPad *) pad, machine->output_line_type,
 			       audio->audio_channels, 0);
-      
+
+	  /* iterate */
+	  pthread_mutex_lock(channel_mutex);
+	
 	  channel = channel->next_pad;
+	
+	  pthread_mutex_unlock(channel_mutex);
 	}
       }
     }
@@ -1184,18 +1354,196 @@ ags_machine_open_files(AgsMachine *machine,
 		       gboolean overwrite_channels,
 		       gboolean create_channels)
 {
+  AgsWindow *window;
+
   AgsOpenFile *open_file;
 
+  AgsMutexManager *mutex_manager;
+  AgsAudioLoop *audio_loop;
+  AgsTaskThread *task_thread;
+
+  AgsMain *ags_main;
+
+  pthread_mutex_t *audio_loop_mutex;
+
+  window = (AgsWindow *) gtk_widget_get_toplevel(machine);
+  
+  ags_main = window->ags_main;
+
+  /* get audio loop */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  audio_loop = ags_main->main_loop;
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* lookup audio loop mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_loop_mutex = ags_mutex_manager_lookup(mutex_manager,
+					      (GObject *) audio_loop);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* get task thread */
+  task_thread = (AgsTaskThread *) ags_thread_find_type(audio_loop,
+						       AGS_TYPE_TASK_THREAD);
+
+  /* instantiate open file task */
   open_file = ags_open_file_new(machine->audio,
 				filenames,
 				overwrite_channels,
 				create_channels);
 
-  ags_task_thread_append_task(AGS_TASK_THREAD(AGS_AUDIO_LOOP(AGS_MAIN(AGS_DEVOUT(machine->audio->devout)->ags_main)->main_loop)->task_thread),
+  ags_task_thread_append_task(task_thread,
 			      AGS_TASK(open_file));
 
 }
 
+void
+ags_machine_copy_pattern(AgsMachine *machine)
+{
+  AgsAudio *audio;
+  AgsChannel *channel;
+  
+  AgsMutexManager *mutex_manager;
+
+  xmlDoc *clipboard;
+  xmlNode *audio_node, *notation_node;
+
+  xmlChar *buffer;
+  int size;
+  gint i;
+
+  pthread_mutex_t *audio_mutex;
+  pthread_mutex_t *current_mutex;
+
+  auto xmlNode* ags_machine_copy_pattern_to_notation(AgsChannel *current);
+
+  xmlNode* ags_machine_copy_pattern_to_notation(AgsChannel *current){
+    AgsPattern *pattern;
+    xmlNode *notation_node, *current_note;
+    guint x_boundary, y_boundary;
+    guint bank_0, bank_1, k;
+    
+    /* create root node */
+    notation_node = xmlNewNode(NULL, BAD_CAST "notation\0");
+
+    xmlNewProp(notation_node, BAD_CAST "program\0", BAD_CAST "ags\0");
+    xmlNewProp(notation_node, BAD_CAST "type\0", BAD_CAST AGS_NOTATION_CLIPBOARD_TYPE);
+    xmlNewProp(notation_node, BAD_CAST "version\0", BAD_CAST AGS_NOTATION_CLIPBOARD_VERSION);
+    xmlNewProp(notation_node, BAD_CAST "format\0", BAD_CAST AGS_NOTATION_CLIPBOARD_FORMAT);
+    xmlNewProp(notation_node, BAD_CAST "base_frequency\0", BAD_CAST g_strdup("0\0"));
+    xmlNewProp(notation_node, BAD_CAST "audio-channel\0", BAD_CAST g_strdup_printf("%u\0", current->audio_channel));
+
+    bank_0 = machine->bank_0;
+    bank_1 = machine->bank_1;
+    
+    x_boundary = G_MAXUINT;
+    y_boundary = G_MAXUINT;
+
+    while(current != NULL){
+      pattern = current->pattern->data;
+
+      for(k = 0; k < pattern->dim[2]; k++){
+	if(ags_pattern_get_bit(pattern, bank_0, bank_1, k)){
+	  current_note = xmlNewChild(notation_node, NULL, BAD_CAST "note\0", NULL);
+	  
+	  xmlNewProp(current_note, BAD_CAST "x\0", BAD_CAST g_strdup_printf("%u\0", k));
+	  xmlNewProp(current_note, BAD_CAST "x1\0", BAD_CAST g_strdup_printf("%u\0", k + 1));
+
+	  if((AGS_MACHINE_REVERSE_NOTATION & (machine->flags)) != 0){
+	    xmlNewProp(current_note, BAD_CAST "y\0", BAD_CAST g_strdup_printf("%u\0", machine->audio->input_pads - current->pad - 1));
+	  }else{
+	    xmlNewProp(current_note, BAD_CAST "y\0", BAD_CAST g_strdup_printf("%u\0", current->pad));
+	  }
+	  
+	  if(x_boundary > k){
+	    x_boundary = k;
+	  }
+      
+	  if((AGS_MACHINE_REVERSE_NOTATION & (machine->flags)) != 0){
+	    guint tmp;
+
+	    tmp = machine->audio->input_pads - current->pad - 1;
+	    
+	    if(y_boundary > tmp){
+	      y_boundary = tmp;
+	    }
+	  }else{
+	    if(y_boundary > current->pad){
+	      y_boundary = current->pad;
+	    }
+	  }
+	}
+      }
+      
+      current = current->next;
+    }
+
+    xmlNewProp(notation_node, BAD_CAST "x_boundary\0", BAD_CAST g_strdup_printf("%u\0", x_boundary));
+    xmlNewProp(notation_node, BAD_CAST "y_boundary\0", BAD_CAST g_strdup_printf("%u\0", y_boundary));
+
+    return(notation_node);
+  }
+  
+  /* create document */
+  clipboard = xmlNewDoc(BAD_CAST XML_DEFAULT_VERSION);
+
+  /* create root node */
+  audio_node = xmlNewNode(NULL, BAD_CAST "audio\0");
+  xmlDocSetRootElement(clipboard, audio_node);
+
+  audio = machine->audio;
+
+  /* lookup audio mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					 (GObject *) audio);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* copy to clipboard */
+  pthread_mutex_lock(audio_mutex);
+
+  channel = audio->input;
+
+  pthread_mutex_unlock(audio_mutex);
+
+  for(i = 0; i < machine->audio->audio_channels; i++){
+    /* lookup channel mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    current_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) channel);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* do it so */
+    pthread_mutex_lock(current_mutex);
+    
+    notation_node = ags_machine_copy_pattern_to_notation(channel);
+    xmlAddChild(audio_node, notation_node);
+
+    channel = channel->next;
+
+    pthread_mutex_unlock(current_mutex);
+  }
+  
+  /* write to clipboard */
+  xmlDocDumpFormatMemoryEnc(clipboard, &buffer, &size, "UTF-8\0", TRUE);
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD),
+			 buffer, size);
+  gtk_clipboard_store(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
+  
+  xmlFreeDoc(clipboard);
+}
+
 /**
  * ags_machine_new:
  * @devout: the assigned devout.
