@@ -31,11 +31,33 @@
 
 #include <ags/X/editor/ags_note_edit.h>
 
+#include <gdk/gdkkeysyms.h>
+
+#include <atk/atk.h>
+
+static GType ags_accessible_note_edit_get_type(void);
 void ags_note_edit_class_init(AgsNoteEditClass *note_edit);
+void ags_accessible_note_edit_class_init(AtkObject *object);
+void ags_accessible_note_edit_action_interface_init(AtkActionIface *action);
 void ags_note_edit_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_note_edit_init(AgsNoteEdit *note_edit);
 void ags_note_edit_connect(AgsConnectable *connectable);
 void ags_note_edit_disconnect(AgsConnectable *connectable);
+AtkObject* ags_note_edit_get_accessible(GtkWidget *widget);
+
+gboolean ags_accessible_note_edit_do_action(AtkAction *action,
+					    gint i);
+gint ags_accessible_note_edit_get_n_actions(AtkAction *action);
+const gchar* ags_accessible_note_edit_get_description(AtkAction *action,
+						      gint i);
+const gchar* ags_accessible_note_edit_get_name(AtkAction *action,
+					       gint i);
+const gchar* ags_accessible_note_edit_get_keybinding(AtkAction *action,
+						     gint i);
+gboolean ags_accessible_note_edit_set_description(AtkAction *action,
+						  gint i);
+gchar* ags_accessible_note_edit_get_localized_name(AtkAction *action,
+						   gint i);
 
 /**
  * SECTION:ags_note_edit
@@ -51,6 +73,8 @@ GtkStyle *note_edit_style;
 
 extern pthread_mutex_t ags_application_mutex;
 
+static GQuark quark_accessible_object = 0;
+
 GType
 ags_note_edit_get_type(void)
 {
@@ -87,6 +111,42 @@ ags_note_edit_get_type(void)
   return(ags_type_note_edit);
 }
 
+static GType
+ags_accessible_note_edit_get_type(void)
+{
+  static GType ags_type_accessible_note_edit = 0;
+
+  if(!ags_type_accessible_note_edit){
+    const GTypeInfo ags_accesssible_note_edit_info = {
+      sizeof(GtkAccessibleClass),
+      NULL,           /* base_init */
+      NULL,           /* base_finalize */
+      (GClassInitFunc) ags_accessible_note_edit_class_init,
+      NULL,           /* class_finalize */
+      NULL,           /* class_data */
+      sizeof(GtkAccessible),
+      0,             /* n_preallocs */
+      NULL, NULL
+    };
+
+    static const GInterfaceInfo atk_action_interface_info = {
+      (GInterfaceInitFunc) ags_accessible_note_edit_action_interface_init,
+      NULL, /* interface_finalize */
+      NULL, /* interface_data */
+    };
+    
+    ags_type_accessible_note_edit = g_type_register_static(GTK_TYPE_ACCESSIBLE,
+							   "AgsAccessibleNoteEdit\0", &ags_accesssible_note_edit_info,
+							   0);
+
+    g_type_add_interface_static(ags_type_accessible_note_edit,
+				ATK_TYPE_ACTION,
+				&atk_action_interface_info);
+  }
+  
+  return(ags_type_accessible_note_edit);
+}
+
 void
 ags_note_edit_connectable_interface_init(AgsConnectableInterface *connectable)
 {
@@ -99,6 +159,25 @@ ags_note_edit_connectable_interface_init(AgsConnectableInterface *connectable)
 void
 ags_note_edit_class_init(AgsNoteEditClass *note_edit)
 {
+  quark_accessible_object = g_quark_from_static_string("ags-accessible-object\0");
+}
+
+void
+ags_accessible_note_edit_class_init(AtkObject *object)
+{
+  /* empty */
+}
+
+void
+ags_accessible_note_edit_action_interface_init(AtkActionIface *action)
+{
+  action->do_action = ags_accessible_note_edit_do_action;
+  action->get_n_actions = ags_accessible_note_edit_get_n_actions;
+  action->get_description = ags_accessible_note_edit_get_description;
+  action->get_name = ags_accessible_note_edit_get_name;
+  action->get_keybinding = ags_accessible_note_edit_get_keybinding;
+  action->set_description = ags_accessible_note_edit_set_description;
+  action->get_localized_name = ags_accessible_note_edit_get_localized_name;
 }
 
 void
@@ -122,7 +201,8 @@ ags_note_edit_init(AgsNoteEdit *note_edit)
 		   0, 0);
 
   note_edit->drawing_area = (GtkDrawingArea *) gtk_drawing_area_new();
-  gtk_widget_set_style((GtkWidget *) note_edit->drawing_area, note_edit_style);
+  gtk_widget_set_style((GtkWidget *) note_edit->drawing_area,
+		       note_edit_style);
   gtk_widget_set_events(GTK_WIDGET(note_edit->drawing_area), GDK_EXPOSURE_MASK
 			| GDK_LEAVE_NOTIFY_MASK
 			| GDK_BUTTON_PRESS_MASK
@@ -131,8 +211,7 @@ ags_note_edit_init(AgsNoteEdit *note_edit)
 			| GDK_POINTER_MOTION_HINT_MASK
 			| GDK_CONTROL_MASK
 			| GDK_KEY_PRESS_MASK
-			| GDK_KEY_RELEASE_MASK
-			);
+			| GDK_KEY_RELEASE_MASK);
   gtk_widget_set_can_focus(note_edit->drawing_area,
 			   TRUE);
   
@@ -242,7 +321,7 @@ ags_note_edit_connect(AgsConnectable *connectable)
 		   G_CALLBACK(ags_note_edit_drawing_area_key_press_event), (gpointer) note_edit);
 
   g_signal_connect((GObject *) note_edit->drawing_area, "key_release_event\0",
-		     G_CALLBACK(ags_note_edit_drawing_area_key_release_event), (gpointer) note_edit);
+		   G_CALLBACK(ags_note_edit_drawing_area_key_release_event), (gpointer) note_edit);
 
   g_signal_connect_after((GObject *) note_edit->vscrollbar, "value-changed\0",
 			 G_CALLBACK(ags_note_edit_vscrollbar_value_changed), (gpointer) note_edit);
@@ -257,6 +336,316 @@ ags_note_edit_disconnect(AgsConnectable *connectable)
   //TODO:JK: implement me
 }
 
+AtkObject*
+ags_note_edit_get_accessible(GtkWidget *widget)
+{
+  AtkObject* accessible;
+
+  accessible = g_object_get_qdata(G_OBJECT(widget),
+				  quark_accessible_object);
+  
+  if(!accessible){
+    accessible = g_object_new(ags_accessible_note_edit_get_type(),
+			      NULL);
+    
+    g_object_set_qdata(G_OBJECT(widget),
+		       quark_accessible_object,
+		       accessible);
+    gtk_accessible_set_widget(accessible,
+			      widget);
+  }
+  
+  return(accessible);
+}
+
+gboolean
+ags_accessible_note_edit_do_action(AtkAction *action,
+				   gint i)
+{
+  AgsNoteEdit *note_edit;
+  
+  GdkEventKey *key_press, *key_release;
+  GdkEventKey *modifier_press, *modifier_release;
+  GdkEventKey *second_level_press, *second_level_release;
+  
+  if(!(i >= 0 && i < 13)){
+    return(FALSE);
+  }
+
+  note_edit = gtk_accessible_get_widget(ATK_OBJECT(action));
+  
+  key_press = gdk_event_new(GDK_KEY_PRESS);
+  key_release = gdk_event_new(GDK_KEY_RELEASE);
+
+  /* create modifier */
+  modifier_press = gdk_event_new(GDK_KEY_PRESS);
+  modifier_release = gdk_event_new(GDK_KEY_RELEASE);
+  
+  modifier_press->keyval =
+    modifier_release->keyval = GDK_KEY_Control_R;
+
+  /* create second level */
+  second_level_press = gdk_event_new(GDK_KEY_PRESS);
+  second_level_release = gdk_event_new(GDK_KEY_RELEASE);
+  
+  second_level_press->keyval =
+    second_level_release->keyval = GDK_KEY_Shift_R;
+
+  switch(i){
+  case 0:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Left;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 1:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Right;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 2:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Up;
+    
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 3:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Down;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 4:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_space;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 5:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Left;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, second_level_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, second_level_release);
+    }
+    break;
+  case 6:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Right;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, second_level_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, second_level_release);
+    }
+    break;
+  case 7:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Delete;
+      
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+    }
+    break;
+  case 8:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_c;
+
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, modifier_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, modifier_release);      
+    }    
+    break;
+  case 9:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_x;
+
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, modifier_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 10:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_v;
+
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, modifier_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 11:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_a;
+
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, modifier_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 12:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_i;
+
+      /* send event */
+      gtk_widget_event(note_edit->drawing_area, modifier_press);
+      gtk_widget_event(note_edit->drawing_area, key_press);
+      gtk_widget_event(note_edit->drawing_area, key_release);
+      gtk_widget_event(note_edit->drawing_area, modifier_release);      
+    }
+    break;
+  }
+
+  return(TRUE);
+}
+
+gint
+ags_accessible_note_edit_get_n_actions(AtkAction *action)
+{
+  return(13);
+}
+
+const gchar*
+ags_accessible_note_edit_get_description(AtkAction *action,
+					 gint i)
+{
+  static const gchar **actions = {
+    "move cursor left\0",
+    "move cursor right\0",
+    "move cursor up\0",
+    "move cursor down\0",
+    "add audio note\0",
+    "shrink audio note\0",
+    "grow audio note\0",
+    "remove audio note\0",
+    "copy note to clipboard\0",
+    "cut note to clipbaord\0",
+    "paste note from clipboard\0",
+    "select all note\0",
+    "invert note\0",
+  };
+
+  if(i >= 0 && i < 13){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+const gchar*
+ags_accessible_note_edit_get_name(AtkAction *action,
+				  gint i)
+{
+  static const gchar **actions = {
+    "left\0",
+    "right\0",
+    "up\0",
+    "down\0",
+    "add\0",
+    "shrink\0",
+    "grow\0",
+    "remove\0",
+    "copy\0",
+    "cut\0",
+    "paste\0",
+    "select-all\0",
+    "invert\0",
+  };
+  
+  if(i >= 0 && i < 13){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+const gchar*
+ags_accessible_note_edit_get_keybinding(AtkAction *action,
+					gint i)
+{
+  static const gchar **actions = {
+    "left\0",
+    "right\0",
+    "up\0",
+    "down\0",
+    "space",
+    "Shft+Left\0",
+    "Shft+Right\0",
+    "Del\0"
+    "Ctrl+c"
+    "Ctrl+x",
+    "Ctrl+v",
+    "Ctrl+a",
+    "Ctrl+i",
+  };
+  
+  if(i >= 0 && i < 13){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+gboolean
+ags_accessible_note_edit_set_description(AtkAction *action,
+					 gint i)
+{
+  //TODO:JK: implement me
+
+  return(FALSE);
+}
+
+gchar*
+ags_accessible_note_edit_get_localized_name(AtkAction *action,
+					    gint i)
+{
+  //TODO:JK: implement me
+
+  return(NULL);
+}
+
 /**
  * ags_note_edit_set_map_height:
  * @note_edit: the #AgsNoteEdit
@@ -368,9 +757,28 @@ ags_note_edit_reset_vertically(AgsNoteEdit *note_edit, guint flags)
 
       if((AGS_NOTE_EDIT_DRAW_FADER & (note_edit->flags)) != 0){
 	AgsCountBeatsAudioRun *count_beats_audio_run;
-	GList *recall;
-	gdouble position;
+
+	AgsMutexManager *mutex_manager;
 	
+	GList *recall;
+
+	gdouble position;
+
+	pthread_mutex_t *audio_mutex;
+
+	/* lookup audio mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+  
+	mutex_manager = ags_mutex_manager_get_instance();
+
+	audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					       (GObject *) editor->selected_machine->audio);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* retrieve position */
+	pthread_mutex_lock(audio_mutex);
+
 	recall = editor->selected_machine->audio->play;
 
 	while((recall = ags_recall_find_type(recall,
@@ -386,7 +794,12 @@ ags_note_edit_reset_vertically(AgsNoteEdit *note_edit, guint flags)
 	  count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(recall->data);
 
 	  position = count_beats_audio_run->notation_counter * note_edit->control_unit.control_width;
+	}
 	
+	pthread_mutex_unlock(audio_mutex);
+
+	/* draw fader */
+	if(recall != NULL){
 	  ags_note_edit_draw_scroll(note_edit, cr,
 				    position);
 	}
@@ -517,7 +930,7 @@ ags_note_edit_reset_horizontally(AgsNoteEdit *note_edit, guint flags)
     note_edit->control_unit.nth_x = 0;
   }
 
-    /* refresh display */
+  /* refresh display */
   if(editor->selected_machine != NULL){
     cairo_t *cr;
 
@@ -545,9 +958,28 @@ ags_note_edit_reset_horizontally(AgsNoteEdit *note_edit, guint flags)
       /* fader */
       if((AGS_NOTE_EDIT_DRAW_FADER & (note_edit->flags)) != 0){
 	AgsCountBeatsAudioRun *count_beats_audio_run;
-	GList *recall;
-	gdouble position;
+
+	AgsMutexManager *mutex_manager;
 	
+	GList *recall;
+
+	gdouble position;
+
+	pthread_mutex_t *audio_mutex;
+	
+	/* lookup audio mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+  
+	mutex_manager = ags_mutex_manager_get_instance();
+
+	audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					       (GObject *) editor->selected_machine->audio);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* retrieve position */
+	pthread_mutex_lock(audio_mutex);
+
 	recall = editor->selected_machine->audio->play;
 
 	while((recall = ags_recall_find_type(recall,
@@ -563,7 +995,12 @@ ags_note_edit_reset_horizontally(AgsNoteEdit *note_edit, guint flags)
 	  count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(recall->data);
 
 	  position = count_beats_audio_run->notation_counter * note_edit->control_unit.control_width;
+	}
 	
+	pthread_mutex_unlock(audio_mutex);
+
+	/* draw fader */
+	if(recall != NULL){
 	  ags_note_edit_draw_scroll(note_edit, cr,
 				    position);
 	}
@@ -592,22 +1029,40 @@ ags_note_edit_draw_segment(AgsNoteEdit *note_edit, cairo_t *cr)
 {
   AgsEditor *editor;
   GtkWidget *widget;
+
+  GtkStyle *note_edit_style;
+  
   double tact;
   guint i, j;
   guint j_set;
 
-  widget = (GtkWidget *) note_edit->drawing_area;
+  static const gdouble white_gc = 65535.0;
 
+  widget = (GtkWidget *) note_edit->drawing_area;
+  note_edit_style = gtk_widget_get_style(widget);
+  
   editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(note_edit),
 						 AGS_TYPE_EDITOR);
 
-  cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
-  cairo_rectangle(cr, 0.0, 0.0, (double) widget->allocation.width, (double) widget->allocation.height);
+  /* clear with background color */
+  cairo_set_source_rgb(cr,
+		       note_edit_style->bg[0].red / white_gc,
+		       note_edit_style->bg[0].green / white_gc,
+		       note_edit_style->bg[0].blue / white_gc);
+
+  cairo_rectangle(cr,
+		  0.0, 0.0,
+		  (double) widget->allocation.width, (double) widget->allocation.height);
+  
   cairo_fill(cr);
 
+  /* horizontal lines */
   cairo_set_line_width(cr, 1.0);
 
-  cairo_set_source_rgb(cr, 0.8, 0.8, 0.8);
+  cairo_set_source_rgb(cr,
+		       note_edit_style->fg[0].red / white_gc,
+		       note_edit_style->fg[0].green / white_gc,
+		       note_edit_style->fg[0].blue / white_gc);
 
   for(i = note_edit->y0 ; i < note_edit->height;){
     cairo_move_to(cr, 0.0, (double) i);
@@ -617,10 +1072,13 @@ ags_note_edit_draw_segment(AgsNoteEdit *note_edit, cairo_t *cr)
     i += note_edit->control_height;
   }
 
-  cairo_move_to(cr, 0.0, (double) i);
-  cairo_line_to(cr, (double) note_edit->width, (double) i);
+  cairo_move_to(cr,
+		0.0, (double) i);
+  cairo_line_to(cr,
+		(double) note_edit->width, (double) i);
   cairo_stroke(cr);
 
+  /* vertical lines */
   tact = exp2((double) gtk_combo_box_get_active((GtkComboBox *) editor->toolbar->zoom) - 2.0);
 
   i = note_edit->control_current.x0;
@@ -628,7 +1086,12 @@ ags_note_edit_draw_segment(AgsNoteEdit *note_edit, cairo_t *cr)
   if(i < note_edit->width &&
      tact > 1.0 ){
     j_set = note_edit->control_current.nth_x % ((guint) tact);
-    cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
+
+    /* thin lines */
+    cairo_set_source_rgb(cr,
+			 note_edit_style->mid[0].red / white_gc,
+			 note_edit_style->mid[0].green / white_gc,
+			 note_edit_style->mid[0].blue / white_gc);
 
     if(j_set != 0){
       j = j_set;
@@ -637,15 +1100,25 @@ ags_note_edit_draw_segment(AgsNoteEdit *note_edit, cairo_t *cr)
   }
 
   for(; i < note_edit->width; ){
-    cairo_set_source_rgb(cr, 1.0, 1.0, 0.0);
+    /* strong lines */
+    cairo_set_source_rgb(cr,
+			 note_edit_style->fg[0].red / white_gc,
+			 note_edit_style->fg[0].green / white_gc,
+			 note_edit_style->fg[0].blue / white_gc);
     
-    cairo_move_to(cr, (double) i, 0.0);
-    cairo_line_to(cr, (double) i, (double) note_edit->height);
+    cairo_move_to(cr,
+		  (double) i, 0.0);
+    cairo_line_to(cr,
+		  (double) i, (double) note_edit->height);
     cairo_stroke(cr);
     
     i += note_edit->control_current.control_width;
     
-    cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
+    /* thin lines */
+    cairo_set_source_rgb(cr,
+			 note_edit_style->mid[0].red / white_gc,
+			 note_edit_style->mid[0].green / white_gc,
+			 note_edit_style->mid[0].blue / white_gc);
     
     for(j = 1; i < note_edit->width && j < tact; j++){
     ags_note_edit_draw_segment0:
@@ -670,11 +1143,17 @@ ags_note_edit_draw_segment(AgsNoteEdit *note_edit, cairo_t *cr)
 void
 ags_note_edit_draw_position(AgsNoteEdit *note_edit, cairo_t *cr)
 {
+  GtkStyle *note_edit_style;
+  
   guint selected_x, selected_y;
   guint x_offset[2], y_offset[2];
   guint x, y, width, height;
   gint size_width, size_height;
 
+  static const gdouble white_gc = 65535.0;
+
+  note_edit_style = gtk_widget_get_style(GTK_WIDGET(note_edit->drawing_area));
+  
   selected_x = note_edit->selected_x * note_edit->control_unit.control_width;
   selected_y = note_edit->selected_y * note_edit->control_height;
 
@@ -703,7 +1182,7 @@ ags_note_edit_draw_position(AgsNoteEdit *note_edit, cairo_t *cr)
     if(selected_x + note_edit->control_current.control_width < x_offset[1]){
       width = note_edit->control_current.control_width;
     }else{
-      width = note_edit->control_current.control_width - (x_offset[1] - (selected_x + note_edit->control_current.control_width));
+      width = x_offset[1] - selected_x;
     }
   }
 
@@ -723,13 +1202,19 @@ ags_note_edit_draw_position(AgsNoteEdit *note_edit, cairo_t *cr)
     if(selected_y + note_edit->control_height < y_offset[1]){
       height = note_edit->control_height;
     }else{
-      height = y_offset[1] - (selected_y + note_edit->control_height);
+      height = y_offset[1] - selected_y;
     }
   }
 
   /* draw */
-  cairo_set_source_rgba(cr, 0.25, 0.5, 1.0, 0.5);
-  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
+  cairo_set_source_rgba(cr,
+			note_edit_style->base[0].red / white_gc,
+			note_edit_style->base[0].green / white_gc,
+			note_edit_style->base[0].blue / white_gc,
+			0.5);
+  cairo_rectangle(cr,
+		  (double) x, (double) y,
+		  (double) width, (double) height);
   cairo_fill(cr);
 }
 
@@ -749,6 +1234,7 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
   AgsEditor *editor;
   GtkWidget *widget;
 
+  GtkStyle *note_edit_style;
   AgsNote *note;
 
   AgsMutexManager *mutex_manager;
@@ -763,9 +1249,21 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
   pthread_mutex_t *audio_mutex;
 
+  static const gdouble white_gc = 65535.0;
+
+  note_edit_style = gtk_widget_get_style(GTK_WIDGET(note_edit->drawing_area));
+  
   editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(note_edit),
 						 AGS_TYPE_EDITOR);
 
+  if(editor->selected_machine == NULL ||
+     (machine = editor->selected_machine) == NULL){
+    return;
+  }
+
+  widget = (GtkWidget *) note_edit->drawing_area;
+
+  /* lookup audio mutex */
   pthread_mutex_lock(&(ags_application_mutex));
   
   mutex_manager = ags_mutex_manager_get_instance();
@@ -775,14 +1273,13 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
   
   pthread_mutex_unlock(&(ags_application_mutex));
 
-  if(editor->selected_machine == NULL ||
-     (machine = editor->selected_machine) == NULL ||
-     machine->audio->notation == NULL)
-    return;
+  /* draw */
+  pthread_mutex_lock(audio_mutex);
 
-  widget = (GtkWidget *) note_edit->drawing_area;
-
-  cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+  cairo_set_source_rgb(cr,
+		       note_edit_style->fg[0].red / white_gc,
+		       note_edit_style->fg[0].green / white_gc,
+		       note_edit_style->fg[0].blue / white_gc);
 
   i = 0;
 
@@ -822,12 +1319,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	    cairo_set_source_rgba(cr,
+				  note_edit_style->light[0].red / white_gc,
+				  note_edit_style->light[0].green / white_gc,
+				  note_edit_style->light[0].blue / white_gc,
+				  0.7);
 
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	    cairo_set_source_rgb(cr,
+				 note_edit_style->fg[0].red / white_gc,
+				 note_edit_style->fg[0].green / white_gc,
+				 note_edit_style->fg[0].blue / white_gc);	    
 	  }
 	}else if(note->y == (note_edit->nth_y - 1) && note_edit->y0 != 0){
 	  if(note_edit->y0 > note_edit->control_margin_y){
@@ -851,12 +1355,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
 	    /* check if note is selected */
 	    if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	      cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	      cairo_set_source_rgba(cr,
+				    note_edit_style->light[0].red / white_gc,
+				    note_edit_style->light[0].green / white_gc,
+				    note_edit_style->light[0].blue / white_gc,
+				    0.7);
 	    
 	      cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	      cairo_stroke(cr);
 	    
-	      cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	      cairo_set_source_rgb(cr,
+				   note_edit_style->fg[0].red / white_gc,
+				   note_edit_style->fg[0].green / white_gc,
+				   note_edit_style->fg[0].blue / white_gc);	    
 	    }
 	  }
 	}else if(note->y == (note_edit->stop_y + 1) && note_edit->y1 != 0){
@@ -881,12 +1392,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 	  
 	    /* check if note is selected */
 	    if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	      cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	      cairo_set_source_rgba(cr,
+				    note_edit_style->light[0].red / white_gc,
+				    note_edit_style->light[0].green / white_gc,
+				    note_edit_style->light[0].blue / white_gc,
+				    0.7);
 	    
 	      cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	      cairo_stroke(cr);
 	    
-	      cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	      cairo_set_source_rgb(cr,
+				   note_edit_style->fg[0].red / white_gc,
+				   note_edit_style->fg[0].green / white_gc,
+				   note_edit_style->fg[0].blue / white_gc);	    
 	    }
 	  }
 	}
@@ -917,12 +1435,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
 	/* check if note is selected */
 	if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	  cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	  cairo_set_source_rgba(cr,
+				note_edit_style->light[0].red / white_gc,
+				note_edit_style->light[0].green / white_gc,
+				note_edit_style->light[0].blue / white_gc,
+				0.7);
 	
 	  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	  cairo_stroke(cr);
 	
-	  cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	  cairo_set_source_rgb(cr,
+			       note_edit_style->fg[0].red / white_gc,
+			       note_edit_style->fg[0].green / white_gc,
+			       note_edit_style->fg[0].blue / white_gc);	    
 	}
       }else if(note->y == (note_edit->nth_y - 1) && note_edit->y0 != 0){
 	if(note_edit->y0 > note_edit->control_margin_y){
@@ -946,12 +1471,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	    cairo_set_source_rgba(cr,
+				  note_edit_style->light[0].red / white_gc,
+				  note_edit_style->light[0].green / white_gc,
+				  note_edit_style->light[0].blue / white_gc,
+				  0.7);
 
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	    cairo_set_source_rgb(cr,
+				 note_edit_style->fg[0].red / white_gc,
+				 note_edit_style->fg[0].green / white_gc,
+				 note_edit_style->fg[0].blue / white_gc);	    
 	  }
 	}
       }else if(note->y == (note_edit->stop_y + 1) && note_edit->y1 != 0){
@@ -976,12 +1508,19 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	    cairo_set_source_rgba(cr,
+				  note_edit_style->light[0].red / white_gc,
+				  note_edit_style->light[0].green / white_gc,
+				  note_edit_style->light[0].blue / white_gc,
+				  0.7);
 
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
+	    cairo_set_source_rgb(cr,
+				 note_edit_style->fg[0].red / white_gc,
+				 note_edit_style->fg[0].green / white_gc,
+				 note_edit_style->fg[0].blue / white_gc);	    
 	  }
 	}
       }
@@ -991,6 +1530,8 @@ ags_note_edit_draw_notation(AgsNoteEdit *note_edit, cairo_t *cr)
 
     i++;
   }
+
+  pthread_mutex_unlock(audio_mutex);
 }
 
 /**
@@ -1007,9 +1548,15 @@ void
 ags_note_edit_draw_scroll(AgsNoteEdit *note_edit, cairo_t *cr,
 			  gdouble position)
 {
+  GtkStyle *note_edit_style;
+  
   double x, y;
   double width, height;
 
+  static const gdouble white_gc = 65535.0;
+
+  note_edit_style = gtk_widget_get_style(GTK_WIDGET(note_edit->drawing_area));
+  
   y = 0.0;
   x = (position) - (GTK_RANGE(note_edit->hscrollbar)->adjustment->value);
 
@@ -1017,8 +1564,14 @@ ags_note_edit_draw_scroll(AgsNoteEdit *note_edit, cairo_t *cr,
   width = 3.0;
 
   /* draw */
-  cairo_set_source_rgba(cr, 0.79, 0.0, 1.0, 0.5);
-  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
+  cairo_set_source_rgba(cr,
+			note_edit_style->dark[0].red / white_gc,
+			note_edit_style->dark[0].green / white_gc,
+			note_edit_style->dark[0].blue / white_gc,
+			0.5);
+  cairo_rectangle(cr,
+		  (double) x, (double) y,
+		  (double) width, (double) height);
   cairo_fill(cr);
 }
 
