@@ -31,11 +31,33 @@
 
 #include <ags/X/editor/ags_pattern_edit.h>
 
+#include <gdk/gdkkeysyms.h>
+
+#include <atk/atk.h>
+
+static GType ags_accessible_pattern_edit_get_type(void);
 void ags_pattern_edit_class_init(AgsPatternEditClass *pattern_edit);
+void ags_accessible_pattern_edit_class_init(AtkObject *object);
+void ags_accessible_pattern_edit_action_interface_init(AtkActionIface *action);
 void ags_pattern_edit_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_pattern_edit_init(AgsPatternEdit *pattern_edit);
 void ags_pattern_edit_connect(AgsConnectable *connectable);
 void ags_pattern_edit_disconnect(AgsConnectable *connectable);
+AtkObject* ags_pattern_edit_get_accessible(GtkWidget *widget);
+
+gboolean ags_accessible_pattern_edit_do_action(AtkAction *action,
+					       gint i);
+gint ags_accessible_pattern_edit_get_n_actions(AtkAction *action);
+const gchar* ags_accessible_pattern_edit_get_description(AtkAction *action,
+							 gint i);
+const gchar* ags_accessible_pattern_edit_get_name(AtkAction *action,
+						  gint i);
+const gchar* ags_accessible_pattern_edit_get_keybinding(AtkAction *action,
+							gint i);
+gboolean ags_accessible_pattern_edit_set_description(AtkAction *action,
+						     gint i);
+gchar* ags_accessible_pattern_edit_get_localized_name(AtkAction *action,
+						      gint i);
 
 void ags_pattern_edit_paint(AgsPatternEdit *pattern_edit);
 
@@ -53,6 +75,8 @@ GtkStyle *pattern_edit_style;
 
 extern pthread_mutex_t ags_application_mutex;
 
+static GQuark quark_accessible_object = 0;
+
 GType
 ags_pattern_edit_get_type(void)
 {
@@ -89,6 +113,42 @@ ags_pattern_edit_get_type(void)
   return(ags_type_pattern_edit);
 }
 
+static GType
+ags_accessible_pattern_edit_get_type(void)
+{
+  static GType ags_type_accessible_pattern_edit = 0;
+
+  if(!ags_type_accessible_pattern_edit){
+    const GTypeInfo ags_accesssible_pattern_edit_info = {
+      sizeof(GtkAccessibleClass),
+      NULL,           /* base_init */
+      NULL,           /* base_finalize */
+      (GClassInitFunc) ags_accessible_pattern_edit_class_init,
+      NULL,           /* class_finalize */
+      NULL,           /* class_data */
+      sizeof(GtkAccessible),
+      0,             /* n_preallocs */
+      NULL, NULL
+    };
+
+    static const GInterfaceInfo atk_action_interface_info = {
+      (GInterfaceInitFunc) ags_accessible_pattern_edit_action_interface_init,
+      NULL, /* interface_finalize */
+      NULL, /* interface_data */
+    };
+    
+    ags_type_accessible_pattern_edit = g_type_register_static(GTK_TYPE_ACCESSIBLE,
+							      "AgsAccessiblePatternEdit\0", &ags_accesssible_pattern_edit_info,
+							      0);
+
+    g_type_add_interface_static(ags_type_accessible_pattern_edit,
+				ATK_TYPE_ACTION,
+				&atk_action_interface_info);
+  }
+  
+  return(ags_type_accessible_pattern_edit);
+}
+
 void
 ags_pattern_edit_connectable_interface_init(AgsConnectableInterface *connectable)
 {
@@ -101,6 +161,25 @@ ags_pattern_edit_connectable_interface_init(AgsConnectableInterface *connectable
 void
 ags_pattern_edit_class_init(AgsPatternEditClass *pattern_edit)
 {
+  quark_accessible_object = g_quark_from_static_string("ags-accessible-object\0");  
+}
+
+void
+ags_accessible_pattern_edit_class_init(AtkObject *object)
+{
+  /* empty */
+}
+
+void
+ags_accessible_pattern_edit_action_interface_init(AtkActionIface *action)
+{
+  action->do_action = ags_accessible_pattern_edit_do_action;
+  action->get_n_actions = ags_accessible_pattern_edit_get_n_actions;
+  action->get_description = ags_accessible_pattern_edit_get_description;
+  action->get_name = ags_accessible_pattern_edit_get_name;
+  action->get_keybinding = ags_accessible_pattern_edit_get_keybinding;
+  action->set_description = ags_accessible_pattern_edit_set_description;
+  action->get_localized_name = ags_accessible_pattern_edit_get_localized_name;
 }
 
 void
@@ -251,6 +330,278 @@ ags_pattern_edit_disconnect(AgsConnectable *connectable)
   //TODO:JK: implement me
 }
 
+AtkObject*
+ags_pattern_edit_get_accessible(GtkWidget *widget)
+{
+  AtkObject* accessible;
+
+  accessible = g_object_get_qdata(G_OBJECT(widget),
+				    quark_accessible_object);
+  
+  if(!accessible){
+    accessible = g_object_new(ags_accessible_pattern_edit_get_type(),
+			      NULL);
+    
+    g_object_set_qdata(G_OBJECT(widget),
+		       quark_accessible_object,
+		       accessible);
+    gtk_accessible_set_widget(accessible,
+			      widget);
+  }
+  
+  return(accessible);
+}
+
+gboolean
+ags_accessible_pattern_edit_do_action(AtkAction *action,
+				      gint i)
+{
+  AgsPatternEdit *pattern_edit;
+  
+  GdkEventKey *key_press, *key_release;
+  GdkEventKey *modifier_press, *modifier_release;
+  
+  if(!(i >= 0 && i < 11)){
+    return(FALSE);
+  }
+
+  pattern_edit = gtk_accessible_get_widget(ATK_OBJECT(action));
+  
+  key_press = gdk_event_new(GDK_KEY_PRESS);
+  key_release = gdk_event_new(GDK_KEY_RELEASE);
+
+  /* create modifier */
+  modifier_press = gdk_event_new(GDK_KEY_PRESS);
+  modifier_release = gdk_event_new(GDK_KEY_RELEASE);
+  
+  modifier_press->keyval =
+    modifier_release->keyval = GDK_KEY_Control_R;
+
+  switch(i){
+  case 0:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Left;
+      
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 1:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Right;
+      
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 2:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Up;
+    
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 3:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Down;
+      
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 4:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_space;
+      
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 5:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_Delete;
+      
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+    }
+    break;
+  case 6:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_c;
+
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, modifier_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+      gtk_widget_event(pattern_edit->drawing_area, modifier_release);      
+    }    
+    break;
+  case 7:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_x;
+
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, modifier_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+      gtk_widget_event(pattern_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 8:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_v;
+
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, modifier_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+      gtk_widget_event(pattern_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 9:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_a;
+
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, modifier_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+      gtk_widget_event(pattern_edit->drawing_area, modifier_release);      
+    }
+    break;
+  case 10:
+    {
+      key_press->keyval =
+	key_release->keyval = GDK_KEY_i;
+
+      /* send event */
+      gtk_widget_event(pattern_edit->drawing_area, modifier_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_press);
+      gtk_widget_event(pattern_edit->drawing_area, key_release);
+      gtk_widget_event(pattern_edit->drawing_area, modifier_release);      
+    }
+    break;
+  }
+
+  return(TRUE);
+}
+
+gint
+ags_accessible_pattern_edit_get_n_actions(AtkAction *action)
+{
+  return(11);
+}
+
+const gchar*
+ags_accessible_pattern_edit_get_description(AtkAction *action,
+					    gint i)
+{
+  static const gchar **actions = {
+    "move cursor left\0",
+    "move cursor right\0",
+    "move cursor up\0",
+    "move cursor down\0",
+    "add audio pattern\0",
+    "remove audio pattern\0"
+    "copy pattern to clipboard\0",
+    "cut pattern to clipbaord\0",
+    "paste pattern from clipboard\0",
+    "select all pattern\0",
+    "invert pattern\0",
+  };
+
+  if(i >= 0 && i < 11){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+const gchar*
+ags_accessible_pattern_edit_get_name(AtkAction *action,
+				     gint i)
+{
+  static const gchar **actions = {
+    "left\0",
+    "right\0",
+    "up\0",
+    "down\0",
+    "add\0",
+    "remove\0",
+    "copy\0",
+    "cut\0",
+    "paste\0",
+    "select-all\0",
+    "invert\0",
+  };
+  
+  if(i >= 0 && i < 11){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+const gchar*
+ags_accessible_pattern_edit_get_keybinding(AtkAction *action,
+					   gint i)
+{
+  static const gchar **actions = {
+    "left\0",
+    "right\0",
+    "up\0",
+    "down\0",
+    "space",
+    "Del\0",
+    "Ctrl+c"
+    "Ctrl+x",
+    "Ctrl+v",
+    "Ctrl+a",
+    "Ctrl+i",
+  };
+  
+  if(i >= 0 && i < 11){
+    return(actions[i]);
+  }else{
+    return(NULL);
+  }
+}
+
+gboolean
+ags_accessible_pattern_edit_set_description(AtkAction *action,
+					    gint i)
+{
+  //TODO:JK: implement me
+
+  return(FALSE);
+}
+
+gchar*
+ags_accessible_pattern_edit_get_localized_name(AtkAction *action,
+					       gint i)
+{
+  //TODO:JK: implement me
+
+  return(NULL);
+}
+
 /**
  * ags_pattern_edit_set_map_height:
  * @pattern_edit: the #AgsPatternEdit
@@ -348,6 +699,57 @@ ags_pattern_edit_reset_vertically(AgsPatternEdit *pattern_edit, guint flags)
 	ags_pattern_edit_draw_position(pattern_edit, cr);
       }
 
+      /* fader */
+      if((AGS_PATTERN_EDIT_DRAW_FADER & (pattern_edit->flags)) != 0){
+	AgsCountBeatsAudioRun *count_beats_audio_run;
+
+	AgsMutexManager *mutex_manager;
+	
+	GList *recall;
+	
+	gdouble position;
+
+	pthread_mutex_t *audio_mutex;
+	
+	/* lookup audio mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+  
+	mutex_manager = ags_mutex_manager_get_instance();
+
+	audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					       (GObject *) editor->selected_machine->audio);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* retrieve position */
+	pthread_mutex_lock(audio_mutex);
+
+	recall = editor->selected_machine->audio->play;
+
+	while((recall = ags_recall_find_type(recall,
+					     AGS_TYPE_COUNT_BEATS_AUDIO_RUN)) != NULL){
+	  if(AGS_RECALL(recall->data)->recall_id != NULL && (AGS_RECALL_NOTATION & (AGS_RECALL(recall->data)->recall_id->flags)) != 0){
+	    break;
+	  }
+
+	  recall = recall->next;
+	}
+
+	if(recall != NULL){
+	  count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(recall->data);
+
+	  position = count_beats_audio_run->notation_counter * pattern_edit->control_unit.control_width;
+	}
+	
+	pthread_mutex_unlock(audio_mutex);
+
+	/* draw fader */
+	if(recall != NULL){
+	  ags_pattern_edit_draw_scroll(pattern_edit, cr,
+				       position);
+	}
+      }
+
       cairo_pop_group_to_source(cr);
       cairo_paint(cr);
 
@@ -497,18 +899,31 @@ ags_pattern_edit_reset_horizontally(AgsPatternEdit *pattern_edit, guint flags)
 	ags_pattern_edit_draw_position(pattern_edit, cr);
       }
 
-      //TODO:JK: implement me
-      //      position = gtk_range_get_value(GTK_RANGE(pattern_edit->hscrollbar));
-      //      position -= floor(position / pattern_edit->control_current.control_width);
-      //      ags_pattern_edit_draw_scroll(pattern_edit, cr,
-      //				position);
-
       /* fader */
       if((AGS_PATTERN_EDIT_DRAW_FADER & (pattern_edit->flags)) != 0){
 	AgsCountBeatsAudioRun *count_beats_audio_run;
-	GList *recall;
-	gdouble position;
+
+	AgsMutexManager *mutex_manager;
 	
+	GList *recall;
+	
+	gdouble position;
+
+	pthread_mutex_t *audio_mutex;
+	
+	/* lookup audio mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+  
+	mutex_manager = ags_mutex_manager_get_instance();
+
+	audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					       (GObject *) editor->selected_machine->audio);
+  
+	pthread_mutex_unlock(&(ags_application_mutex));
+
+	/* retrieve position */
+	pthread_mutex_lock(audio_mutex);
+
 	recall = editor->selected_machine->audio->play;
 
 	while((recall = ags_recall_find_type(recall,
@@ -524,7 +939,12 @@ ags_pattern_edit_reset_horizontally(AgsPatternEdit *pattern_edit, guint flags)
 	  count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(recall->data);
 
 	  position = count_beats_audio_run->notation_counter * pattern_edit->control_unit.control_width;
+	}
 	
+	pthread_mutex_unlock(audio_mutex);
+
+	/* draw fader */
+	if(recall != NULL){
 	  ags_pattern_edit_draw_scroll(pattern_edit, cr,
 				       position);
 	}
@@ -553,22 +973,40 @@ ags_pattern_edit_draw_segment(AgsPatternEdit *pattern_edit, cairo_t *cr)
 {
   AgsEditor *editor;
   GtkWidget *widget;
+
+  GtkStyle *pattern_edit_style;
+  
   double tact;
   guint i, j;
   guint j_set;
 
+  static const gdouble white_gc = 65535.0;
+
+  pattern_edit_style = gtk_widget_get_style(GTK_WIDGET(pattern_edit->drawing_area));
+  
   widget = (GtkWidget *) pattern_edit->drawing_area;
 
   editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(pattern_edit),
 						 AGS_TYPE_EDITOR);
 
-  cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
-  cairo_rectangle(cr, 0.0, 0.0, (double) widget->allocation.width, (double) widget->allocation.height);
+  /* clear with background color */
+  cairo_set_source_rgb(cr,
+		       pattern_edit_style->bg[0].red / white_gc,
+		       pattern_edit_style->bg[0].green / white_gc,
+		       pattern_edit_style->bg[0].blue / white_gc);
+  
+  cairo_rectangle(cr,
+		  0.0, 0.0,
+		  (double) widget->allocation.width, (double) widget->allocation.height);
   cairo_fill(cr);
 
+  /* horizontal lines */
   cairo_set_line_width(cr, 1.0);
 
-  cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+  cairo_set_source_rgb(cr,
+		       pattern_edit_style->fg[0].red / white_gc,
+		       pattern_edit_style->fg[0].green / white_gc,
+		       pattern_edit_style->fg[0].blue / white_gc);
 
   for(i = pattern_edit->y0 ; i < pattern_edit->height;){
     cairo_move_to(cr, 0.0, (double) i);
@@ -578,10 +1016,13 @@ ags_pattern_edit_draw_segment(AgsPatternEdit *pattern_edit, cairo_t *cr)
     i += pattern_edit->control_height;
   }
 
-  cairo_move_to(cr, 0.0, (double) i);
-  cairo_line_to(cr, (double) pattern_edit->width, (double) i);
+  cairo_move_to(cr,
+		0.0, (double) i);
+  cairo_line_to(cr,
+		(double) pattern_edit->width, (double) i);
   cairo_stroke(cr);
 
+  /* vertical lines */
   tact = exp2((double) gtk_combo_box_get_active(editor->toolbar->zoom) - 2.0);
 
   i = pattern_edit->control_current.x0;
@@ -589,7 +1030,12 @@ ags_pattern_edit_draw_segment(AgsPatternEdit *pattern_edit, cairo_t *cr)
   if(i < pattern_edit->width &&
      tact > 1.0 ){
     j_set = pattern_edit->control_current.nth_x % ((guint) tact);
-    cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
+
+    /* thin lines */
+    cairo_set_source_rgb(cr,
+		       pattern_edit_style->mid[0].red / white_gc,
+		       pattern_edit_style->mid[0].green / white_gc,
+		       pattern_edit_style->mid[0].blue / white_gc);
 
     if(j_set != 0){
       j = j_set;
@@ -598,20 +1044,32 @@ ags_pattern_edit_draw_segment(AgsPatternEdit *pattern_edit, cairo_t *cr)
   }
 
   for(; i < pattern_edit->width; ){
-    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+    /* strong lines */
+    cairo_set_source_rgb(cr,
+		       pattern_edit_style->fg[0].red / white_gc,
+		       pattern_edit_style->fg[0].green / white_gc,
+		       pattern_edit_style->fg[0].blue / white_gc);
     
-    cairo_move_to(cr, (double) i, 0.0);
-    cairo_line_to(cr, (double) i, (double) pattern_edit->height);
+    cairo_move_to(cr,
+		  (double) i, 0.0);
+    cairo_line_to(cr,
+		  (double) i, (double) pattern_edit->height);
     cairo_stroke(cr);
     
     i += pattern_edit->control_current.control_width;
     
-    cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
+    /* thin lines */
+    cairo_set_source_rgb(cr,
+		       pattern_edit_style->mid[0].red / white_gc,
+		       pattern_edit_style->mid[0].green / white_gc,
+		       pattern_edit_style->mid[0].blue / white_gc);
     
     for(j = 1; i < pattern_edit->width && j < tact; j++){
     ags_pattern_edit_draw_segment0:
-      cairo_move_to(cr, (double) i, 0.0);
-      cairo_line_to(cr, (double) i, (double) pattern_edit->height);
+      cairo_move_to(cr,
+		    (double) i, 0.0);
+      cairo_line_to(cr,
+		    (double) i, (double) pattern_edit->height);
       cairo_stroke(cr);
       
       i += pattern_edit->control_current.control_width;
@@ -631,11 +1089,17 @@ ags_pattern_edit_draw_segment(AgsPatternEdit *pattern_edit, cairo_t *cr)
 void
 ags_pattern_edit_draw_position(AgsPatternEdit *pattern_edit, cairo_t *cr)
 {
+  GtkStyle *pattern_edit_style;
+  
   guint selected_x, selected_y;
   guint x_offset[2], y_offset[2];
   guint x, y, width, height;
   gint size_width, size_height;
 
+  static const gdouble white_gc = 65535.0;
+
+  pattern_edit_style = gtk_widget_get_style(GTK_WIDGET(pattern_edit->drawing_area));
+  
   selected_x = pattern_edit->selected_x * pattern_edit->control_unit.control_width;
   selected_y = pattern_edit->selected_y * pattern_edit->control_height;
 
@@ -664,7 +1128,7 @@ ags_pattern_edit_draw_position(AgsPatternEdit *pattern_edit, cairo_t *cr)
     if(selected_x + pattern_edit->control_current.control_width < x_offset[1]){
       width = pattern_edit->control_current.control_width;
     }else{
-      width = pattern_edit->control_current.control_width - (x_offset[1] - (selected_x + pattern_edit->control_current.control_width));
+      width = x_offset[1] - selected_x;
     }
   }
 
@@ -684,13 +1148,20 @@ ags_pattern_edit_draw_position(AgsPatternEdit *pattern_edit, cairo_t *cr)
     if(selected_y + pattern_edit->control_height < y_offset[1]){
       height = pattern_edit->control_height;
     }else{
-      height = y_offset[1] - (selected_y + pattern_edit->control_height);
+      height = y_offset[1] - selected_y;
     }
   }
 
   /* draw */
-  cairo_set_source_rgba(cr, 0.25, 0.5, 1.0, 0.5);
-  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
+  cairo_set_source_rgba(cr,
+			pattern_edit_style->base[0].red / white_gc,
+			pattern_edit_style->base[0].green / white_gc,
+			pattern_edit_style->base[0].blue / white_gc,
+			0.5);
+  
+  cairo_rectangle(cr,
+		  (double) x, (double) y,
+		  (double) width, (double) height);
   cairo_fill(cr);
 }
 
@@ -710,6 +1181,7 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
   AgsEditor *editor;
   GtkWidget *widget;
 
+  GtkStyle *pattern_edit_style;
   AgsNote *note;
 
   AgsMutexManager *mutex_manager;
@@ -724,10 +1196,21 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
   pthread_mutex_t *audio_mutex;
 
-
+  static const gdouble white_gc = 65535.0;
+  
+  pattern_edit_style = gtk_widget_get_style(GTK_WIDGET(pattern_edit->drawing_area));
+  
   editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(pattern_edit),
 						 AGS_TYPE_EDITOR);
+  
+  if(editor->selected_machine == NULL ||
+     (machine = editor->selected_machine) == NULL){
+    return;
+  }
 
+  widget = (GtkWidget *) pattern_edit->drawing_area;
+
+  /* lookup audio mutex */
   pthread_mutex_lock(&(ags_application_mutex));
   
   mutex_manager = ags_mutex_manager_get_instance();
@@ -737,19 +1220,13 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
   
   pthread_mutex_unlock(&(ags_application_mutex));
 
+  /* draw */
   pthread_mutex_lock(audio_mutex);
-  
-  if(editor->selected_machine == NULL ||
-     (machine = editor->selected_machine) == NULL ||
-     machine->audio->notation == NULL){
-    pthread_mutex_unlock(audio_mutex);
-    
-    return;
-  }
 
-  widget = (GtkWidget *) pattern_edit->drawing_area;
-
-  cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+  cairo_set_source_rgb(cr,
+		       pattern_edit_style->fg[0].red / white_gc,
+		       pattern_edit_style->fg[0].green / white_gc,
+		       pattern_edit_style->fg[0].blue / white_gc);
 
   i = 0;
 
@@ -789,12 +1266,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
-
+	    cairo_set_source_rgba(cr,
+				  pattern_edit_style->light[0].red / white_gc,
+				  pattern_edit_style->light[0].green / white_gc,
+				  pattern_edit_style->light[0].blue / white_gc,
+				  0.7);
+	    
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	    cairo_set_source_rgb(cr,
+				 pattern_edit_style->fg[0].red / white_gc,
+				 pattern_edit_style->fg[0].green / white_gc,
+				 pattern_edit_style->fg[0].blue / white_gc);	    
 	  }
 	}else if(note->y == (pattern_edit->nth_y - 1) && pattern_edit->y0 != 0){
 	  if(pattern_edit->y0 > pattern_edit->control_margin_y){
@@ -818,12 +1302,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
 	    /* check if note is selected */
 	    if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	      cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	      cairo_set_source_rgba(cr,
+				    pattern_edit_style->light[0].red / white_gc,
+				    pattern_edit_style->light[0].green / white_gc,
+				    pattern_edit_style->light[0].blue / white_gc,
+				    0.7);
 	    
 	      cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	      cairo_stroke(cr);
 	    
-	      cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	      cairo_set_source_rgb(cr,
+				   pattern_edit_style->fg[0].red / white_gc,
+				   pattern_edit_style->fg[0].green / white_gc,
+				   pattern_edit_style->fg[0].blue / white_gc);
 	    }
 	  }
 	}else if(note->y == (pattern_edit->stop_y + 1) && pattern_edit->y1 != 0){
@@ -848,12 +1339,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 	  
 	    /* check if note is selected */
 	    if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	      cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	      cairo_set_source_rgba(cr,
+				    pattern_edit_style->light[0].red / white_gc,
+				    pattern_edit_style->light[0].green / white_gc,
+				    pattern_edit_style->light[0].blue / white_gc,
+				    0.7);
 	    
 	      cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	      cairo_stroke(cr);
 	    
-	      cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	      cairo_set_source_rgb(cr,
+				   pattern_edit_style->fg[0].red / white_gc,
+				   pattern_edit_style->fg[0].green / white_gc,
+				   pattern_edit_style->fg[0].blue / white_gc);
 	    }
 	  }
 	}
@@ -884,12 +1382,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
 	/* check if note is selected */
 	if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	  cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	  cairo_set_source_rgba(cr,
+				pattern_edit_style->light[0].red / white_gc,
+				pattern_edit_style->light[0].green / white_gc,
+				pattern_edit_style->light[0].blue / white_gc,
+				0.7);
 	
 	  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	  cairo_stroke(cr);
 	
-	  cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	  cairo_set_source_rgb(cr,
+			       pattern_edit_style->fg[0].red / white_gc,
+			       pattern_edit_style->fg[0].green / white_gc,
+			       pattern_edit_style->fg[0].blue / white_gc);
 	}
       }else if(note->y == (pattern_edit->nth_y - 1) && pattern_edit->y0 != 0){
 	if(pattern_edit->y0 > pattern_edit->control_margin_y){
@@ -913,12 +1418,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	    cairo_set_source_rgba(cr,
+				  pattern_edit_style->light[0].red / white_gc,
+				  pattern_edit_style->light[0].green / white_gc,
+				  pattern_edit_style->light[0].blue / white_gc,
+				  0.7);
 
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	    cairo_set_source_rgb(cr,
+				 pattern_edit_style->fg[0].red / white_gc,
+				 pattern_edit_style->fg[0].green / white_gc,
+				 pattern_edit_style->fg[0].blue / white_gc);
 	  }
 	}
       }else if(note->y == (pattern_edit->stop_y + 1) && pattern_edit->y1 != 0){
@@ -943,12 +1455,19 @@ ags_pattern_edit_draw_notation(AgsPatternEdit *pattern_edit, cairo_t *cr)
 
 	  /* check if note is selected */
 	  if((AGS_NOTE_IS_SELECTED & (note->flags)) != 0){
-	    cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.7);
+	    cairo_set_source_rgba(cr,
+				  pattern_edit_style->light[0].red / white_gc,
+				  pattern_edit_style->light[0].green / white_gc,
+				  pattern_edit_style->light[0].blue / white_gc,
+				  0.7);
 
 	    cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
 	    cairo_stroke(cr);
 
-	    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+	    cairo_set_source_rgb(cr,
+				 pattern_edit_style->fg[0].red / white_gc,
+				 pattern_edit_style->fg[0].green / white_gc,
+				 pattern_edit_style->fg[0].blue / white_gc);
 	  }
 	}
       }
@@ -976,9 +1495,15 @@ void
 ags_pattern_edit_draw_scroll(AgsPatternEdit *pattern_edit, cairo_t *cr,
 			     gdouble position)
 {
+  GtkStyle *pattern_edit_style;
+  
   double x, y;
   double width, height;
 
+  static const gdouble white_gc = 65535.0;
+
+  pattern_edit_style = gtk_widget_get_style(GTK_WIDGET(pattern_edit->drawing_area));
+  
   y = 0.0;
   x = (position) - (GTK_RANGE(pattern_edit->hscrollbar)->adjustment->value);
 
@@ -986,8 +1511,14 @@ ags_pattern_edit_draw_scroll(AgsPatternEdit *pattern_edit, cairo_t *cr,
   width = 3.0;
 
   /* draw */
-  cairo_set_source_rgba(cr, 0.79, 0.0, 1.0, 0.5);
-  cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
+  cairo_set_source_rgba(cr,
+			pattern_edit_style->dark[0].red / white_gc,
+			pattern_edit_style->dark[0].green / white_gc,
+			pattern_edit_style->dark[0].blue / white_gc,
+			0.5);
+  cairo_rectangle(cr,
+		  (double) x, (double) y,
+		  (double) width, (double) height);
   cairo_fill(cr);
 }
 
