@@ -19,13 +19,26 @@
 
 #include <ags/X/editor/ags_pattern_edit_callbacks.h>
 
-#include <ags/thread/ags_mutex_manager.h>
+#include <ags/main.h>
 
+#include <ags/thread/ags_mutex_manager.h>
+#include <ags/thread/ags_audio_loop.h>
+#include <ags/thread/ags_task_thread.h>
+
+#include <ags/audio/ags_devout.h>
 #include <ags/audio/ags_audio.h>
 #include <ags/audio/ags_channel.h>
 #include <ags/audio/ags_output.h>
 #include <ags/audio/ags_input.h>
 
+#include <ags/audio/recall/ags_play_channel_run.h>
+
+#include <ags/audio/task/ags_start_devout.h>
+#include <ags/audio/task/ags_init_channel.h>
+#include <ags/audio/task/ags_append_channel.h>
+#include <ags/audio/task/ags_append_recall.h>
+#include <ags/audio/task/ags_add_audio_signal.h>
+
 #include <ags/X/ags_editor.h>
 
 #include <ags/X/machine/ags_panel.h>
@@ -37,6 +50,8 @@
 
 #include <gdk/gdkkeysyms.h>
 
+void ags_pattern_edit_init_channel_launch_callback(AgsTask *task, gpointer data);
+
 extern pthread_mutex_t ags_application_mutex;
 
 void
@@ -226,7 +241,7 @@ ags_pattern_edit_drawing_area_configure_event(GtkWidget *widget, GdkEventConfigu
 }
 
 gboolean
-ags_pattern_edit_drawing_area_button_press_event (GtkWidget *widget, GdkEventButton *event, AgsPatternEdit *pattern_edit)
+ags_pattern_edit_drawing_area_button_press_event(GtkWidget *widget, GdkEventButton *event, AgsPatternEdit *pattern_edit)
 {
   AgsMachine *machine;
   AgsEditor *editor;
@@ -321,42 +336,78 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
 {
   AgsMachine *machine;
   AgsEditor *editor;
+
+  GtkStyle *pattern_edit_style;
   AgsNote *note, *note0;
+  
   double tact;
   
+  static const gdouble white_gc = 65535.0;
+
   auto void ags_pattern_edit_drawing_area_button_release_event_set_control();
   auto void ags_pattern_edit_drawing_area_button_release_event_draw_control(cairo_t *cr);
   auto void ags_pattern_edit_drawing_area_button_release_event_delete_point();
   auto void ags_pattern_edit_drawing_area_button_release_event_select_region();
 
   void ags_pattern_edit_drawing_area_button_release_event_set_control(){
+    AgsAudio *audio;
+    
+    AgsMutexManager *mutex_manager;
+
     GList *list_notation;
+    
     gint i;
     guint note_x, note_y;
     guint note_offset_x1;
     gint history;
 
-    if(pattern_edit->control.x0 >= pattern_edit->map_width)
-      pattern_edit->control.x0 = pattern_edit->map_width - 1;
+    pthread_mutex_t *audio_mutex;
 
+    audio = machine->audio;
+    
+    /* lookup audio mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    mutex_manager = ags_mutex_manager_get_instance();
+    
+    audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) audio);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get offset x1 */
+    if(pattern_edit->control.x0 >= pattern_edit->map_width){
+      pattern_edit->control.x0 = pattern_edit->map_width - 1;
+    }
+    
     note_offset_x1 = (guint) (ceil((double) (pattern_edit->control.x1_offset)  / (double) (pattern_edit->control_current.control_width)));
 
-    if(pattern_edit->control.x1 >= pattern_edit->control_current.x0)
+    if(pattern_edit->control.x1 >= pattern_edit->control_current.x0){
       note_x = (guint) (ceil((double) (pattern_edit->control.x1 - pattern_edit->control_current.x0) / (double) (pattern_edit->control_current.control_width)));
-    else{
+    }else{
       note_offset_x1 -= 1;
       note_x = 0;
     }
 
-    list_notation = machine->audio->notation;
+    /* add note */
+    pthread_mutex_lock(audio_mutex);
+
+    list_notation = audio->notation;
+
+    pthread_mutex_unlock(audio_mutex);
 
     i = 0;
 
     while((i = ags_notebook_next_active_tab(editor->current_notebook,
 							   i)) != -1){
-      list_notation = g_list_nth(machine->audio->notation,
+      /* retrieve notation */
+      pthread_mutex_lock(audio_mutex);
+
+      list_notation = g_list_nth(audio->notation,
 				 i);
 
+      pthread_mutex_unlock(audio_mutex);
+
       if(list_notation == NULL){
 	i++;
 	
@@ -365,8 +416,14 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
       
       note0 = ags_note_duplicate(note);
 
+      /* do it so */
+      pthread_mutex_lock(audio_mutex);
+
       ags_notation_add_note(AGS_NOTATION(list_notation->data), note0, FALSE);
 
+      pthread_mutex_unlock(audio_mutex);
+
+      /* iterate */
       i++;
     }
 
@@ -424,32 +481,67 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
       }
     }
 
-    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+    cairo_set_source_rgb(cr,
+			 pattern_edit_style->fg[0].red / white_gc,
+			 pattern_edit_style->fg[0].green / white_gc,
+			 pattern_edit_style->fg[0].blue / white_gc);
     cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
     cairo_fill(cr);
   }
   void ags_pattern_edit_drawing_area_button_release_event_delete_point(){
+    AgsAudio *audio;
+    
+    AgsMutexManager *mutex_manager;
+
     GList *list_notation;
+
     guint x, y;
     gint history;
     gint i;
 
+    pthread_mutex_t *audio_mutex;
+
+    audio = machine->audio;
+    
+    /* lookup audio mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    mutex_manager = ags_mutex_manager_get_instance();
+    
+    audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) audio);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* retrieve point */
     x = pattern_edit->control.x0_offset + pattern_edit->control.x0 - 1;
     y = pattern_edit->control.y0_offset + pattern_edit->control.y0;
 
     x = (guint) ceil((double) x / (double) (pattern_edit->control_unit.control_width));
     y = (guint) floor((double) y / (double) (pattern_edit->control_height));
 
+#ifdef AGS_DEBUG
     g_message("%d, %d\0", x, y);
-
+#endif
+    
     /* select notes */
-    list_notation = machine->audio->notation;
+    pthread_mutex_lock(audio_mutex);
+
+    list_notation = audio->notation;
+
+    pthread_mutex_unlock(audio_mutex);
+
     i = 0;
 
     while((i = ags_notebook_next_active_tab(editor->current_notebook,
 					    i)) != -1){
-      list_notation = g_list_nth(machine->audio->notation,
+      /* retrieve notation */
+      pthread_mutex_lock(audio_mutex);
+
+      list_notation = g_list_nth(audio->notation,
 				 i);
+
+      pthread_mutex_unlock(audio_mutex);
       
       if(list_notation == NULL){
 	i++;
@@ -457,17 +549,42 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
 	continue;
       }
 
+      /* do it so */
+      pthread_mutex_lock(audio_mutex);
+
       ags_notation_remove_note_at_position(AGS_NOTATION(list_notation->data),
 					   x, y);
 
+      pthread_mutex_unlock(audio_mutex);
+
+      /* iterate */
       i++;
     }
   }
   void ags_pattern_edit_drawing_area_button_release_event_select_region(){
+    AgsAudio *audio;
+    
+    AgsMutexManager *mutex_manager;
+
     GList *list_notation;
+    
     guint x0, x1, y0, y1;
     gint i;
     
+    pthread_mutex_t *audio_mutex;
+
+    audio = machine->audio;
+    
+    /* lookup audio mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    mutex_manager = ags_mutex_manager_get_instance();
+    
+    audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) audio);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
     /* get real size and offset */
     x0 = pattern_edit->control.x0_offset + pattern_edit->control.x0;
     x1 = pattern_edit->control.x1_offset + pattern_edit->control.x1;
@@ -501,24 +618,41 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
     y1 = (guint) ceil((double) y1 / (double) (pattern_edit->control_height));
 
     /* select notes */
-    list_notation = machine->audio->notation;
+    pthread_mutex_lock(audio_mutex);
 
+    list_notation = audio->notation;
+
+    pthread_mutex_unlock(audio_mutex);
+
+    i = 0;
+    
     while((i = ags_notebook_next_active_tab(editor->current_notebook,
 					    i)) != -1){
-      list_notation = g_list_nth(machine->audio->notation,
+      /* retrieve notation */
+      pthread_mutex_lock(audio_mutex);
+
+      list_notation = g_list_nth(audio->notation,
 				 i);
 
+      pthread_mutex_unlock(audio_mutex);
+
       if(list_notation == NULL){
 	i++;
 	
 	continue;
       }
 
+      /* do it so */
+      pthread_mutex_lock(audio_mutex);
+
       ags_notation_add_region_to_selection(AGS_NOTATION(list_notation->data),
 					   x0, y0,
 					   x1, y1,
 					   TRUE);
 
+      pthread_mutex_unlock(audio_mutex);
+
+      /* iterate */
       i++;
     }
   }
@@ -557,6 +691,8 @@ ags_pattern_edit_drawing_area_button_release_event(GtkWidget *widget, GdkEventBu
     machine = editor->selected_machine;
     note = pattern_edit->control.note;
 
+    pattern_edit_style = gtk_widget_get_style(widget);
+
     /* store the events position */
     pattern_edit->control.x1_offset = (guint) round((double) pattern_edit->hscrollbar->scrollbar.range.adjustment->value);
     pattern_edit->control.y1_offset = (guint) round((double) pattern_edit->vscrollbar->scrollbar.range.adjustment->value);
@@ -655,33 +791,39 @@ ags_pattern_edit_drawing_area_motion_notify_event (GtkWidget *widget, GdkEventMo
 {
   AgsMachine *machine;
   AgsEditor *editor;
+
+  GtkStyle *pattern_edit_style;
   AgsNote *note, *note0;
+
   double value[2];
   double tact;
   guint note_x1;
   guint prev_x1;
+
+  static const gdouble white_gc = 65535.0;
+
   void ags_pattern_edit_drawing_area_motion_notify_event_set_control(){
-    GList *list_notation;
     guint note_x, note_y;
     guint note_offset_x1;
 
-    if(pattern_edit->control.x0 >= pattern_edit->map_width)
+    if(pattern_edit->control.x0 >= pattern_edit->map_width){
       pattern_edit->control.x0 = pattern_edit->map_width - 1;
-
+    }
+    
     note_offset_x1 = (guint) (ceil(pattern_edit->control.x1_offset / (double) (pattern_edit->control_current.control_width)));
 
-    if(pattern_edit->control.x1 >= pattern_edit->control_current.x0)
+    if(pattern_edit->control.x1 >= pattern_edit->control_current.x0){
       note_x = (guint) (ceil((double) (pattern_edit->control.x1 - pattern_edit->control_current.x0) / (double) (pattern_edit->control_current.control_width)));
-    else{
+    }else{
       note_offset_x1 -= 1;
       note_x = 0;
     }
 
     note_x1 = (note_x * tact) + (note_offset_x1 * tact);
 
-    list_notation = machine->audio->notation;
-
+#ifdef AGS_DEBUG
     fprintf(stdout, "x0 = %llu\nx1 = %llu\ny  = %llu\n\n\0", (long long unsigned int) note->x[0], (long long unsigned int) note->x[1], (long long unsigned int) note->y);
+#endif
   }
   void ags_pattern_edit_drawing_area_motion_notify_event_draw_control(cairo_t *cr){
     guint x, y, width, height;
@@ -736,7 +878,10 @@ ags_pattern_edit_drawing_area_motion_notify_event (GtkWidget *widget, GdkEventMo
       }
     }
 
-    cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
+    cairo_set_source_rgb(cr,
+			 pattern_edit_style->fg[0].red / white_gc,
+			 pattern_edit_style->fg[0].green / white_gc,
+			 pattern_edit_style->fg[0].blue / white_gc);
     cairo_rectangle(cr, (double) x, (double) y, (double) width, (double) height);
     cairo_fill(cr);
   }
@@ -845,6 +990,8 @@ ags_pattern_edit_drawing_area_motion_notify_event (GtkWidget *widget, GdkEventMo
     machine = editor->selected_machine;
     note = pattern_edit->control.note;
 
+    pattern_edit_style = gtk_widget_get_style(widget);
+
     pattern_edit->control.x1_offset = (guint) round((double) pattern_edit->hscrollbar->scrollbar.range.adjustment->value);
     pattern_edit->control.y1_offset = (guint) round((double) pattern_edit->vscrollbar->scrollbar.range.adjustment->value);
 
@@ -890,11 +1037,14 @@ ags_pattern_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *ev
 {
   AgsEditor *editor;
 
+  if(event->keyval == GDK_KEY_Tab){
+    return(FALSE);
+  }
+
   editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(pattern_edit),
 						 AGS_TYPE_EDITOR);
 
   if(editor->selected_machine != NULL){
-
     switch(event->keyval){
     case GDK_KEY_Control_L:
       {
@@ -948,22 +1098,470 @@ ags_pattern_edit_drawing_area_key_press_event(GtkWidget *widget, GdkEventKey *ev
       break;
     }
   }
+
+  return(TRUE);
 }
 
 gboolean
 ags_pattern_edit_drawing_area_key_release_event(GtkWidget *widget, GdkEventKey *event, AgsPatternEdit *pattern_edit)
 {
-  switch(event->keyval){
-  case GDK_KEY_Control_L:
-    {
-      pattern_edit->key_mask &= (~AGS_PATTERN_EDIT_KEY_L_CONTROL);
+  AgsEditor *editor;
+  AgsMachine *machine;
+      
+  AgsChannel *channel;
+
+  AgsMutexManager *mutex_manager;
+
+  GList *list_notation;
+
+  gint i;
+  gboolean do_feedback;
+      
+  pthread_mutex_t *audio_mutex;
+  
+  auto void ags_pattern_edit_drawing_area_key_release_event_play_channel(AgsChannel *channel);
+
+  void ags_pattern_edit_drawing_area_key_release_event_play_channel(AgsChannel *channel){
+    AgsDevout *devout;
+    AgsAudio *audio;
+
+    AgsStartDevout *start_devout;
+    AgsInitChannel *init_channel;
+    AgsAppendChannel *append_channel;
+
+    AgsAudioLoop *audio_loop;
+    AgsTaskThread *task_thread;
+    AgsDevoutThread *devout_thread;
+
+    AgsMutexManager *mutex_manager;
+
+    AgsMain *ags_main;
+    
+    GList *tasks;
+
+    pthread_mutex_t *devout_mutex;
+    pthread_mutex_t *audio_mutex;
+    pthread_mutex_t *channel_mutex;
+
+    /* lookup channel mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+  
+    mutex_manager = ags_mutex_manager_get_instance();
+
+    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) channel);
+
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get audio */
+    pthread_mutex_lock(channel_mutex);
+    
+    audio = (AgsAudio *) channel->audio;
+
+    pthread_mutex_unlock(channel_mutex);
+    
+    /* lookup audio mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+    
+    audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) audio);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get devout */
+    pthread_mutex_lock(audio_mutex);
+
+    devout = (AgsDevout *) audio->devout;
+
+    pthread_mutex_unlock(audio_mutex);
+
+    /* lookup audio mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+    
+    devout_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) devout);
+  
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get ags_main */
+    pthread_mutex_lock(devout_mutex);
+
+    ags_main = (AgsMain *) devout->ags_main;
+
+    pthread_mutex_unlock(devout_mutex);
+
+    /* get threads */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    audio_loop = (AgsAudioLoop *) ags_main->main_loop;
+
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get task thread and devout thread */
+    task_thread = (AgsTaskThread *) ags_thread_find_type(audio_loop,
+							 AGS_TYPE_TASK_THREAD);
+    devout_thread = (AgsDevoutThread *) ags_thread_find_type(audio_loop,
+							     AGS_TYPE_DEVOUT_THREAD);
+
+    /* create tasks */
+    tasks = NULL;
+
+    /* init channel for playback */
+    init_channel = ags_init_channel_new(channel, FALSE,
+					TRUE, FALSE, FALSE);
+    g_signal_connect_after(G_OBJECT(init_channel), "launch\0",
+			   G_CALLBACK(ags_pattern_edit_init_channel_launch_callback), NULL);
+    tasks = g_list_prepend(tasks, init_channel);
+    
+    /* append channel for playback */
+    append_channel = ags_append_channel_new((GObject *) audio_loop,
+					    (GObject *) channel);
+    tasks = g_list_prepend(tasks, append_channel);
+
+    /* create start task */
+    start_devout = ags_start_devout_new(devout);
+    tasks = g_list_prepend(tasks,
+			   start_devout);
+
+    /* perform playback */
+    tasks = g_list_reverse(tasks);
+    ags_task_thread_append_tasks(task_thread, tasks);
+  }
+  
+  if(event->keyval == GDK_KEY_Tab){
+    return(FALSE);
+  }
+
+  editor = (AgsEditor *) gtk_widget_get_ancestor(GTK_WIDGET(pattern_edit),
+						 AGS_TYPE_EDITOR);
+
+  machine = editor->selected_machine;
+
+  do_feedback = FALSE;
+  
+  /* lookup audio mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+    
+  mutex_manager = ags_mutex_manager_get_instance();
+    
+  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+					 (GObject *) machine->audio);
+    
+  pthread_mutex_unlock(&(ags_application_mutex));
+ 
+  if(machine != NULL){
+    switch(event->keyval){
+    case GDK_KEY_Control_L:
+      {
+	pattern_edit->key_mask &= (~AGS_PATTERN_EDIT_KEY_L_CONTROL);
+      }
+      break;
+    case GDK_KEY_Control_R:
+      {
+	pattern_edit->key_mask &= (~AGS_PATTERN_EDIT_KEY_R_CONTROL);
+      }
+      break;
+    case GDK_KEY_Left:
+    case GDK_KEY_leftarrow:
+      {
+	gdouble tact;
+	guint x0_offset;
+
+	tact = exp2(6.0 - (double) gtk_combo_box_get_active(editor->toolbar->zoom));
+      
+	if(pattern_edit->selected_x > 0){
+	  if(pattern_edit->selected_x - (1 * tact) > 0){
+	    pattern_edit->selected_x -= (1 * tact);
+	  }else{
+	    pattern_edit->selected_x = 0;
+	  }
+
+	  do_feedback = TRUE;
+	}
+
+	x0_offset = pattern_edit->selected_x * pattern_edit->control_unit.control_width;
+      
+	if(x0_offset < GTK_RANGE(pattern_edit->hscrollbar)->adjustment->value){
+	  gtk_range_set_value(GTK_RANGE(pattern_edit->hscrollbar),
+			      x0_offset * GTK_RANGE(pattern_edit->hscrollbar)->adjustment->step_increment);
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+      }
+      break;
+    case GDK_KEY_Right:
+    case GDK_KEY_rightarrow:
+      {
+	gdouble tact;
+	guint x0_offset;
+
+	tact = exp2(6.0 - (double) gtk_combo_box_get_active(editor->toolbar->zoom));
+      
+	if(pattern_edit->selected_x < 16.0 * AGS_PATTERN_EDIT_MAX_CONTROLS){
+	  pattern_edit->selected_x += (1.0 * tact);
+
+	  do_feedback = TRUE;
+	}
+
+	x0_offset = pattern_edit->selected_x * pattern_edit->control_unit.control_width;
+      
+	if(x0_offset + pattern_edit->control_current.control_width > GTK_RANGE(pattern_edit->hscrollbar)->adjustment->value + GTK_WIDGET(pattern_edit->drawing_area)->allocation.width){
+	  gtk_range_set_value(GTK_RANGE(pattern_edit->hscrollbar),
+			      x0_offset * GTK_RANGE(pattern_edit->hscrollbar)->adjustment->step_increment);
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+      }
+      break;
+    case GDK_KEY_Up:
+    case GDK_KEY_uparrow:
+      {
+	gdouble y0_offset;
+      
+	if(pattern_edit->selected_y > 0){
+	  pattern_edit->selected_y -= 1;
+
+	  do_feedback = TRUE;
+	}
+
+	y0_offset = pattern_edit->selected_y * pattern_edit->control_height;
+      
+	if(y0_offset < GTK_RANGE(pattern_edit->vscrollbar)->adjustment->value){
+	  gtk_range_set_value(GTK_RANGE(pattern_edit->vscrollbar),
+			      y0_offset * GTK_RANGE(pattern_edit->vscrollbar)->adjustment->step_increment);
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+      }
+      break;
+    case GDK_KEY_Down:
+    case GDK_KEY_downarrow:
+      {
+	gdouble y0_offset;
+      
+	if(pattern_edit->selected_y * (pattern_edit->control_height) < pattern_edit->map_height){
+	  pattern_edit->selected_y += 1;
+
+	  do_feedback = TRUE;
+	}
+
+	y0_offset = pattern_edit->selected_y * pattern_edit->control_height;
+
+	if(y0_offset + pattern_edit->control_height > GTK_RANGE(pattern_edit->vscrollbar)->adjustment->value + (GTK_WIDGET(pattern_edit->drawing_area)->allocation.height)){
+	  gtk_range_set_value(GTK_RANGE(pattern_edit->vscrollbar),
+			      y0_offset * GTK_RANGE(pattern_edit->vscrollbar)->adjustment->step_increment);
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+      }
+      break;
+    case GDK_KEY_space:
+      {
+	AgsNote *note;
+      
+	i = 0;
+	do_feedback = TRUE;
+
+	while((i = ags_notebook_next_active_tab(editor->current_notebook,
+						i)) != -1){
+	  pthread_mutex_lock(audio_mutex);
+	  
+	  list_notation = g_list_nth(machine->audio->notation,
+				     i);
+
+	  pthread_mutex_unlock(audio_mutex);
+	  
+	  if(list_notation == NULL){
+	    i++;
+	
+	    continue;
+	  }
+      
+	  note = ags_note_new();
+	  note->x[0] = pattern_edit->selected_x;
+	  note->x[1] = pattern_edit->selected_x + 1;
+	  note->y = pattern_edit->selected_y;
+
+	  pthread_mutex_lock(audio_mutex);
+
+	  ags_notation_add_note(AGS_NOTATION(list_notation->data), note, FALSE);
+
+	  pthread_mutex_unlock(audio_mutex);
+
+	  i++;
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+
+	fprintf(stdout, "x0 = %llu\nx1 = %llu\ny  = %llu\n\n\0", (long long unsigned int) note->x[0], (long long unsigned int) note->x[1], (long long unsigned int) note->y);
+      }
+      break;
+    case GDK_KEY_Delete:
+      {      
+	machine = editor->selected_machine;
+
+	i = 0;
+
+	while((i = ags_notebook_next_active_tab(editor->current_notebook,
+						i)) != -1){
+	  pthread_mutex_lock(audio_mutex);
+	  
+	  list_notation = g_list_nth(machine->audio->notation,
+				     i);
+
+	  pthread_mutex_unlock(audio_mutex);
+
+	  if(list_notation == NULL){
+	    i++;
+	
+	    continue;
+	  }
+
+	  pthread_mutex_lock(audio_mutex);
+
+	  ags_notation_remove_note_at_position(AGS_NOTATION(list_notation->data),
+					       pattern_edit->selected_x, pattern_edit->selected_y);
+
+	  pthread_mutex_unlock(audio_mutex);
+
+	  i++;
+	}
+
+	gtk_widget_queue_draw(pattern_edit);
+      }
+      break;
     }
-    break;
-  case GDK_KEY_Control_R:
-    {
-      pattern_edit->key_mask &= (~AGS_PATTERN_EDIT_KEY_R_CONTROL);
+  }
+  
+  if(do_feedback){  
+    AgsAudio *audio;
+    AgsNote *current_note;
+
+    guint flags;
+    gboolean has_note;
+
+    audio = machine->audio;
+
+    /* audible feedback */
+    i = 0;
+
+    while((i = ags_notebook_next_active_tab(editor->current_notebook,
+					    i)) != -1){
+      pthread_mutex_lock(audio_mutex);
+      
+      flags = audio->flags;
+      list_notation = g_list_nth(audio->notation,
+				 i);
+
+      pthread_mutex_unlock(audio_mutex);
+
+      if(list_notation == NULL){
+	i++;
+	
+	continue;
+      }
+
+      pthread_mutex_lock(audio_mutex);
+      
+      has_note = ((ags_notation_find_point(list_notation->data,
+					   pattern_edit->selected_x, pattern_edit->selected_y,
+					   FALSE) != NULL) ?
+		  TRUE:
+		  FALSE);
+
+      pthread_mutex_unlock(audio_mutex);
+      
+      if(has_note){
+	channel = ags_channel_nth(machine->audio->input,
+				  i);
+
+	if((AGS_AUDIO_REVERSE_MAPPING & (flags)) != 0){
+	  channel = ags_channel_pad_nth(channel, machine->audio->input_pads - pattern_edit->selected_y - 1);
+	}else{
+	  channel = ags_channel_pad_nth(channel, pattern_edit->selected_y);
+	}
+
+	ags_pattern_edit_drawing_area_key_release_event_play_channel(channel);
+      }
+	  
+      i++;
     }
-    break;
+  }
+  
+  return(TRUE);
+}
+
+void
+ags_pattern_edit_init_channel_launch_callback(AgsTask *task, gpointer data)
+{
+  AgsDevout *devout;
+  AgsChannel *channel;
+  AgsRecycling *recycling;
+
+  AgsAddAudioSignal *add_audio_signal;
+
+  AgsAudioLoop *audio_loop;
+  AgsTaskThread *task_thread;
+
+  AgsMutexManager *mutex_manager;
+
+  GList *recall, *tmp;
+
+  pthread_mutex_t *audio_mutex;
+
+  channel = AGS_INIT_CHANNEL(task)->channel;
+
+  devout = AGS_DEVOUT(AGS_AUDIO(channel->audio)->devout);
+
+  audio_loop = AGS_AUDIO_LOOP(AGS_MAIN(devout->ags_main)->main_loop);
+  task_thread = ags_thread_find_type(audio_loop,
+				     AGS_TYPE_TASK_THREAD);
+
+#ifdef AGS_DEBUG
+  g_message("launch\0");
+#endif
+  
+  if(AGS_DEVOUT_PLAY(channel->devout_play) == NULL ||
+     AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0] == NULL){    
+    return;
+  }
+
+  /* connect done */
+  recall = ags_recall_find_provider_with_recycling_container(channel->play,
+							     G_OBJECT(channel),
+							     G_OBJECT(AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0]->recycling_container));
+  
+  tmp = recall;
+  recall = ags_recall_find_type(recall,
+				AGS_TYPE_PLAY_CHANNEL_RUN);
+  //TODO:JK: fix me
+  //    g_list_free(tmp);
+
+  if(recall != NULL){
+    AgsAudioSignal *audio_signal;
+      
+    /* add audio signal */
+    recycling = channel->first_recycling;
+
+    while(recycling != channel->last_recycling->next){
+      audio_signal = ags_audio_signal_new((GObject *) devout,
+					  (GObject *) recycling,
+					  (GObject *) AGS_RECALL(recall->data)->recall_id);
+      /* add audio signal */
+      ags_recycling_create_audio_signal_with_defaults(recycling,
+						      audio_signal,
+						      0.0, 0);
+      audio_signal->stream_current = audio_signal->stream_beginning;
+      ags_audio_signal_connect(audio_signal);
+  
+      /*
+       * emit add_audio_signal on AgsRecycling
+       */
+      ags_recycling_add_audio_signal(recycling,
+				     audio_signal);
+
+      recycling = recycling->next;
+    }    
   }
 }
 
