@@ -83,22 +83,31 @@ ags_drum_input_pad_open_callback(GtkWidget *widget, AgsDrumInputPad *drum_input_
 									 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 									 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 									 NULL);
-  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(file_chooser), FALSE);
+  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(file_chooser),
+				       FALSE);
   g_object_set_data((GObject *) file_chooser, (char *) g_type_name(AGS_TYPE_AUDIO_FILE), NULL);
   g_object_set_data((GObject *) file_chooser, AGS_DRUM_INPUT_PAD_OPEN_AUDIO_FILE_NAME, NULL);
 
   hbox = (GtkHBox *) gtk_hbox_new(FALSE, 0);
-  gtk_file_chooser_set_extra_widget((GtkFileChooser *) file_chooser, (GtkWidget *) hbox);
+  gtk_file_chooser_set_extra_widget((GtkFileChooser *) file_chooser,
+				    (GtkWidget *) hbox);
   
   label = (GtkLabel *) gtk_label_new(g_strdup("channel: \0"));
-  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) label, FALSE, FALSE, 0);
+  gtk_box_pack_start((GtkBox *) hbox,
+		     (GtkWidget *) label,
+		     FALSE, FALSE,
+		     0);
 
   spin_button = (GtkSpinButton *) gtk_spin_button_new_with_range(0.0, AGS_AUDIO(AGS_PAD(drum_input_pad)->channel->audio)->audio_channels - 1, 1.0);
   g_object_set_data((GObject *) file_chooser, AGS_DRUM_INPUT_PAD_OPEN_SPIN_BUTTON, spin_button);
-  gtk_box_pack_start((GtkBox *) hbox, (GtkWidget *) spin_button, FALSE, FALSE, 0);
+  gtk_box_pack_start((GtkBox *) hbox,
+		     (GtkWidget *) spin_button,
+		     FALSE, FALSE,
+		     0);
 
   if(drum_input_pad->pad.group->active)
-    gtk_widget_set_sensitive((GtkWidget *) spin_button, FALSE);
+    gtk_widget_set_sensitive((GtkWidget *) spin_button,
+			     FALSE);
 
   //  play = (GtkToggleButton *) g_object_new(GTK_TYPE_TOGGLE_BUTTON,
   //					  "label\0", GTK_STOCK_MEDIA_PLAY,
@@ -119,17 +128,41 @@ ags_drum_input_pad_open_callback(GtkWidget *widget, AgsDrumInputPad *drum_input_
 void
 ags_drum_input_pad_open_play_callback(GtkToggleButton *toggle_button, AgsDrumInputPad *drum_input_pad)
 {
+  AgsWindow *window;
   GtkFileChooserDialog *file_chooser;
+  
   AgsDevout *devout;
+
+  AgsThread *main_loop;
+  AgsThread *task_thread;
+  
+  AgsMain *ags_main;
+  
   GList *list, *tasks;
   gchar *name0, *name1;
 
+  window = gtk_widget_get_ancestor(drum_input_pad,
+				   AGS_TYPE_WINDOW);
+  
   file_chooser = drum_input_pad->file_chooser;
   name0 = (gchar *) gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
   name1 = g_object_get_data((GObject *) file_chooser, AGS_DRUM_INPUT_PAD_OPEN_AUDIO_FILE_NAME);
 
   devout = AGS_DEVOUT(AGS_AUDIO(AGS_PAD(drum_input_pad)->channel->audio)->devout);
 
+  ags_main = window->ags_main;
+
+  /* get main loop */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  main_loop = ags_main->main_loop;
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* find task thread */
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
+		  
   if(toggle_button->active){
     AgsDevoutPlay *devout_play;
     AgsPlayAudioSignal *play_audio_signal;
@@ -226,7 +259,7 @@ ags_drum_input_pad_open_play_callback(GtkToggleButton *toggle_button, AgsDrumInp
     start_devout = ags_start_devout_new(devout);
     tasks = g_list_prepend(tasks, start_devout);
 
-    ags_task_thread_append_tasks(AGS_TASK_THREAD(AGS_AUDIO_LOOP(AGS_MAIN(devout->ags_main)->main_loop)->task_thread),
+    ags_task_thread_append_tasks(task_thread,
 				 tasks);
   }else{
     if((AGS_DRUM_INPUT_PAD_OPEN_PLAY_DONE & (drum_input_pad->flags)) == 0){
@@ -282,17 +315,44 @@ ags_drum_input_pad_open_play_done(AgsRecall *recall, AgsDrumInputPad *drum_input
 void
 ags_drum_input_pad_open_response_callback(GtkWidget *widget, gint response, AgsDrumInputPad *drum_input_pad)
 {
+  AgsWindow *window;
   AgsDrum *drum;
-  AgsAudioFile *audio_file;
+
   GtkFileChooserDialog *file_chooser;
   GtkSpinButton *spin_button;
+
+  AgsAudioFile *audio_file;
+
   AgsOpenSingleFile *open_single_file;
+
+  AgsThread *main_loop;
+  AgsThread *task_thread;
+  
+  AgsMain *ags_main;
+  
   char *name0, *name1;
 
   drum = (AgsDrum *) gtk_widget_get_ancestor(GTK_WIDGET(drum_input_pad), AGS_TYPE_DRUM);
 
+  window = gtk_widget_get_ancestor(drum_input_pad,
+				   AGS_TYPE_WINDOW);
+
   file_chooser = drum_input_pad->file_chooser;
 
+  
+  ags_main = window->ags_main;
+
+  /* get main loop */
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  main_loop = ags_main->main_loop;
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* find task thread */
+  task_thread = ags_thread_find_type(main_loop,
+				     AGS_TYPE_TASK_THREAD);
+
   if(response == GTK_RESPONSE_ACCEPT){
     name0 = gtk_file_chooser_get_filename((GtkFileChooser *) file_chooser);
     name1 = (char *) g_object_get_data((GObject *) file_chooser, AGS_DRUM_INPUT_PAD_OPEN_AUDIO_FILE_NAME);
@@ -330,8 +390,8 @@ ags_drum_input_pad_open_response_callback(GtkWidget *widget, gint response, AgsD
       g_list_free(list);
     }
 
-    ags_task_thread_append_task(AGS_TASK_THREAD(AGS_AUDIO_LOOP(AGS_MAIN(AGS_DEVOUT(AGS_AUDIO(AGS_MACHINE(drum)->audio)->devout)->ags_main)->main_loop)->task_thread),
-									AGS_TASK(open_single_file));
+    ags_task_thread_append_task(task_thread,
+				AGS_TASK(open_single_file));
 
     gtk_widget_destroy((GtkWidget *) file_chooser);
   }else if(response == GTK_RESPONSE_CANCEL){
@@ -350,271 +410,7 @@ ags_drum_input_pad_open_response_callback(GtkWidget *widget, gint response, AgsD
 void
 ags_drum_input_pad_play_callback(GtkToggleButton *toggle_button, AgsDrumInputPad *drum_input_pad)
 {
-  AgsDrum *drum;
-
-  AgsDevout *devout;
-  AgsChannel *channel;
-
-  AgsAudioLoop *audio_loop;
-  AgsTaskThread *task_thread;
-  AgsDevoutThread *devout_thread;
-
-  AgsMutexManager *mutex_manager;
-
-  GList *tasks;
-
-  pthread_mutex_t *audio_mutex;
-  
-  drum = (AgsDrum *) gtk_widget_get_ancestor((GtkWidget *) drum_input_pad, AGS_TYPE_DRUM);
-
-  pthread_mutex_lock(&(ags_application_mutex));
-  
-  mutex_manager = ags_mutex_manager_get_instance();
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) AGS_MACHINE(drum)->audio);
-  
-  pthread_mutex_unlock(&(ags_application_mutex));
-
-  pthread_mutex_lock(audio_mutex);
-  
-  devout = AGS_DEVOUT(AGS_MACHINE(drum)->audio->devout);
-
-  audio_loop = AGS_AUDIO_LOOP(AGS_MAIN(devout->ags_main)->main_loop);
-  task_thread = AGS_TASK_THREAD(audio_loop->task_thread);
-  devout_thread = AGS_DEVOUT_THREAD(audio_loop->devout_thread);
-
-  tasks = NULL;
-
-  if(toggle_button->active){
-    AgsStartDevout *start_devout;
-    AgsInitChannel *init_channel;
-    AgsAppendChannel *append_channel;
-    gboolean play_all;
-    guint flags;
-
-    play_all = AGS_PAD(drum_input_pad)->group->active;
-
-    flags = AGS_AUDIO_SIGNAL_STANDALONE;
-
-    channel = AGS_PAD(drum_input_pad)->channel;
-
-    /* init channel for playback */
-    init_channel = ags_init_channel_new(channel, play_all,
-					TRUE, FALSE, FALSE);
-    g_signal_connect_after(G_OBJECT(init_channel), "launch\0",
-			   G_CALLBACK(ags_drum_input_pad_init_channel_launch_callback), drum_input_pad);
-    tasks = g_list_prepend(tasks, init_channel);
-
-    if(play_all){
-      AgsChannel *next_pad;
-
-      next_pad = channel->next_pad;
-
-      while(channel != next_pad){
-	/* append channel for playback */
-	append_channel = ags_append_channel_new(G_OBJECT(audio_loop),
-						G_OBJECT(channel));
-	tasks = g_list_prepend(tasks, append_channel);
-
-	//	drum_input_pad->pad_play_ref++;
-	channel = channel->next;
-      }
-    }else{
-      AgsLine *line;
-      GList *list;
-
-      list = gtk_container_get_children(GTK_CONTAINER(AGS_PAD(drum_input_pad)->expander_set));
-      line = AGS_LINE(ags_line_find_next_grouped(list)->data);
-
-      /* append channel for playback */
-      append_channel = ags_append_channel_new(G_OBJECT(audio_loop),
-					      G_OBJECT(line->channel));
-      tasks = g_list_prepend(tasks, append_channel);
-
-      g_list_free(list);
-      //      drum_input_pad->pad_play_ref++;
-    }
-
-    /* create start task */
-    start_devout = ags_start_devout_new(devout);
-    tasks = g_list_prepend(tasks,
-			   start_devout);
-
-    /* perform playback */
-    tasks = g_list_reverse(tasks);
-    ags_task_thread_append_tasks(task_thread, tasks);
-  }else{
-    AgsCancelChannel *cancel_channel;
-
-    channel = AGS_PAD(drum_input_pad)->channel;
-
-    if(AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0] == NULL ||
-       (AGS_DEVOUT_PLAY_DONE & (g_atomic_int_get(&(AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0]->flags)))) != 0){
-      pthread_mutex_unlock(audio_mutex);
-      return;
-    }
-
-    if((AGS_DEVOUT_PLAY_PAD & (g_atomic_int_get(&(AGS_DEVOUT_PLAY(channel->devout_play)->flags)))) != 0){
-      AgsChannel *next_pad;
-
-      next_pad = channel->next_pad;
-
-      if((AGS_DEVOUT_PLAY_DONE & (g_atomic_int_get(&(AGS_DEVOUT_PLAY(channel->devout_play)->flags)))) == 0){
-	/* cancel request */
-	while(channel != next_pad){
-	  cancel_channel = ags_cancel_channel_new(channel, AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0],
-						  AGS_DEVOUT_PLAY(channel->devout_play));
-
-	  ags_task_thread_append_task(task_thread, (AgsTask *) cancel_channel);
-
-	  channel = channel->next;
-	}
-      }else{
-	/* done */
-	while(channel != next_pad){
-	  g_atomic_int_or(&(AGS_DEVOUT_PLAY(channel->devout_play)->flags),
-			  AGS_DEVOUT_PLAY_REMOVE);
-	  g_atomic_int_and(&(AGS_DEVOUT_PLAY(channel->devout_play)->flags),
-			   (~AGS_DEVOUT_PLAY_DONE));
-
-	  channel = channel->next;
-	}
-      }
-    }else{
-      AgsLine *line;
-      GList *list;
-
-      list = gtk_container_get_children(GTK_CONTAINER(AGS_PAD(drum_input_pad)->expander_set));
-      line = AGS_LINE(ags_line_find_next_grouped(list)->data);
-
-      g_list_free(list);
-
-      /*  */
-      channel = line->channel;
-
-      if((AGS_DEVOUT_PLAY_DONE & (g_atomic_int_get(&(AGS_DEVOUT_PLAY(channel->devout_play)->flags)))) == 0){
-	/* cancel request */
-	cancel_channel = ags_cancel_channel_new(channel, AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0],
-						AGS_DEVOUT_PLAY(channel->devout_play));
-
-	ags_task_thread_append_task(task_thread, (AgsTask *) cancel_channel);
-      }else{
-	/* done */
-	AGS_DEVOUT_PLAY(channel->devout_play)->flags |= AGS_DEVOUT_PLAY_REMOVE;
-	AGS_DEVOUT_PLAY(channel->devout_play)->flags &= (~AGS_DEVOUT_PLAY_DONE);
-      }
-    }
-  }
-  
-  pthread_mutex_unlock(audio_mutex);
-}
-
-void
-ags_drum_input_pad_init_channel_launch_callback(AgsTask *task,
-						AgsDrumInputPad *drum_input_pad)
-{
-  AgsDevout *devout;
-  AgsChannel *channel, *next_pad;
-  AgsRecycling *recycling;
-
-  AgsAddAudioSignal *add_audio_signal;
-
-  AgsAudioLoop *audio_loop;
-  AgsTaskThread *task_thread;
-
-  AgsMutexManager *mutex_manager;
-
-  GList *recall, *tmp;
-  GList *list, *list_start;
-
-  pthread_mutex_t *audio_mutex;
-
-  pthread_mutex_lock(&(ags_application_mutex));
-  
-  mutex_manager = ags_mutex_manager_get_instance();
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) AGS_PAD(drum_input_pad)->channel->audio);
-  
-  pthread_mutex_unlock(&(ags_application_mutex));
-
-  pthread_mutex_lock(audio_mutex);
-  
-  devout = AGS_DEVOUT(AGS_AUDIO(AGS_PAD(drum_input_pad)->channel->audio)->devout);
-
-  audio_loop = AGS_AUDIO_LOOP(AGS_MAIN(devout->ags_main)->main_loop);
-  task_thread = AGS_TASK_THREAD(audio_loop->task_thread);
-
-
-  list_start = 
-    list = gtk_container_get_children((GtkContainer *) AGS_PAD(drum_input_pad)->expander_set);
-  
-  channel = AGS_PAD(drum_input_pad)->channel;
-  next_pad = channel->next_pad;
-
-  g_message("launch\0");
-  
-  while(channel != next_pad){
-    if(AGS_DEVOUT_PLAY(channel->devout_play) == NULL ||
-       AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0] == NULL){
-      channel = channel->next;
-      list = list->next;
-
-      continue;
-    }
-
-    /* connect done */
-    recall = ags_recall_find_provider_with_recycling_container(channel->play,
-							       G_OBJECT(channel),
-							       G_OBJECT(AGS_DEVOUT_PLAY(channel->devout_play)->recall_id[0]->recycling_container));
-
-    tmp = recall;
-    recall = ags_recall_find_type(recall,
-				  AGS_TYPE_PLAY_CHANNEL_RUN);
-    //TODO:JK: fix me
-    //    g_list_free(tmp);
-
-    if(recall != NULL){
-      AgsAudioSignal *audio_signal;
-      
-      g_signal_connect_after(channel, "done\0",
-			     G_CALLBACK(ags_line_channel_done_callback), AGS_LINE(list->data));
-      
-      /* add audio signal */
-      recycling = channel->first_recycling;
-
-      while(recycling != channel->last_recycling->next){
-	audio_signal = ags_audio_signal_new((GObject *) devout,
-					    (GObject *) recycling,
-					    (GObject *) AGS_RECALL(recall->data)->recall_id);
-	/* add audio signal */
-	ags_recycling_create_audio_signal_with_defaults(recycling,
-							audio_signal,
-							0.0, 0);
-	audio_signal->stream_current = audio_signal->stream_beginning;
-	ags_audio_signal_connect(audio_signal);
-  
-	/*
-	 * emit add_audio_signal on AgsRecycling
-	 */
-	ags_recycling_add_audio_signal(recycling,
-				       audio_signal);
-
-	recycling = recycling->next;
-      }    
-    }
-
-    //TODO:JK: fix me
-    //    g_list_free(recall);
-
-    channel = channel->next;
-    list = list->next;
-  }
-
-  g_list_free(list_start);
-
-  pthread_mutex_unlock(audio_mutex);
+  ags_pad_play(AGS_PAD(drum_input_pad));
 }
 
 void
@@ -627,16 +423,27 @@ ags_drum_input_pad_edit_callback(GtkWidget *toggle_button, AgsDrumInputPad *drum
 
   if(drum->selected_edit_button != NULL){
     if(GTK_TOGGLE_BUTTON(toggle_button) != drum->selected_edit_button){
+      /* unset old */
       toggle = drum->selected_edit_button;
+
       drum->selected_edit_button = NULL;
       gtk_toggle_button_set_active((GtkToggleButton *) toggle, FALSE);
+
+      /* apply new */
       drum->selected_edit_button = (GtkToggleButton *) toggle_button;
       drum->selected_pad = (AgsDrumInputPad *) gtk_widget_get_ancestor((GtkWidget *) toggle_button, AGS_TYPE_DRUM_INPUT_PAD);
-      ags_drum_set_pattern(drum);
+
+      AGS_MACHINE(drum)->selected_input_pad = drum->selected_pad;
+      
+      ags_pattern_box_set_pattern(drum->pattern_box);
     }else{
+      /* chain up */
       toggle = drum->selected_edit_button;
+      
       drum->selected_edit_button = NULL;
       gtk_toggle_button_set_active((GtkToggleButton *) toggle, TRUE);
+
+      /* reset */
       drum->selected_edit_button = toggle;
     }
   }
