@@ -37,6 +37,10 @@
 #include <ags/file/ags_file_launch.h>
 #include <ags/file/ags_file_gui.h>
 
+#include <ags/thread/ags_mutex_manager.h>
+#include <ags/thread/ags_thread-posix.h>
+#include <ags/thread/ags_audio_loop.h>
+
 #include <ags/audio/ags_audio.h>
 #include <ags/audio/ags_input.h>
 #include <ags/audio/ags_output.h>
@@ -45,6 +49,7 @@
 #include <ags/audio/ags_recall_container.h>
 
 #include <ags/audio/file/ags_audio_file.h>
+#include <ags/audio/file/ags_ipatch.h>
 #include <ags/audio/file/ags_ipatch_sf2_reader.h>
 
 #include <ags/audio/recall/ags_delay_audio.h>
@@ -63,6 +68,7 @@
 #include <ags/X/ags_editor.h>
 
 #include <math.h>
+#include <string.h>
 
 void ags_ffplayer_class_init(AgsFFPlayerClass *ffplayer);
 void ags_ffplayer_connectable_interface_init(AgsConnectableInterface *connectable);
@@ -140,8 +146,8 @@ ags_ffplayer_get_type(void)
     };
 
     ags_type_ffplayer = g_type_register_static(AGS_TYPE_MACHINE,
-					    "AgsFFPlayer\0", &ags_ffplayer_info,
-					    0);
+					       "AgsFFPlayer\0", &ags_ffplayer_info,
+					       0);
     
     g_type_add_interface_static(ags_type_ffplayer,
 				AGS_TYPE_CONNECTABLE,
@@ -221,9 +227,9 @@ ags_ffplayer_init(AgsFFPlayer *ffplayer)
 		   AGS_AUDIO_INPUT_HAS_RECYCLING |
 		   AGS_AUDIO_INPUT_TAKES_FILE |
 		   AGS_AUDIO_SYNC |
-		   AGS_AUDIO_ASYNC |
-		   AGS_AUDIO_HAS_NOTATION | 
-		   AGS_AUDIO_NOTATION_DEFAULT);
+		   AGS_AUDIO_ASYNC | 
+		   AGS_AUDIO_NOTATION_DEFAULT |
+		   AGS_AUDIO_HAS_NOTATION);
   
   AGS_MACHINE(ffplayer)->flags |= (AGS_MACHINE_IS_SYNTHESIZER |
 				   AGS_MACHINE_REVERSE_NOTATION);
@@ -334,18 +340,26 @@ void
 ags_ffplayer_map_recall(AgsMachine *machine)
 {
   AgsWindow *window;
-  
+  AgsFFPlayer *ffplayer;
+
   AgsAudio *audio;
+  AgsChannel *channel;
 
   AgsDelayAudio *play_delay_audio;
   AgsDelayAudioRun *play_delay_audio_run;
   AgsCountBeatsAudio *play_count_beats_audio;
   AgsCountBeatsAudioRun *play_count_beats_audio_run;
+  AgsPlayNotationAudio  *play_notation;
   AgsPlayNotationAudio *recall_notation_audio;
   AgsPlayNotationAudioRun *recall_notation_audio_run;
 
+  GList *notation;
   GList *list;
 
+  guint i, j;
+
+  GValue value = {0,};
+
   if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0 ||
      (AGS_MACHINE_PREMAPPED_RECALL & (machine->flags)) != 0){
     return;
@@ -354,6 +368,7 @@ ags_ffplayer_map_recall(AgsMachine *machine)
   window = gtk_widget_get_ancestor(machine,
 				   AGS_TYPE_WINDOW);
 
+  ffplayer = AGS_FFPLAYER(machine);
   audio = machine->audio;
 
   /* ags-delay */
@@ -388,8 +403,6 @@ ags_ffplayer_map_recall(AgsMachine *machine)
   list = ags_recall_find_type(audio->play, AGS_TYPE_COUNT_BEATS_AUDIO_RUN);
 
   if(list != NULL){
-    GValue value = {0,};
-
     play_count_beats_audio_run = AGS_COUNT_BEATS_AUDIO_RUN(list->data);
 
     /* set dependency */  
@@ -421,7 +434,7 @@ ags_ffplayer_map_recall(AgsMachine *machine)
 
   if(list != NULL){
     recall_notation_audio_run = AGS_PLAY_NOTATION_AUDIO_RUN(list->data);
-    
+
     /* set dependency */
     g_object_set(G_OBJECT(recall_notation_audio_run),
 		 "delay-audio-run\0", play_delay_audio_run,
@@ -433,7 +446,13 @@ ags_ffplayer_map_recall(AgsMachine *machine)
 		 NULL);
   }
 
-  AGS_MACHINE_CLASS(ags_ffplayer_parent_class)->map_recall(machine);  
+  /* depending on destination */
+  ags_ffplayer_input_map_recall(ffplayer, 0);
+
+  /* depending on destination */
+  ags_ffplayer_output_map_recall(ffplayer, 0);
+
+  AGS_MACHINE_CLASS(ags_ffplayer_parent_class)->map_recall(machine);
 }
 
 void
@@ -631,12 +650,8 @@ void
 ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
 {
   AgsWindow *window;
-  AgsFFPlayer *gobject;
-  GtkTreeModel *list_store;
-  GtkTreeIter iter;
   xmlNode *node;
   gchar *filename;
-  gchar *preset, *instrument;
 
   window = AGS_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(ffplayer)));
   node = file_launch->node;
@@ -646,12 +661,16 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
 
   if(g_str_has_suffix(filename, ".sf2\0")){
     AgsIpatch *ipatch;
+    GtkTreeModel *list_store;
+    GtkTreeIter iter;
     AgsPlayable *playable;
-    gchar **preset, **instrument, *selected;
+    gchar **preset, **instrument;
+    xmlChar *selected;
     GError *error;
 
     /* clear preset, instrument and sample*/
-    ags_combo_box_text_remove_all(ffplayer->instrument);
+    gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(ffplayer->preset)));
+    gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(ffplayer->instrument)));
 
     /* Ipatch related */
     ffplayer->ipatch =
@@ -663,7 +682,7 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     ags_ipatch_open(ipatch, filename);
 
     playable = AGS_PLAYABLE(ipatch);
-      
+
     ags_playable_open(playable, filename);
 
     error = NULL;
@@ -675,7 +694,7 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     if(error != NULL){
       g_warning("%s\0", error->message);
     }
-    
+
     /* select first preset */
     ipatch->nth_level = 1;
     preset = ags_playable_sublevel_names(playable);
@@ -688,7 +707,7 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     if(error != NULL){
       g_warning("%s\0", error->message);
     }
-    
+
     /* fill ffplayer->preset */
     while(*preset != NULL){
       gtk_combo_box_text_append_text(ffplayer->preset,
@@ -704,9 +723,13 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     list_store = gtk_combo_box_get_model((GtkComboBox *) ffplayer->preset);
 
     if(gtk_tree_model_get_iter_first(list_store, &iter)){
+      guint i;
+
+      i = 0;
+      
       do{
 	gchar *str;
-	
+
 	gtk_tree_model_get(list_store, &iter,
 			   0, &str,
 			   -1);
@@ -714,12 +737,14 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
 		      str)){
 	  break;
 	}
+
+	i++;
       }while(gtk_tree_model_iter_next(list_store, &iter));
-      
-      gtk_combo_box_set_active_iter(GTK_COMBO_BOX(ffplayer->preset),
-				    &iter);
+
+      gtk_combo_box_set_active(GTK_COMBO_BOX(ffplayer->preset),
+			       i);
     }
-  
+
     /* select first instrument */
     ipatch->nth_level = 2;
     instrument = ags_playable_sublevel_names(playable);
@@ -732,7 +757,7 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     if(error != NULL){
       g_warning("%s\0", error->message);
     }
-    
+
     /* fill ffplayer->instrument */
     while(*instrument != NULL){
       gtk_combo_box_text_append_text(ffplayer->instrument,
@@ -748,6 +773,10 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
     list_store = gtk_combo_box_get_model((GtkComboBox *) ffplayer->instrument);
 
     if(gtk_tree_model_get_iter_first(list_store, &iter)){
+      guint i;
+
+      i = 0;
+      
       do{
 	gchar *str;
 
@@ -759,10 +788,12 @@ ags_ffplayer_launch_task(AgsFileLaunch *file_launch, AgsFFPlayer *ffplayer)
 		      str)){
 	  break;
 	}
+
+	i++;
       }while(gtk_tree_model_iter_next(list_store, &iter));
 
-      gtk_combo_box_set_active_iter(GTK_COMBO_BOX(ffplayer->instrument),
-				    &iter);
+      gtk_combo_box_set_active(GTK_COMBO_BOX(ffplayer->instrument),
+			       i);
     }
   }
 }
@@ -829,34 +860,39 @@ ags_ffplayer_set_pads(AgsAudio *audio, GType type,
 		      gpointer data)
 {
   AgsFFPlayer *ffplayer;
+  AgsMachine *machine;
   gboolean grow;
 
   ffplayer = AGS_FFPLAYER(audio->machine);
-
+  machine = audio->machine;
+  
   if(pads_old == pads){
     return;
   }
 
-  if(pads_old == pads)
-    return;
-  if(pads_old < pads)
+  if(pads_old < pads){
     grow = TRUE;
-  else
+  }else{
     grow = FALSE;
-
+  }
+  
   if(type == AGS_TYPE_INPUT){
     if(grow){
       /* depending on destination */
-      ags_ffplayer_input_map_recall(ffplayer, pads_old);
+      if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0){
+	ags_ffplayer_input_map_recall(ffplayer, pads_old);
+      }
     }else{
-      ffplayer->mapped_input_pad = audio->input_pads;
+      ffplayer->mapped_input_pad = pads;
     }
   }else{
     if(grow){
       /* depending on destination */
-      ags_ffplayer_output_map_recall(ffplayer, pads_old);
+      if((AGS_MACHINE_MAPPED_RECALL & (machine->flags)) != 0){
+	ags_ffplayer_output_map_recall(ffplayer, pads_old);
+      }
     }else{
-      ffplayer->mapped_output_pad = audio->output_pads;
+      ffplayer->mapped_output_pad = pads;
     }
   }
 }
@@ -866,15 +902,13 @@ ags_ffplayer_input_map_recall(AgsFFPlayer *ffplayer, guint input_pad_start)
 {
   AgsAudio *audio;
   AgsChannel *source, *current;
-
+  
   GList *list;
 
   audio = AGS_MACHINE(ffplayer)->audio;
 
   if(ffplayer->mapped_input_pad > input_pad_start){
     return;
-  }else{
-    ffplayer->mapped_input_pad = audio->input_pads;
   }
 
   source = ags_channel_nth(audio->input,
@@ -897,6 +931,24 @@ ags_ffplayer_input_map_recall(AgsFFPlayer *ffplayer, guint input_pad_start)
 
     current = current->next_pad;
   }
+  
+  /*  */
+  current = source;
+
+  while(current != NULL){
+    /* ags-play */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-play\0",
+			      0, audio->audio_channels, 
+			      current->pad, current->pad + 1,
+			      (AGS_RECALL_FACTORY_INPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+
+    current = current->next_pad;
+  }
 
   /*  */
   current = source;
@@ -906,7 +958,7 @@ ags_ffplayer_input_map_recall(AgsFFPlayer *ffplayer, guint input_pad_start)
     ags_recall_factory_create(audio,
 			      NULL, NULL,
 			      "ags-stream\0",
-			      0, audio->audio_channels, 
+			      0, audio->audio_channels,
 			      current->pad, current->pad + 1,
 			      (AGS_RECALL_FACTORY_INPUT |
 			       AGS_RECALL_FACTORY_PLAY |
@@ -916,26 +968,30 @@ ags_ffplayer_input_map_recall(AgsFFPlayer *ffplayer, guint input_pad_start)
 
     current = current->next_pad;
   }
+
+  ffplayer->mapped_input_pad = audio->input_pads;
 }
 
 void
 ags_ffplayer_output_map_recall(AgsFFPlayer *ffplayer, guint output_pad_start)
 {
   AgsAudio *audio;
-  AgsChannel *source, *input;;
+  AgsChannel *source, *input, *current;
+
+  AgsDelayAudio *recall_delay_audio;
+  AgsCountBeatsAudioRun *recall_count_beats_audio_run;
+
+  GList *list;
 
   audio = AGS_MACHINE(ffplayer)->audio;
 
   if(ffplayer->mapped_output_pad > output_pad_start){
     return;
-  }else{
-    ffplayer->mapped_output_pad = audio->output_pads;
   }
-  
+
   source = ags_channel_nth(audio->output,
 			   output_pad_start * audio->audio_channels);
 
-  
   /* remap for input */
   input = audio->input;
 
@@ -953,31 +1009,46 @@ ags_ffplayer_output_map_recall(AgsFFPlayer *ffplayer, guint output_pad_start)
 
     input = input->next_pad;
   }
-  
-  /* ags-stream */
-  ags_recall_factory_create(audio,
-			    NULL, NULL,
-			    "ags-stream\0",
-			    source->audio_channel, source->audio_channel + 1,
-			    output_pad_start, audio->output_pads,
-			    (AGS_RECALL_FACTORY_OUTPUT |
-			     AGS_RECALL_FACTORY_PLAY |
-			     AGS_RECALL_FACTORY_RECALL | 
-			     AGS_RECALL_FACTORY_ADD),
-			    0);
+
+  current = ags_channel_nth(audio->output,
+			    output_pad_start * audio->audio_channels);
+
+  while(current != NULL){
+    /* ags-stream */
+    ags_recall_factory_create(audio,
+			      NULL, NULL,
+			      "ags-stream\0",
+			      0, audio->audio_channels,
+			      current->pad, current->pad + 1,
+			      (AGS_RECALL_FACTORY_OUTPUT |
+			       AGS_RECALL_FACTORY_PLAY |
+			       AGS_RECALL_FACTORY_ADD),
+			      0);
+
+    current = current->next_pad;
+  }
+
+  ffplayer->mapped_output_pad = audio->output_pads;
 }
 
 void
 ags_ffplayer_paint(AgsFFPlayer *ffplayer)
 {
   GtkWidget *widget;
+
+  GtkStyle *ffplayer_style;
+  
   cairo_t *cr;
+
   double semi_key_height;
   guint bitmap;
   guint x[2];
   guint i, i_stop, j, j0;
-
+  
+  static const gdouble white_gc = 65535.0;
+    
   widget = (GtkWidget *) ffplayer->drawing_area;
+  ffplayer_style = gtk_widget_get_style(widget);
 
   semi_key_height = 2.0 / 3.0 * (double) ffplayer->control_height;
   bitmap = 0x52a52a; // description of the keyboard
@@ -996,24 +1067,29 @@ ags_ffplayer_paint(AgsFFPlayer *ffplayer)
 
   cr = gdk_cairo_create(widget->window);
 
-  cairo_set_source_rgb(cr, 0.015625, 0.03125, 0.21875);
+  /* clear with background color */
+  cairo_set_source_rgb(cr,
+		       ffplayer_style->bg[0].red / white_gc,
+		       ffplayer_style->bg[0].green / white_gc,
+		       ffplayer_style->bg[0].blue / white_gc);
   cairo_rectangle(cr, 0.0, 0.0, (double) widget->allocation.width, (double) widget->allocation.height);
   cairo_fill(cr);
 
+  /* draw piano */
   cairo_set_line_width(cr, 1.0);
-  cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);
+
+  cairo_set_source_rgb(cr,
+		       ffplayer_style->fg[0].red / white_gc,
+		       ffplayer_style->fg[0].green / white_gc,
+		       ffplayer_style->fg[0].blue / white_gc);
 
   if(x[0] != 0){
     j0 = (j != 0) ? j -1: 11;
 
     if(((1 << j0) & bitmap) != 0){
-      cairo_set_source_rgb(cr, 0.5, 0.5, 1.0);
-
       cairo_rectangle(cr, 0.0, 0.0, x[0], (double) semi_key_height);
       cairo_fill(cr); 	
 
-      cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);
-
       if(x[0] > ffplayer->control_width / 2){
 	cairo_move_to(cr, (double) (x[0] - ffplayer->control_width / 2),  semi_key_height);
 	cairo_line_to(cr, (double) (x[0] - ffplayer->control_width / 2), (double) ffplayer->control_height);
@@ -1039,13 +1115,9 @@ ags_ffplayer_paint(AgsFFPlayer *ffplayer)
   for(i = 0; i < i_stop; i++){
     if(((1 << j) & bitmap) != 0){
       // draw semi tone key
-      cairo_set_source_rgb(cr, 0.5, 0.5, 1.0);
-
       cairo_rectangle(cr, (double) (i * ffplayer->control_width + x[0]), 0.0, (double) ffplayer->control_width, semi_key_height);
       cairo_fill(cr); 	
 
-      cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);
-
       cairo_move_to(cr, (double) (i * ffplayer->control_width + x[0] + ffplayer->control_width / 2), semi_key_height);
       cairo_line_to(cr, (double) (i * ffplayer->control_width + x[0] + ffplayer->control_width / 2), ffplayer->control_height);
       cairo_stroke(cr);
@@ -1076,13 +1148,9 @@ ags_ffplayer_paint(AgsFFPlayer *ffplayer)
     j0 = j;
 
     if(((1 << j0) & bitmap) != 0){
-      cairo_set_source_rgb(cr, 0.5, 0.5, 1.0);
-
       cairo_rectangle(cr, (double) (widget->allocation.width - x[1]), 0.0, (double) x[1], semi_key_height);
       cairo_fill(cr); 	
 
-      cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);
-
       if(x[1] > ffplayer->control_width / 2){
 	cairo_move_to(cr, (double) (widget->allocation.width - x[1] + ffplayer->control_width / 2), semi_key_height);
 	cairo_line_to(cr, (double) (widget->allocation.width - x[1] + ffplayer->control_width / 2), ffplayer->control_height);
