@@ -26,7 +26,8 @@
 #include <ags/audio/ags_devout.h>
 
 #include <stdint.h>
-#include <emmintrin.h>
+//TODO:JK: do vector optimization
+//#include <emmintrin.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
@@ -793,13 +794,33 @@ ags_audio_signal_morph_samplerate(AgsAudioSignal *audio_signal, guint samplerate
 void
 ags_audio_signal_copy_buffer_to_buffer(signed short *destination, guint dchannels, signed short *source, guint schannels, guint size)
 {
-  signed long value;
+  guint limit;
+  guint i;
+
+  /* unrolled function */
+  if(size > 8){
+    limit = size - 8;
+  
+    for(i = 0; i < limit; i += 8){
+      *destination = 0xffff & ((signed long) ((*destination) + (*source)));
+      destination[1 * dchannels] = 0xffff & ((signed long) (destination[1 * dchannels] + source[1 * schannels]));
+      destination[2 * dchannels] = 0xffff & ((signed long) (destination[2 * dchannels] + source[2 * schannels]));
+      destination[3 * dchannels] = 0xffff & ((signed long) (destination[3 * dchannels] + source[3 * schannels]));
+      destination[4 * dchannels] = 0xffff & ((signed long) (destination[4 * dchannels] + source[4 * schannels]));
+      destination[5 * dchannels] = 0xffff & ((signed long) (destination[5 * dchannels] + source[5 * schannels]));
+      destination[6 * dchannels] = 0xffff & ((signed long) (destination[6 * dchannels] + source[6 * schannels]));
+      destination[7 * dchannels] = 0xffff & ((signed long) (destination[7 * dchannels] + source[7 * schannels]));
+
+      destination += (8 * dchannels);
+      source += (8 * schannels);
+    }
+  }else{
+    i = 0;
+  }
+  
+  for(; i < size; i++){
+    *destination = 0xffff & ((signed long) ((*destination) + (*source)));
 
-  for(; 0<size; --size){
-    //    *destination += *source;
-    value = 0xffff & ((*destination) + (*source));
-    *destination = (signed short) value;
-    
     destination += dchannels;
     source += schannels;
   }
@@ -1759,6 +1780,79 @@ ags_audio_signal_scale(AgsAudioSignal *audio_signal,
   }
 }
 
+/**
+ * ags_audio_signal_envelope:
+ * @audio_signal:
+ * @attack:
+ * @decay:
+ * @sustain:
+ * @release:
+ * @ratio:
+ *
+ * Envelope audio signal.
+ *
+ * Since: 0.6.7
+ */
+void
+ags_audio_signal_envelope(AgsAudioSignal *audio_signal,
+			  gdouble attack,
+			  gdouble decay,
+			  gdouble sustain,
+			  gdouble release,
+			  gdouble ratio)
+{
+  GList *stream;
+  
+  signed short *buffer;
+  gdouble volume;
+  guint frame_count, current_frame_count;
+  guint i, j, k;
+  
+  if(ratio == 0.0){
+    return;
+  }
+
+  stream = audio_signal->stream_beginning;
+  buffer = stream->data;
+  
+  frame_count = audio_signal->buffer_size * audio_signal->length - (audio_signal->buffer_size - audio_signal->last_frame);
+
+  for(j = 0, k = 0; j < 4; j++){
+    for(i = 0; ratio * (k + i) < (1.0 / ratio) * (frame_count / 4.0); i++, k++){
+      if(k != 0 && k % audio_signal->buffer_size == 0){
+	stream = stream->next;
+	buffer = stream->data;
+      }
+
+      switch(j){
+      case 0:
+	{
+	  volume = (1.0 / (k - i)) / (attack / (k - i));
+	}
+	break;
+      case 1:
+	{
+	  volume = (1.0 / (k - i)) / (decay / (k - i));
+	}
+	break;
+      case 2:
+	{
+	  volume = (1.0 / (k - i)) / (sustain / (k - i));
+	}
+	break;
+      case 3:
+	{
+	  volume = (1.0 / (k - i)) / (release / (k - i));
+	}
+	break;
+      }
+
+      *buffer = (signed short) (volume * buffer[0]);
+      buffer++;
+    }
+  }
+}
+
 /**
  * ags_audio_signal_new:
  * @devout: the assigned #AgsDevout
