@@ -499,10 +499,10 @@ ags_port_real_safe_read(AgsPort *port, GValue *value)
   guint overall_size;
   gpointer data;
 
-  overall_size = port->port_value_length * port->port_value_size;
-
   pthread_mutex_lock(&(port->mutex));
 
+  overall_size = port->port_value_length * port->port_value_size;
+
   if(!port->port_value_is_pointer){
     if(port->port_value_type == G_TYPE_BOOLEAN){
       g_value_set_boolean(value, port->port_value.ags_port_boolean);
@@ -511,7 +511,7 @@ ags_port_real_safe_read(AgsPort *port, GValue *value)
     }else if(port->port_value_type == G_TYPE_UINT64){
       g_value_set_uint64(value, port->port_value.ags_port_uint);
     }else if(port->port_value_type == G_TYPE_FLOAT){
-      g_value_set_float(value, (gdouble) port->port_value.ags_port_float);
+      g_value_set_float(value, port->port_value.ags_port_float);
     }else if(port->port_value_type == G_TYPE_DOUBLE){
       g_value_set_double(value, port->port_value.ags_port_double);
     }
@@ -533,7 +533,7 @@ ags_port_real_safe_read(AgsPort *port, GValue *value)
 	guint i;
 
 	for(i = 0; i < port->port_value_length; i++){
-	  port->port_value.ags_port_float_ptr[i] = ((gdouble *) data)[i];
+	  port->port_value.ags_port_float_ptr[i] = ((gfloat *) data)[i];
 	}
       }else if(port->port_value_type == G_TYPE_DOUBLE){
 	memcpy(data, port->port_value.ags_port_double_ptr, overall_size);
@@ -576,10 +576,10 @@ ags_port_real_safe_write(AgsPort *port, GValue *value)
     return;
   }
 
-  overall_size = port->port_value_length * port->port_value_size;
-
   pthread_mutex_lock(&(port->mutex));
 
+  overall_size = port->port_value_length * port->port_value_size;
+
   if(!port->port_value_is_pointer){
     if(port->port_value_type == G_TYPE_BOOLEAN){
       port->port_value.ags_port_boolean = g_value_get_boolean(value);
