@@ -1024,6 +1024,10 @@ ags_recall_disconnect_dynamic(AgsDynamicConnectable *dynamic_connectable)
 
   recall = AGS_RECALL(dynamic_connectable);
 
+  if((AGS_RECALL_DYNAMIC_CONNECTED & (recall->flags)) == 0){
+    return;
+  }
+
   /* disconnect children */
   list = recall->children;
 
@@ -1043,6 +1047,8 @@ ags_recall_disconnect_dynamic(AgsDynamicConnectable *dynamic_connectable)
 
     list = list->next;
   }
+
+  recall->flags &= (~AGS_RECALL_DYNAMIC_CONNECTED);
 }
 
 gchar*
@@ -1583,20 +1589,22 @@ ags_recall_done(AgsRecall *recall)
 void
 ags_recall_real_cancel(AgsRecall *recall)
 {
-  GList *list;
+  GList *list, *list_next;
 
   if((AGS_RECALL_TEMPLATE & (recall->flags)) != 0){
     return;
   }
 
   /* call cancel for children */
-  //  list = recall->children;
+  list = recall->children;
 
-  //  while(list != NULL){
-  //    ags_recall_cancel(AGS_RECALL(list->data));
-
-  //    list = list->next;
-  //  }
+  while(list != NULL){
+    list_next = list->next;
+    
+    ags_recall_cancel(AGS_RECALL(list->data));
+    
+    list = list_next;
+  }
 
   if((AGS_RECALL_PERSISTENT & (recall->flags)) != 0 ||
      (AGS_RECALL_PERSISTENT_PLAYBACK & (recall->flags)) != 0){
@@ -1735,7 +1743,7 @@ ags_recall_real_duplicate(AgsRecall *recall,
 				 NULL);
 
   copy = g_object_newv(G_OBJECT_TYPE(recall), *n_params, parameter);
-
+  
   ags_recall_set_flags(copy,
 		       (recall->flags & (~ (AGS_RECALL_TEMPLATE |
 					    AGS_RECALL_RUN_INITIALIZED |
@@ -2437,8 +2445,7 @@ ags_recall_child_done(AgsRecall *child,
     return;
   }
   
-  ags_recall_remove_child(parent,
-			  child);
+  ags_recall_remove(child);
 }
 
 /**
