@@ -371,7 +371,7 @@ ags_recall_ladspa_set_ports(AgsPlugin *plugin, GList *port)
   LADSPA_PortRangeHintDescriptor hint_descriptor;
 
   recall_ladspa = AGS_RECALL_LADSPA(plugin);
-
+  
   ags_ladspa_manager_load_file(recall_ladspa->filename);
   ladspa_plugin = ags_ladspa_manager_find_ladspa_plugin(recall_ladspa->filename);
   
@@ -404,6 +404,8 @@ ags_recall_ladspa_set_ports(AgsPlugin *plugin, GList *port)
 	    current = NULL;
 
 	    while(list != NULL){
+	      g_message("%s %s\0", specifier,
+			AGS_PORT(list->data)->specifier);
 	      if(!g_strcmp0(specifier,
 			    AGS_PORT(list->data)->specifier)){
 		current = list->data;
@@ -412,10 +414,10 @@ ags_recall_ladspa_set_ports(AgsPlugin *plugin, GList *port)
 
 	      list = list->next;
 	    }
-	    
-	    current->port_value.ags_port_float = plugin_descriptor->PortRangeHints[i].LowerBound;
 
-	    g_message("connecting port: %d/%d\0", i, port_count);
+	    if(current != NULL){	      
+	      g_message("connecting port: %d/%d\0", i, port_count);
+	    }
 	  }
 	}else if(LADSPA_IS_PORT_AUDIO(port_descriptor[i])){
 	  if(LADSPA_IS_PORT_INPUT(port_descriptor[i])){
@@ -443,8 +445,8 @@ ags_recall_ladspa_set_ports(AgsPlugin *plugin, GList *port)
 	  }
 	}
       }
-
-      AGS_RECALL(recall_ladspa)->port = g_list_reverse(port);
+      
+      AGS_RECALL(recall_ladspa)->port = port;
     }
   }
 }
@@ -557,7 +559,7 @@ ags_recall_ladspa_load(AgsRecallLadspa *recall_ladspa)
   void *plugin_so;
   LADSPA_Descriptor_Function ladspa_descriptor;
   LADSPA_Descriptor *plugin_descriptor;
-
+  
   /*  */
   ags_ladspa_manager_load_file(recall_ladspa->filename);
   ladspa_plugin = ags_ladspa_manager_find_ladspa_plugin(recall_ladspa->filename);
