@@ -34,6 +34,7 @@
 
 #include <ags/audio/ags_devout.h>
 #include <ags/audio/ags_audio.h>
+#include <ags/audio/ags_channel.h>
 #include <ags/audio/ags_output.h>
 #include <ags/audio/ags_input.h>
 #include <ags/audio/ags_audio_signal.h>
@@ -570,9 +571,13 @@ ags_recall_recycling_disconnect_dynamic(AgsDynamicConnectable *dynamic_connectab
 {
   AgsRecallRecycling *recall_recycling;
   GObject *gobject;
-
+  
   recall_recycling = AGS_RECALL_RECYCLING(dynamic_connectable);
 
+  if((AGS_RECALL_DYNAMIC_CONNECTED & (AGS_RECALL(recall_recycling)->flags)) == 0){
+    return;
+  }
+
   /* destination */
   if(recall_recycling->destination != NULL &&
      (AGS_RECALL_TEMPLATE & (AGS_RECALL(recall_recycling)->flags)) == 0){
@@ -660,6 +665,12 @@ ags_recall_recycling_source_add_audio_signal_callback(AgsRecycling *source,
   }
 
   recall = AGS_RECALL(recall_recycling);
+
+  //FIXME:JK: work-around for crashing while resetting link
+  if(recall->parent == NULL){
+    return;
+  }
+  
   channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
     
   pthread_mutex_lock(&(ags_application_mutex));
@@ -685,9 +696,21 @@ ags_recall_recycling_source_add_audio_signal_callback(AgsRecycling *source,
   if(recall_id->recycling_container != recall->recall_id->recycling_container){
     if(AGS_IS_INPUT(channel)){
       if(channel->link != NULL){
-	if(recall_id->recycling_container->parent != recall->recall_id->recycling_container){
-	  pthread_mutex_unlock(mutex);
-	  return;
+	if(recall->recall_id->recycling_container->parent != NULL){
+	  if(recall_id->recycling_container->parent != recall->recall_id->recycling_container){
+  	    pthread_mutex_unlock(mutex);
+	    return;
+	  }
+	}else{
+	  //	  AgsRecyclingContainer *parent_container;
+
+	  //	  parent_container = ags_recall_id_find_parent_recycling_container(AGS_AUDIO(channel->audio)->recall_id,
+	  //								   recall->recall_id->recycling_container);
+
+	  //	  if(recall_id->recycling_container->parent != parent_container){
+	    pthread_mutex_unlock(mutex);
+	    return;
+	    //	  }
 	}
       }else{
 	pthread_mutex_unlock(mutex);
@@ -698,6 +721,7 @@ ags_recall_recycling_source_add_audio_signal_callback(AgsRecycling *source,
       return;
     }
   }
+
   
   if(AGS_RECALL_CHANNEL_RUN(recall->parent)->destination != NULL &&
      ags_recall_id_find_recycling_container(AGS_RECALL_CHANNEL_RUN(recall->parent)->destination->recall_id,
@@ -754,8 +778,11 @@ ags_recall_recycling_source_add_audio_signal_callback(AgsRecycling *source,
 				       //"destination\0", recall_recycling->child_destination,
 				       "source\0", audio_signal,
 				       NULL);
-    //    g_message("%s\0", G_OBJECT_TYPE_NAME(recall_audio_signal));
 
+#ifdef AGS_DEBUG
+    g_message("%s\0", G_OBJECT_TYPE_NAME(recall_audio_signal));
+#endif
+    
     ags_recall_add_child(AGS_RECALL(recall_recycling), AGS_RECALL(recall_audio_signal));
   }
   
@@ -786,7 +813,13 @@ ags_recall_recycling_source_remove_audio_signal_callback(AgsRecycling *source,
   }
 
   recall = AGS_RECALL(recall_recycling);
-  channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
+
+  //FIXME:JK: work-around for crashing while resetting link
+  if(recall->parent == NULL){
+    return;
+  }
+
+ channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
 
   pthread_mutex_lock(&(ags_application_mutex));
   
@@ -889,6 +922,12 @@ ags_recall_recycling_destination_add_audio_signal_callback(AgsRecycling *destina
   }
 
   recall = AGS_RECALL(recall_recycling);
+  
+  //FIXME:JK: work-around for crashing while resetting link
+  if(recall->parent == NULL){
+    return;
+  }
+  
   channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
 
   pthread_mutex_lock(&(ags_application_mutex));
@@ -981,7 +1020,13 @@ ags_recall_recycling_destination_remove_audio_signal_callback(AgsRecycling *dest
   }
 
   recall = AGS_RECALL(recall_recycling);
-  channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
+
+  //FIXME:JK: work-around for crashing while resetting link
+  if(recall->parent == NULL){
+    return;
+  }
+
+ channel = AGS_RECALL_CHANNEL_RUN(recall->parent)->source;
   
   pthread_mutex_lock(&(ags_application_mutex));
   
