@@ -23,6 +23,7 @@
 
 #include <ags/object/ags_marshal.h>
 #include <ags-lib/object/ags_connectable.h>
+#include <ags/object/ags_concurrent_tree.h>
 
 #include <ags/thread/ags_mutex_manager.h>
 
@@ -35,6 +36,7 @@
 
 void ags_recycling_class_init(AgsRecyclingClass *recycling_class);
 void ags_recycling_connectable_interface_init(AgsConnectableInterface *connectable);
+void ags_recycling_concurrent_tree_interface_init(AgsConcurrentTreeInterface *concurrent_tree);
 void ags_recycling_set_property(GObject *gobject,
 				guint prop_id,
 				const GValue *value,
@@ -46,6 +48,8 @@ void ags_recycling_get_property(GObject *gobject,
 void ags_recycling_init(AgsRecycling *recycling);
 void ags_recycling_connect(AgsConnectable *connectable);
 void ags_recycling_disconnect(AgsConnectable *connectable);
+pthread_mutex_t* ags_recycling_get_lock(AgsConcurrentTree *concurrent_tree);
+pthread_mutex_t* ags_recycling_get_parent_lock(AgsConcurrentTree *concurrent_tree);
 void ags_recycling_finalize(GObject *gobject);
 
 void ags_recycling_real_add_audio_signal(AgsRecycling *recycling,
@@ -83,7 +87,7 @@ static guint recycling_signals[LAST_SIGNAL];
 extern pthread_mutex_t ags_application_mutex;
 
 GType
-ags_recycling_get_type (void)
+ags_recycling_get_type(void)
 {
   static GType ags_type_recycling = 0;
 
@@ -106,6 +110,12 @@ ags_recycling_get_type (void)
       NULL, /* interface_data */
     };
 
+    static const GInterfaceInfo ags_concurrent_tree_interface_info = {
+      (GInterfaceInitFunc) ags_recycling_concurrent_tree_interface_init,
+      NULL, /* interface_finalize */
+      NULL, /* interface_data */
+    };
+
     ags_type_recycling = g_type_register_static(G_TYPE_OBJECT,
 						"AgsRecycling\0",
 						&ags_recycling_info, 0);
@@ -113,6 +123,10 @@ ags_recycling_get_type (void)
     g_type_add_interface_static(ags_type_recycling,
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
+
+    g_type_add_interface_static(ags_type_recycling,
+				AGS_TYPE_CONCURRENT_TREE,
+				&ags_concurrent_tree_interface_info);
   }
 
   return(ags_type_recycling);
@@ -215,6 +229,13 @@ ags_recycling_connectable_interface_init(AgsConnectableInterface *connectable)
   connectable->disconnect = ags_recycling_disconnect;
 }
 
+void
+ags_recycling_concurrent_tree_interface_init(AgsConcurrentTreeInterface *concurrent_tree)
+{
+  concurrent_tree->get_lock = ags_recycling_get_lock;
+  concurrent_tree->get_parent_lock = ags_recycling_get_parent_lock;
+}
+
 void
 ags_recycling_init(AgsRecycling *recycling)
 {
@@ -263,7 +284,7 @@ ags_recycling_set_property(GObject *gobject,
 			   GParamSpec *param_spec)
 {
   AgsRecycling *recycling;
-
+  
   recycling = AGS_RECYCLING(gobject);
 
   switch(prop_id){
@@ -310,9 +331,9 @@ ags_recycling_get_property(GObject *gobject,
 			   GParamSpec *param_spec)
 {
   AgsRecycling *recycling;
-
+  
   recycling = AGS_RECYCLING(gobject);
-
+  
   switch(prop_id){
   case PROP_CHANNEL:
     g_value_set_object(value, recycling->channel);
@@ -329,11 +350,58 @@ ags_recycling_get_property(GObject *gobject,
 void
 ags_recycling_connect(AgsConnectable *connectable)
 {
+  /* empty */
 }
 
 void
 ags_recycling_disconnect(AgsConnectable *connectable)
 {
+  /* empty */
+}
+
+pthread_mutex_t*
+ags_recycling_get_lock(AgsConcurrentTree *concurrent_tree)
+{
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *recycling_mutex;
+  
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+  
+  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     AGS_RECYCLING(concurrent_tree));
+
+  pthread_mutex_unlock(&ags_application_mutex);
+
+  return(recycling_mutex);
+}
+
+pthread_mutex_t*
+ags_recycling_get_parent_lock(AgsConcurrentTree *concurrent_tree)
+{
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *parent_mutex;
+  
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+
+  if(AGS_RECYCLING(concurrent_tree)->parent != NULL){
+    parent_mutex = ags_mutex_manager_lookup(mutex_manager,
+					    AGS_RECYCLING(concurrent_tree)->parent);
+  }else{
+    parent_mutex = ags_mutex_manager_lookup(mutex_manager,
+					    AGS_RECYCLING(concurrent_tree)->devout);
+  }
+  
+  pthread_mutex_unlock(&ags_application_mutex);
+
+  return(parent_mutex);
 }
 
 void
@@ -403,14 +471,47 @@ void
 ags_recycling_add_audio_signal(AgsRecycling *recycling,
 			       AgsAudioSignal *audio_signal)
 {
-  g_return_if_fail(AGS_IS_RECYCLING(recycling));
-  g_return_if_fail(AGS_IS_AUDIO_SIGNAL(audio_signal));
+  AgsMutexManager *mutex_manager;
 
+  pthread_mutex_t *recycling_mutex;
+
+  if(recycling == NULL){
+    return;
+  }
+  
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+  
+  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) recycling);
+
+  pthread_mutex_unlock(&ags_application_mutex);
+  
+  pthread_mutex_lock(recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(recycling)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+  
+  if(!AGS_IS_AUDIO_SIGNAL(audio_signal)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+
+  /* emit signal */
   g_object_ref(G_OBJECT(recycling));
   g_signal_emit(G_OBJECT(recycling),
 		recycling_signals[ADD_AUDIO_SIGNAL], 0,
 		audio_signal);
   g_object_unref(G_OBJECT(recycling));
+
+  /* release lock */
+  pthread_mutex_unlock(recycling_mutex);
 }
 
 void
@@ -435,8 +536,39 @@ void
 ags_recycling_remove_audio_signal(AgsRecycling *recycling,
 				  AgsAudioSignal *audio_signal)
 {
-  g_return_if_fail(AGS_IS_RECYCLING(recycling));
+  AgsMutexManager *mutex_manager;
 
+  pthread_mutex_t *recycling_mutex;
+
+  if(recycling == NULL){
+    return;
+  }
+  
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+  
+  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) recycling);
+
+  pthread_mutex_unlock(&ags_application_mutex);
+  
+  pthread_mutex_lock(recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(recycling)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+  
+  if(!AGS_IS_AUDIO_SIGNAL(audio_signal)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+
+  /* emit signal */
   g_object_ref((GObject *) recycling);
   g_object_ref((GObject *) audio_signal);
   g_signal_emit(G_OBJECT(recycling),
@@ -444,6 +576,9 @@ ags_recycling_remove_audio_signal(AgsRecycling *recycling,
 		audio_signal);
   g_object_unref((GObject *) audio_signal);
   g_object_unref((GObject *) recycling);
+
+  /* release lock */
+  pthread_mutex_unlock(recycling_mutex);
 }
 
 void
@@ -475,6 +610,35 @@ ags_recycling_create_audio_signal_with_defaults(AgsRecycling *recycling,
 {
   AgsAudioSignal *template;
 
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *recycling_mutex;
+
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+  
+  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) recycling);
+
+  pthread_mutex_unlock(&ags_application_mutex);
+  
+  pthread_mutex_lock(recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(recycling)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+
+  if(!AGS_IS_AUDIO_SIGNAL(audio_signal)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+
+  /* create audio signal */
   template = ags_audio_signal_get_template(recycling->audio_signal);
 
   audio_signal->delay = delay;
@@ -483,6 +647,10 @@ ags_recycling_create_audio_signal_with_defaults(AgsRecycling *recycling,
   if(template == NULL){
     ags_audio_signal_stream_resize(audio_signal,
 				   0);
+
+    /* release lock */
+    pthread_mutex_unlock(recycling_mutex);
+
     return;
   }
 
@@ -515,6 +683,9 @@ ags_recycling_create_audio_signal_with_defaults(AgsRecycling *recycling,
 
   ags_audio_signal_duplicate_stream(audio_signal,
 				    template);
+
+  /* release lock */
+  pthread_mutex_unlock(recycling_mutex);
 }
 
 /**
@@ -537,31 +708,85 @@ ags_recycling_create_audio_signal_with_frame_count(AgsRecycling *recycling,
 {
   AgsDevout *devout;
   AgsAudioSignal *template;
+
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *recycling_mutex;
+
   GList *stream, *template_stream, *template_loop;
+
   guint frames_copied;
-  guint loop_start, loop_attack;
+  guint loop_start, loop_end, loop_attack;
+  guint i, j;
   gboolean enter_loop, initial_loop;
 
-  /* some init */
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();
+  
+  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) recycling);
+
+  pthread_mutex_unlock(&ags_application_mutex);
+  
+  pthread_mutex_lock(recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(recycling)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+  
+  if(!AGS_IS_AUDIO_SIGNAL(audio_signal)){
+    pthread_mutex_unlock(recycling_mutex);
+
+    return;
+  }
+
+  /* create audio signal */
   template = ags_audio_signal_get_template(recycling->audio_signal);
 
   audio_signal->devout = template->devout;
 
-  devout = AGS_DEVOUT(audio_signal->devout);
-
   audio_signal->recycling = (GObject *) recycling;
 
+  audio_signal->samplerate = template->samplerate;
+  audio_signal->buffer_size = template->buffer_size;
+  audio_signal->resolution = template->resolution;
+
+  audio_signal->last_frame = (((guint)(delay *
+				       template->buffer_size) +
+			       attack +
+			       template->last_frame) %
+			      template->buffer_size);
+  audio_signal->loop_start = (((guint) (delay *
+					template->buffer_size) +
+			       attack +
+			       template->loop_start) %
+			      template->buffer_size);
+  audio_signal->loop_end = (((guint)(delay *
+				     template->buffer_size) +
+			     attack +
+			     template->loop_end) %
+			    template->buffer_size);
+
+  devout = AGS_DEVOUT(audio_signal->devout);
+
   /* resize */
   ags_audio_signal_stream_resize(audio_signal,
-				 (guint) ceil(((double) delay +
-					       (double) attack +
+				 (guint) ceil(((double) attack +
 					       (double) frame_count) /
 					      (double) audio_signal->buffer_size));
   
-  if(template->length == 0)
-    return;
+  if(template->length == 0){
+    /* release lock */
+    pthread_mutex_unlock(recycling_mutex);
 
-  audio_signal->last_frame = ((guint) (delay * audio_signal->buffer_size) + frame_count + attack) % audio_signal->buffer_size;
+    return;
+  }
+
+  audio_signal->last_frame = ((guint) frame_count + attack) % audio_signal->buffer_size;
 
   /* generic copying */
   stream = audio_signal->stream_beginning;
@@ -570,11 +795,13 @@ ags_recycling_create_audio_signal_with_frame_count(AgsRecycling *recycling,
   frames_copied = 0;
 
   loop_start = template->loop_start;
-
+  loop_end = template->loop_end;
+  
   initial_loop = TRUE;
 
   /* loop related copying */
-  if(frame_count >= template->loop_start){
+  if(frame_count >= template->loop_start &&
+     template->loop_start < template->loop_end){
     template_loop = g_list_nth(template->stream_beginning,
 			       (guint) floor((double)loop_start / audio_signal->buffer_size));
 
@@ -585,13 +812,16 @@ ags_recycling_create_audio_signal_with_frame_count(AgsRecycling *recycling,
   }
 
   /* the copy loops */
+  j = 0;
+  
   while(stream != NULL && template_stream != NULL && frames_copied < frame_count){
-    if(frames_copied + audio_signal->buffer_size < loop_start &&
+    if(!(enter_loop &&
+	 frames_copied + audio_signal->buffer_size > loop_start) &&
        frames_copied < frame_count){
       ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[attack]), 1,
 					     (short *) template_stream->data, 1,
 					     audio_signal->buffer_size - attack);
-
+	
       if(stream->next != NULL && attack != 0){
 	ags_audio_signal_copy_buffer_to_buffer((short *) stream->next->data, 1,
 					       &(((short *) template_stream->data)[audio_signal->buffer_size - attack]), 1,
@@ -602,34 +832,67 @@ ags_recycling_create_audio_signal_with_frame_count(AgsRecycling *recycling,
     if(enter_loop &&
        ((frames_copied > loop_start || frames_copied + audio_signal->buffer_size > loop_start) ||
 	(frames_copied < frame_count))){
-      if(template_stream == NULL){
-	template_stream = template_loop;
-      }
+      for(i = 0; i < (guint) ceil(audio_signal->buffer_size / (loop_end - loop_start)); i++, j++){
+	if(template_stream == NULL){
+	  template_stream = template_loop;
+	}
 
-      if(initial_loop &&
-	 (loop_start % audio_signal->buffer_size) == 0){
-	loop_attack = 0;
-      }else{
-	loop_attack = loop_start % audio_signal->buffer_size;
-      }
+	initial_loop = FALSE;
 
-      initial_loop = FALSE;
-
-      ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[loop_attack]), 1,
-					     &(((short *) template_stream->data)[audio_signal->buffer_size - loop_attack]), 1,
-					     audio_signal->buffer_size - loop_attack);
-      
-      if(loop_attack != 0 && stream->next != NULL){
-	ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->next->data)[loop_attack]), 1,
-					       &(((short *) template_stream->data)[audio_signal->buffer_size - loop_attack]), 1,
-					       loop_attack);
+	if((loop_end - loop_start) < audio_signal->buffer_size){
+	  if(initial_loop &&
+	     (loop_start % audio_signal->buffer_size) == 0){
+	    loop_attack = 0;
+	  }else{
+	    loop_attack = (loop_start + j * (loop_end - loop_start)) % (loop_end - loop_start);
+	  }
+	
+	  if(loop_attack + i * (loop_end - loop_start) < audio_signal->buffer_size){
+	    ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[loop_attack + i * (loop_end - loop_start)]), 1,
+						   &(((short *) template_stream->data)[loop_start % audio_signal->buffer_size]), 1,
+						   loop_end - loop_start);
+	  }else{
+	    ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[loop_attack + i * (loop_end - loop_start)]), 1,
+						   &(((short *) template_stream->data)[loop_start % audio_signal->buffer_size]), 1,
+						   audio_signal->buffer_size - (loop_attack + i * (loop_end - loop_start)));
+	  }
+	}else{
+	  if(initial_loop){
+	    if((loop_start % audio_signal->buffer_size) == 0){
+	      loop_attack = 0;
+	    }else{
+	      loop_attack = (loop_start + audio_signal->buffer_size - (loop_start % audio_signal->buffer_size)) % audio_signal->buffer_size;
+	    }
+	  }else{
+	    loop_attack = (loop_start + j * (audio_signal->buffer_size - (loop_start % audio_signal->buffer_size)) + (j - 1) * (loop_start % audio_signal->buffer_size)) % audio_signal->buffer_size;
+	  }
+	  
+	  if(initial_loop){
+	    ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[loop_attack]), 1,
+						   (short *) template_stream->data, 1,
+						   audio_signal->buffer_size - loop_attack);
+	  }else{
+	    ags_audio_signal_copy_buffer_to_buffer(&(((short *) stream->data)[loop_attack]), 1,
+						   &(((short *) template_stream->data)[loop_start % audio_signal->buffer_size]), 1,
+						   loop_start - loop_end);
+	  }
+	  
+	  if(loop_attack != 0 && stream->next != NULL){
+	    ags_audio_signal_copy_buffer_to_buffer((short *) stream->next->data, 1,
+						   &(((short *) template_stream->data)[audio_signal->buffer_size - loop_attack]), 1,
+						   loop_attack);
+	  }
+	}
       }
     }
-
+    
     stream = stream->next;
     template_stream = template_stream->next;
     frames_copied += audio_signal->buffer_size;
   }
+
+  /* release lock */
+  pthread_mutex_unlock(recycling_mutex);
 }
 
 /**
@@ -648,18 +911,74 @@ AgsRecycling*
 ags_recycling_find_next_channel(AgsRecycling *start_region, AgsRecycling *end_region,
 				GObject *prev_channel)
 {
-  AgsRecycling *recycling;
+  AgsRecycling *recycling, *end_recycling;
+  
+  AgsMutexManager *mutex_manager;
 
+  pthread_mutex_t *recycling_mutex, *start_recycling_mutex, *end_recycling_mutex;
+
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();  
+
+  start_recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+						   (GObject *) start_region);
+
+  end_recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+						 (GObject *) end_region);
+
+  pthread_mutex_unlock(&ags_application_mutex);
+
+  /* verify objects and get pointer for safe access */
+  pthread_mutex_lock(start_recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(start_region)){
+    pthread_mutex_unlock(start_recycling_mutex);
+    
+    return;
+  }
+
+  pthread_mutex_lock(start_recycling_mutex);
+
+  pthread_mutex_lock(end_recycling_mutex);
+
+  if(!AGS_IS_RECYCLING(end_region)){
+    pthread_mutex_unlock(end_recycling_mutex);
+    
+    return;
+  }
+
+  end_recycling = end_region->next;
+  
+  pthread_mutex_unlock(end_recycling_mutex);
+
+  /* find */
   recycling = start_region;
 
-  while(recycling != end_region->next){
+  while(recycling != end_recycling){
+    /* lock current */
+    pthread_mutex_lock(&ags_application_mutex);
+    
+    recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+					       (GObject *) recycling);
+
+    pthread_mutex_unlock(&ags_application_mutex);
+
+    /* check if new match */
+    pthread_mutex_lock(recycling_mutex);
+
     if(recycling->channel != prev_channel){
+      pthread_mutex_unlock(recycling_mutex);
+
       return(recycling);
     }
 
     recycling = recycling->next;
+    pthread_mutex_unlock(recycling_mutex);
   }
 
+  /* no new channel within region */
   return(NULL);
 }
 
@@ -676,29 +995,57 @@ ags_recycling_find_next_channel(AgsRecycling *start_region, AgsRecycling *end_re
  * Since: 0.4
  */
 gint
-ags_recycling_position(AgsRecycling *start_recycling, AgsRecycling *end_region,
+ags_recycling_position(AgsRecycling *start_region, AgsRecycling *end_region,
 		       AgsRecycling *recycling)
 {
   AgsRecycling *current;
+
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *current_mutex;
+
   gint position;
 
-  if(start_recycling == NULL){
+  if(start_region == NULL){
     return(-1);
   }
 
-  current = start_recycling;
+  /* lookup mutex */
+  pthread_mutex_lock(&ags_application_mutex);
+  
+  mutex_manager = ags_mutex_manager_get_instance();  
+
+  pthread_mutex_unlock(&ags_application_mutex);
+
+  /* determine position */
+  current = start_region;
   position = -1;
 
   while(current != NULL && current != end_region){
     position++;
+
+    /* lock current */
+    pthread_mutex_lock(&ags_application_mutex);
     
+    current_mutex = ags_mutex_manager_lookup(mutex_manager,
+					     (GObject *) current);
+
+    pthread_mutex_unlock(&ags_application_mutex);
+
+    /* check if new match */
+    pthread_mutex_lock(current_mutex);
+
     if(current == recycling){
+      pthread_mutex_unlock(current_mutex);
+
       return(position);
     }
 
     current = current->next;
+    pthread_mutex_unlock(current_mutex);
   }
 
+  /* no match within region */
   return(-1);
 }
 
@@ -718,7 +1065,9 @@ ags_recycling_new(GObject *devout)
   AgsRecycling *recycling;
   AgsAudioSignal *audio_signal;
 
-  recycling = (AgsRecycling *) g_object_new(AGS_TYPE_RECYCLING, NULL);
+  recycling = (AgsRecycling *) g_object_new(AGS_TYPE_RECYCLING,
+					    "devout\0", devout,
+					    NULL);
 
   audio_signal = ags_audio_signal_new(devout,
 				      (GObject *) recycling,
