@@ -713,6 +713,10 @@ ags_ipatch_info(AgsPlayable *playable,
   AgsIpatch *ipatch;
   IpatchSample *sample;
 
+  if(playable == NULL){
+    return;
+  }
+  
   ipatch = AGS_IPATCH(playable);
 
   if(ipatch->iter == NULL){
@@ -732,11 +736,15 @@ ags_ipatch_info(AgsPlayable *playable,
       *loop_end = 0;
     }
 
-    g_set_error(error,
-		AGS_PLAYABLE_ERROR,
-		AGS_PLAYABLE_ERROR_NO_SAMPLE,
-		"no sample selected for file: %s\0",
-		ipatch->filename);
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_PLAYABLE_ERROR,
+		  AGS_PLAYABLE_ERROR_NO_SAMPLE,
+		  "no sample selected for file: %s\0",
+		  ipatch->filename);
+    }
+
+    return;
   }
 
   sample = IPATCH_SAMPLE(ipatch->iter->data);
@@ -763,9 +771,13 @@ ags_ipatch_read(AgsPlayable *playable, guint channel,
   guint loop_start, loop_end;
   guint i;
   GError *this_error;
-
+  
   ipatch = AGS_IPATCH(playable);
 
+  if(ipatch == NULL){
+    return(NULL);
+  }
+
   this_error = NULL;
   ags_playable_info(playable,
 		    &channels, &frames,
@@ -773,13 +785,17 @@ ags_ipatch_read(AgsPlayable *playable, guint channel,
 		    &this_error);
 
   if(this_error != NULL){
-    g_error("%s\0", this_error->message);
+    g_warning("%s\0", this_error->message);
   }
 
   buffer = (signed short *) malloc(channels * frames * sizeof(signed short));
   
   if(ipatch->nth_level == 3){
-    sample = IPATCH_SAMPLE(ipatch->iter->data);
+    if(ipatch->iter != NULL){
+      sample = IPATCH_SAMPLE(ipatch->iter->data);
+    }else{
+      sample = NULL;
+    }
   }else{
     sample = NULL;
 
