@@ -41,9 +41,11 @@
 
 #include <ags/audio/ags_recall_id.h>
 #include <ags/audio/ags_recall_container.h>
-
 #include <ags/audio/ags_config.h>
 
+#include <ags/audio/recall/ags_delay_audio.h>
+#include <ags/audio/recall/ags_delay_audio_run.h>
+
 void ags_play_notation_audio_run_class_init(AgsPlayNotationAudioRunClass *play_notation_audio_run);
 void ags_play_notation_audio_run_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_play_notation_audio_run_dynamic_connectable_interface_init(AgsDynamicConnectableInterface *dynamic_connectable);
@@ -81,8 +83,6 @@ void ags_play_notation_audio_run_write_resolve_dependency(AgsFileLookup *file_lo
 void ags_play_notation_audio_run_read_resolve_dependency(AgsFileLookup *file_lookup,
 						       GObject *recall);
 
-extern AgsConfig *config;
-
 /**
  * SECTION:ags_play_notation_audio_run
  * @short_description: play notation
@@ -104,6 +104,7 @@ static AgsConnectableInterface* ags_play_notation_audio_run_parent_connectable_i
 static AgsDynamicConnectableInterface *ags_play_notation_audio_run_parent_dynamic_connectable_interface;
 static AgsPluginInterface *ags_play_notation_audio_run_parent_plugin_interface;
 
+extern AgsConfig *config;
 extern pthread_mutex_t ags_application_mutex;
 
 GType
@@ -655,8 +656,10 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
   AgsNotation *notation;
   AgsNote *note;
 
+  AgsDelayAudio *delay_audio;
   AgsPlayNotationAudio *play_notation_audio;
 
+  AgsThread *main_loop;
   AgsTimestampThread *timestamp_thread;
 
   AgsMutexManager *mutex_manager;
@@ -673,12 +676,18 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
   GValue value = {0,};
 
   pthread_mutex_t *audio_mutex;
-
+  pthread_mutex_t *channel_mutex;
+  pthread_mutex_t *recycling_mutex;
+  
   play_notation_audio = AGS_PLAY_NOTATION_AUDIO(AGS_RECALL_AUDIO_RUN(play_notation_audio_run)->recall_audio);
 
+  delay_audio = AGS_RECALL_AUDIO_RUN(delay_audio_run)->recall_audio;
+  
   audio = AGS_RECALL_AUDIO(play_notation_audio)->audio;
-  devout = AGS_DEVOUT(audio->devout);
 
+  /* read config and audio mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+  
   str = ags_config_get(config,
 		       AGS_CONFIG_DEVOUT,
 		       "buffer-size\0");
@@ -694,9 +703,7 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
 				NULL,
 				10);
   free(str);
-  
-  pthread_mutex_lock(&(ags_application_mutex));
-  
+
   mutex_manager = ags_mutex_manager_get_instance();
 
   audio_mutex = ags_mutex_manager_lookup(mutex_manager,
@@ -706,6 +713,7 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
 
   pthread_mutex_lock(audio_mutex);
 
+  devout = (AgsDevout *) audio->devout;
   list = audio->notation;//(GList *) g_value_get_pointer(&value);
   
   if(list == NULL){
@@ -725,14 +733,21 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
 			      audio_channel);
   }
 
-  timestamp_thread = (AgsTimestampThread *) AGS_DEVOUT_THREAD(AGS_AUDIO_LOOP(AGS_MAIN(devout->ags_main)->main_loop)->devout_thread)->timestamp_thread;
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  main_loop = AGS_MAIN(devout->ags_main)->main_loop;
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  timestamp_thread = (AgsTimestampThread *) ags_thread_find_type(main_loop,
+								 AGS_TYPE_TIMESTAMP_THREAD);
   
   //TODO:JK: make it advanced
   notation = AGS_NOTATION(g_list_nth(list, audio_channel)->data);//AGS_NOTATION(ags_notation_find_near_timestamp(list, audio_channel,
     //						   timestamp_thread->timestamp)->data);
 
   current_position = notation->notes; // start_loop
-  
+
   while(current_position != NULL){
     if(current_position != NULL){
       note = AGS_NOTE(current_position->data);
@@ -747,16 +762,37 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
 	if(selected_channel == NULL){
 	  continue;
 	}
+
+	/* lookup channel mutex */
+	pthread_mutex_lock(&(ags_application_mutex));
+
+	channel_mutex = ags_mutex_manager_lookup(mutex_manager,
+						 (GObject *) channel);
 	
+	pthread_mutex_unlock(&(ags_application_mutex));
+
 	/* recycling */
+	pthread_mutex_lock(channel_mutex);
+	
 	recycling = selected_channel->first_recycling;
 
+	pthread_mutex_unlock(channel_mutex);
+	
 	//#ifdef AGS_DEBUG	
 	g_message("playing[%u]: %u | %u\n\0", audio_channel, note->x[0], note->y);
 	//#endif
 
 	while(recycling != selected_channel->last_recycling->next){
-	  audio_signal = ags_audio_signal_new((GObject *) audio->devout,
+	  /* lookup recycling mutex */
+	  pthread_mutex_lock(&(ags_application_mutex));
+
+	  recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
+						     (GObject *) recycling);
+	
+	  pthread_mutex_unlock(&(ags_application_mutex));
+
+	  /* create audio signal */
+	  audio_signal = ags_audio_signal_new((GObject *) devout,
 					      (GObject *) recycling,
 					      (GObject *) AGS_RECALL(play_notation_audio_run)->recall_id);
 
@@ -765,19 +801,37 @@ ags_play_notation_audio_run_alloc_input_callback(AgsDelayAudioRun *delay_audio_r
 							    audio_signal,
 							    delay, attack);
 	  }else{
+	    gdouble notation_delay;
+
+	    GValue value = {0,};
+
+	    g_value_init(&value,
+			 G_TYPE_DOUBLE);
+	    ags_port_safe_read(delay_audio->notation_delay,
+			       &value);
+
+	    notation_delay = g_value_get_double(&value);
+	    
 	    ags_recycling_create_audio_signal_with_frame_count(recycling,
 							       audio_signal,
-							       samplerate /  ((double) samplerate / (double) buffer_size) * (note->x[1] - note->x[0]),
+							       ((double) samplerate / notation_delay) * (note->x[1] - note->x[0]),
 							       delay, attack);
 	  }
+	  
 	  ags_audio_signal_connect(audio_signal);
 
 	  audio_signal->stream_current = audio_signal->stream_beginning;
+	  
 	  ags_recycling_add_audio_signal(recycling,
 					 audio_signal);
 	  g_object_unref(audio_signal);
 
+	  /* iterate */
+	  pthread_mutex_lock(recycling_mutex);
+
 	  recycling = recycling->next;
+
+	  pthread_mutex_unlock(recycling_mutex);
 	}
       }else if(note->x[0] > play_notation_audio_run->count_beats_audio_run->notation_counter){
 	break;
