@@ -33,6 +33,8 @@
 #include <ags/thread/ags_task_thread.h>
 #include <ags/thread/ags_returnable_thread.h>
 #include <ags/thread/ags_devout_thread.h>
+#include <ags/thread/ags_audio_thread.h>
+#include <ags/thread/ags_channel_thread.h>
 
 void ags_file_read_thread_start(AgsFileLaunch *file_launch, AgsThread *thread);
 
@@ -43,19 +45,6 @@ void ags_file_write_thread_resolve_devout(AgsFileLookup *file_lookup,
 
 void ags_file_read_thread_pool_start(AgsFileLaunch *file_launch, AgsThreadPool *thread_pool);
 
-void ags_file_read_audio_loop_resolve_task_thread(AgsFileLookup *file_lookup,
-						  AgsAudioLoop *audio_loop);
-void ags_file_write_audio_loop_resolve_task_thread(AgsFileLookup *file_lookup,
-						   AgsAudioLoop *audio_loop);
-void ags_file_read_audio_loop_resolve_gui_thread(AgsFileLookup *file_lookup,
-						 AgsAudioLoop *audio_loop);
-void ags_file_write_audio_loop_resolve_gui_thread(AgsFileLookup *file_lookup,
-						  AgsAudioLoop *audio_loop);
-void ags_file_read_audio_loop_resolve_devout_thread(AgsFileLookup *file_lookup,
-						    AgsAudioLoop *audio_loop);
-void ags_file_write_audio_loop_resolve_devout_thread(AgsFileLookup *file_lookup,
-						     AgsAudioLoop *audio_loop);
-
 void
 ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 {
@@ -85,7 +74,12 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 
     type = g_type_from_name(type_name);
 
-    if(type == AGS_TYPE_RETURNABLE_THREAD){
+    if(g_type_is_a(type,
+		   AGS_TYPE_RETURNABLE_THREAD) ||
+       g_type_is_a(type,
+		   AGS_TYPE_AUDIO_THREAD) ||
+       g_type_is_a(type,
+		   AGS_TYPE_CHANNEL_THREAD)){
       return;
     }
 
@@ -119,11 +113,11 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
   if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(gobject->flags)))) != 0){
     //FIXME:JK: workaround file setting AGS_THREAD_RUNNING is just ignored
     if(AGS_IS_AUDIO_LOOP(gobject)){
-      file_launch = (AgsFileLaunch *) g_object_new(AGS_TYPE_FILE_LAUNCH,
-						   NULL);
-      ags_file_add_launch(file, (GObject *) file_launch);
-      g_signal_connect(G_OBJECT(file_launch), "start\0",
-		       G_CALLBACK(ags_file_read_thread_start), gobject);
+      //      file_launch = (AgsFileLaunch *) g_object_new(AGS_TYPE_FILE_LAUNCH,
+      //					   NULL);
+      //      ags_file_add_launch(file, (GObject *) file_launch);
+      //      g_signal_connect(G_OBJECT(file_launch), "start\0",
+      //	       G_CALLBACK(ags_file_read_thread_start), gobject);
     }
   }
 
@@ -146,27 +140,39 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 		     "ags-thread-list\0",
 		     16)){
 	if(AGS_IS_AUDIO_LOOP(gobject)){
+	  AgsThread *async_queue;
+	  AgsThread *devout_thread;
+	  AgsThread *gui_thread;
+	  AgsThread *timestamp_thread;
+	  AgsThread *export_thread;
+	  
 	  xmlXPathContext *xpath_context;
 	  xmlXPathObject *xpath_object;
     
 	  /* task thread */
+	  async_queue = NULL;
+	  
 	  xpath_context = xmlXPathNewContext(file->doc);
 	  xpath_context->node = child;
 	  //	  xmlXPathSetContextNode(child,
 	  //			 xpath_context);
 	  xpath_object = xmlXPathCompiledEval(xmlXPathCompile("./ags-thread[@type='AgsTaskThread']\0"),
 					      xpath_context);
-	    //xmlXPathNodeEval(child,
-	    //		  "./ags-thread[@type='AgsTaskThread']\0",
-	    //		  xpath_context);
+	  //	  xmlXPathNodeEval(child,
+	  //		   "./ags-thread[@type='AgsTaskThread']\0",
+	  //		   xpath_context);
 
 	  ags_file_read_thread(file,
 			       xpath_object->nodesetval->nodeTab[0],
-			       &(AGS_AUDIO_LOOP(gobject)->task_thread));
-	  ags_thread_add_child(gobject,
-			       AGS_AUDIO_LOOP(gobject)->task_thread);
+			       &(async_queue));
+	  AGS_AUDIO_LOOP(gobject)->async_queue = async_queue;
+	  ags_thread_add_child_extended(gobject,
+					async_queue,
+					TRUE, TRUE);
 
 	  /* devout thread */
+	  devout_thread = NULL;
+	  
 	  xpath_context = xmlXPathNewContext(file->doc);
 	  xpath_context->node = child;
 	  xpath_object = xmlXPathCompiledEval(xmlXPathCompile("./ags-thread[@type='AgsDevoutThread']\0"),
@@ -174,11 +180,14 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 
 	  ags_file_read_thread(file,
 			       xpath_object->nodesetval->nodeTab[0],
-			       &(AGS_AUDIO_LOOP(gobject)->devout_thread));
-	  ags_thread_add_child(gobject,
-			       AGS_AUDIO_LOOP(gobject)->devout_thread);
+			       &(devout_thread));
+	  ags_thread_add_child_extended(gobject,
+					devout_thread,
+					TRUE, TRUE);
 
 	  /* timestamp thread */
+	  timestamp_thread = NULL;
+	  
 	  xpath_context = xmlXPathNewContext(file->doc);
 	  xpath_context->node = child;
 	  xpath_object = xmlXPathCompiledEval(xmlXPathCompile("./ags-thread[@type='AgsDevoutThread']/ags-thread-list/ags-thread[@type='AgsTimestampThread']\0"),
@@ -186,11 +195,14 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 
 	  ags_file_read_thread(file,
 			       xpath_object->nodesetval->nodeTab[0],
-			       &(AGS_DEVOUT_THREAD(AGS_AUDIO_LOOP(gobject)->devout_thread)->timestamp_thread));
-	  ags_thread_add_child(AGS_AUDIO_LOOP(gobject)->devout_thread,
-			       AGS_DEVOUT_THREAD(AGS_AUDIO_LOOP(gobject)->devout_thread)->timestamp_thread);
+			       &(timestamp_thread));
+	  ags_thread_add_child_extended(devout_thread,
+					timestamp_thread,
+					TRUE, TRUE);
 
 	  /* gui thread */
+	  gui_thread = NULL;
+
 	  xpath_context = xmlXPathNewContext(file->doc);
 	  xpath_context->node = child;
 	  xpath_object = xmlXPathCompiledEval(xmlXPathCompile("./ags-thread[@type='AgsGuiThread']\0"),
@@ -198,9 +210,25 @@ ags_file_read_thread(AgsFile *file, xmlNode *node, AgsThread **thread)
 
 	  ags_file_read_thread(file,
 			       xpath_object->nodesetval->nodeTab[0],
-			       &(AGS_AUDIO_LOOP(gobject)->gui_thread));
-	  ags_thread_add_child(gobject,
-			       AGS_AUDIO_LOOP(gobject)->gui_thread);
+			       &(gui_thread));
+	  ags_thread_add_child_extended(gobject,
+					gui_thread,
+					TRUE, TRUE);
+
+	  /* export thread */
+	  export_thread = NULL;
+
+	  xpath_context = xmlXPathNewContext(file->doc);
+	  xpath_context->node = child;
+	  xpath_object = xmlXPathCompiledEval(xmlXPathCompile("./ags-thread[@type='AgsExportThread']\0"),
+					      xpath_context);
+
+	  ags_file_read_thread(file,
+			       xpath_object->nodesetval->nodeTab[0],
+			       &(export_thread));
+	  ags_thread_add_child_extended(gobject,
+					export_thread,
+					TRUE, TRUE);
 	}else{
 	  GList *list;
 
@@ -253,17 +281,23 @@ ags_file_read_thread_resolve_devout(AgsFileLookup *file_lookup,
 void
 ags_file_read_thread_start(AgsFileLaunch *file_launch, AgsThread *thread)
 {
-  thread->flags &= (~AGS_THREAD_RUNNING);
   ags_thread_start(thread);
 
+  /* wait thread */
   pthread_mutex_lock(thread->start_mutex);
 
-  while((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) == 0){
-    pthread_cond_wait(thread->start_cond,
-		      thread->start_mutex);
-    
+  g_atomic_int_set(&(thread->start_wait),
+		   TRUE);
+	
+  if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+     g_atomic_int_get(&(thread->start_done)) == FALSE){
+    while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	  g_atomic_int_get(&(thread->start_done)) == FALSE){
+      pthread_cond_wait(thread->start_cond,
+			thread->start_mutex);
+    }
   }
-
+	
   pthread_mutex_unlock(thread->start_mutex);
 }
 
@@ -510,7 +544,6 @@ ags_file_read_thread_pool_start(AgsFileLaunch *file_launch, AgsThreadPool *threa
 void
 ags_file_read_audio_loop(AgsFile *file, xmlNode *node, AgsAudioLoop *audio_loop)
 {
-  AgsFileLookup *file_lookup;
   xmlNode *child;
 
   audio_loop = AGS_AUDIO_LOOP(audio_loop);
@@ -523,99 +556,11 @@ ags_file_read_audio_loop(AgsFile *file, xmlNode *node, AgsAudioLoop *audio_loop)
 				   "xpath\0", g_strdup_printf("xpath=//*[@id='%s']\0", xmlGetProp(node, AGS_FILE_ID_PROP)),
 				   "reference\0", audio_loop,
 				   NULL));
-
-  /* task-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-  					       "file\0", file,
-  					       "node\0", node,
-  					       "reference\0", audio_loop,
-  					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-  		   G_CALLBACK(ags_file_read_audio_loop_resolve_task_thread), audio_loop);
-
-  /* gui-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-  					       "file\0", file,
-					       "node\0", node,
-  					       "reference\0", audio_loop,
-  					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-  		   G_CALLBACK(ags_file_read_audio_loop_resolve_gui_thread), audio_loop);
-
-  /* devout-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-  					       "file\0", file,
-  					       "node\0", node,
-  					       "reference\0", audio_loop,
-  					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-  		   G_CALLBACK(ags_file_read_audio_loop_resolve_devout_thread), audio_loop);
-}
-
-void
-ags_file_read_audio_loop_resolve_task_thread(AgsFileLookup *file_lookup,
-					     AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *xpath;
-
-  xpath = (gchar *) xmlGetProp(file_lookup->node,
-			       "task-thread\0");
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_xpath(file_lookup->file, xpath);
-
-  if(id_ref != NULL){
-    g_object_set(G_OBJECT(audio_loop),
-		 "task-thread\0", (AgsThread *) id_ref->ref,
-		 NULL);
-  }
-}
-
-void
-ags_file_read_audio_loop_resolve_gui_thread(AgsFileLookup *file_lookup,
-					    AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *xpath;
-
-  xpath = (gchar *) xmlGetProp(file_lookup->node,
-			       "gui-thread\0");
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_xpath(file_lookup->file, xpath);
-
-  if(id_ref != NULL){
-    g_object_set(G_OBJECT(audio_loop),
-		 "gui-thread\0", (AgsThread *) id_ref->ref,
-		 NULL);
-  }
-}
-
-void
-ags_file_read_audio_loop_resolve_devout_thread(AgsFileLookup *file_lookup,
-					       AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *xpath;
-
-  xpath = (gchar *) xmlGetProp(file_lookup->node,
-			       "devout-thread\0");
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_xpath(file_lookup->file, xpath);
-
-  if(id_ref != NULL){
-    g_object_set(G_OBJECT(audio_loop),
-		 "devout-thread\0", (AgsThread *) id_ref->ref,
-		 NULL);
-  }
 }
 
 xmlNode*
 ags_file_write_audio_loop(AgsFile *file, xmlNode *parent, AgsAudioLoop *audio_loop)
 {
-  AgsFileLookup *file_lookup;
   xmlNode *node;
   gchar *id;
 
@@ -636,84 +581,6 @@ ags_file_write_audio_loop(AgsFile *file, xmlNode *parent, AgsAudioLoop *audio_lo
 				   "reference\0", audio_loop,
 				   NULL));
 
-  /* task-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-					       "file\0", file,
-					       "node\0", node,
-					       "reference\0", audio_loop,
-					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-		   G_CALLBACK(ags_file_write_audio_loop_resolve_task_thread), audio_loop);
-
-  /* gui-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-					       "file\0", file,
-					       "node\0", node,
-					       "reference\0", audio_loop,
-					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-		   G_CALLBACK(ags_file_write_audio_loop_resolve_gui_thread), audio_loop);
-
-  /* devout-thread */
-  file_lookup = (AgsFileLookup *) g_object_new(AGS_TYPE_FILE_LOOKUP,
-					       "file\0", file,
-					       "node\0", node,
-					       "reference\0", audio_loop,
-					       NULL);
-  ags_file_add_lookup(file, (GObject *) file_lookup);
-  g_signal_connect(G_OBJECT(file_lookup), "resolve\0",
-		   G_CALLBACK(ags_file_write_audio_loop_resolve_devout_thread), audio_loop);
-
   xmlAddChild(parent,
 	      node);
 }
-
-void
-ags_file_write_audio_loop_resolve_task_thread(AgsFileLookup *file_lookup,
-					      AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *id;
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_reference(file_lookup->file, audio_loop->task_thread);
-
-  id = xmlGetProp(id_ref->node, AGS_FILE_ID_PROP);
-
-  xmlNewProp(file_lookup->node,
-	     "task-thread\0",
-	     g_strdup_printf("xpath=//ags-thread[@id='%s']\0", id));
-}
-
-void
-ags_file_write_audio_loop_resolve_gui_thread(AgsFileLookup *file_lookup,
-					     AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *id;
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_reference(file_lookup->file, audio_loop->gui_thread);
-
-  id = xmlGetProp(id_ref->node, AGS_FILE_ID_PROP);
-
-  xmlNewProp(file_lookup->node,
-	     "gui-thread\0",
-	     g_strdup_printf("xpath=//ags-thread[@id='%s']\0", id));
-}
-
-void
-ags_file_write_audio_loop_resolve_devout_thread(AgsFileLookup *file_lookup,
-						AgsAudioLoop *audio_loop)
-{
-  AgsFileIdRef *id_ref;
-  gchar *id;
-
-  id_ref = (AgsFileIdRef *) ags_file_find_id_ref_by_reference(file_lookup->file, audio_loop->devout_thread);
-
-  id = xmlGetProp(id_ref->node, AGS_FILE_ID_PROP);
-
-  xmlNewProp(file_lookup->node,
-	     "devout-thread\0",
-	     g_strdup_printf("xpath=//ags-thread[@id='%s']\0", id));
-}
