@@ -28,6 +28,9 @@
 #include <ags/file/ags_file.h>
 
 #include <ags/thread/ags_audio_loop.h>
+#include <ags/thread/ags_task_thread.h>
+#include <ags/thread/ags_devout_thread.h>
+#include <ags/thread/ags_export_thread.h>
 #include <ags/thread/ags_audio_thread.h>
 #include <ags/thread/ags_channel_thread.h>
 #include <ags/thread/ags_gui_thread.h>
@@ -85,12 +88,15 @@ extern void ags_thread_suspend_handler(int sig);
 
 extern AgsConfig *config;
 
-extern GtkStyle *matrix_style;
+extern GtkStyle *cell_pattern_style;
 extern GtkStyle *ffplayer_style;
-extern GtkStyle *editor_style;
-extern GtkStyle *notebook_style;
 extern GtkStyle *meter_style;
+extern GtkStyle *pattern_edit_style;
 extern GtkStyle *note_edit_style;
+extern GtkStyle *meter_style;
+extern GtkStyle *dial_style;
+extern GtkStyle *indicator_style;
+extern GtkStyle *led_style;
 
 extern AgsLadspaManager *ags_ladspa_manager;
 
@@ -282,225 +288,6 @@ void
 ags_colors_alloc()
 {
   static gboolean allocated = FALSE;
-
-  if(!allocated){
-    allocated = TRUE;
-
-    matrix_style = gtk_style_new();
-    matrix_style->fg[0].red = 120 * (65535/255);
-    matrix_style->fg[0].green = 220 * (65535/255);
-    matrix_style->fg[0].blue = 120 * (65535/255);
-    matrix_style->fg[0].pixel = (gulong)(120*65536 + 220*256 + 120);
-
-    matrix_style->light[0].red = 100 * (65535/255);
-    matrix_style->light[0].green = 200 * (65535/255);
-    matrix_style->light[0].blue = 100 * (65535/255);
-    matrix_style->light[0].pixel = (gulong)(100*65536 + 200*256 + 100);
-
-    matrix_style->bg[0].red = 0 * (65535/255);
-    matrix_style->bg[0].green = 80 * (65535/255);
-    matrix_style->bg[0].blue = 0 * (65535/255);
-    matrix_style->bg[0].pixel = (gulong)(0*65536 + 80*256 + 0);
-
-    matrix_style->bg[1].red = 120 * (65535/255);
-    matrix_style->bg[1].green = 220 * (65535/255);
-    matrix_style->bg[1].blue = 120 * (65535/255);
-    matrix_style->bg[1].pixel = (gulong)(120*65536 + 220*256 + 120);
-
-    matrix_style->bg[2].red = 120 * (65535/255);
-    matrix_style->bg[2].green = 220 * (65535/255);
-    matrix_style->bg[2].blue = 120 * (65535/255);
-    matrix_style->bg[2].pixel = (gulong)(120*65536 + 220*256 + 120);
-
-    matrix_style->base[0].red = 120 * (65535/255);
-    matrix_style->base[0].green = 220 * (65535/255);
-    matrix_style->base[0].blue = 120 * (65535/255);
-    matrix_style->base[0].pixel = (gulong)(120*65536 + 220*256 + 120);
-
-
-    /*  matrix_style->fg[1] = matrix_green;
-	matrix_style->fg[2] = matrix_green;
-	matrix_style->fg[3] = matrix_green;
-	matrix_style->fg[4] = matrix_green;
-
-	matrix_style->bg[0] = matrix_led_bg;
-	matrix_style->bg[1] = matrix_led_bg;
-	matrix_style->bg[2] = matrix_led_bg;
-	matrix_style->bg[2] = matrix_led_bg;
-	matrix_style->bg[3] = matrix_led_bg;
-	matrix_style->bg[4] = matrix_led_bg;
-    */
-
-    /* ffplayer style */
-    ffplayer_style = gtk_style_new();
-    ffplayer_style->fg[0].red = 180 * (65535/255);
-    ffplayer_style->fg[0].green = 180 * (65535/255);
-    ffplayer_style->fg[0].blue = 250 * (65535/255);
-    ffplayer_style->fg[0].pixel = (gulong)(120*65536 + 120*256 + 220);
-
-    ffplayer_style->bg[0].red = 4 * (65535/255);
-    ffplayer_style->bg[0].green = 8 * (65535/255);
-    ffplayer_style->bg[0].blue = 56 * (65535/255);
-    ffplayer_style->bg[0].pixel = (gulong)(4*65536 + 8*256 + 56);
-
-    ffplayer_style->bg[1].red = 120 * (65535/255);
-    ffplayer_style->bg[1].green = 120 *(65535/255);
-    ffplayer_style->bg[1].blue = 120 * (65535/255);
-    ffplayer_style->bg[1].pixel = (gulong)(120*65536 + 120*256 + 120);
-
-    ffplayer_style->bg[2].red = 120 * (65535/255);
-    ffplayer_style->bg[2].green = 120 *(65535/255);
-    ffplayer_style->bg[2].blue = 120 * (65535/255);
-    ffplayer_style->bg[2].pixel = (gulong)(255*65536 + 255*256 + 255);
-
-    ffplayer_style->mid[0].red = 4 * (65535/255);
-    ffplayer_style->mid[0].green = 8 * (65535/255);
-    ffplayer_style->mid[0].blue = 56 * (65535/255);
-    ffplayer_style->mid[0].pixel = (gulong)(4*65536 + 8*256 + 56);
-
-    ffplayer_style->light[0].red = 120 * (65535/255);
-    ffplayer_style->light[0].green = 120 * (65535/255);
-    ffplayer_style->light[0].blue = 220 * (65535/255);
-    ffplayer_style->light[0].pixel = (gulong)(120*65536 + 120*256 + 220);
-
-    ffplayer_style->base[0].red = 120 * (65535/255);
-    ffplayer_style->base[0].green = 120 * (65535/255);
-    ffplayer_style->base[0].blue = 220 * (65535/255);
-    ffplayer_style->base[0].pixel = (gulong)(120*65536 + 120*256 + 220);
-
-
-    /* editor style */
-    editor_style = gtk_style_new();
-    editor_style->fg[0].red = 255 * (65535/255);
-    editor_style->fg[0].green = 240 *(65535/255);
-    editor_style->fg[0].blue = 200 * (65535/255);
-    editor_style->fg[0].pixel = (gulong)(255*65536 + 240*256 + 200);
-
-    editor_style->bg[0].red = 255 * (65535/255);
-    editor_style->bg[0].green = 255 *(65535/255);
-    editor_style->bg[0].blue = 100 * (65535/255);
-    editor_style->bg[0].pixel = (gulong)(255*65536 + 255*256 + 255);
-
-    editor_style->mid[0].red = 250 * (65535/255);
-    editor_style->mid[0].green = 0 *(65535/255);
-    editor_style->mid[0].blue = 250 * (65535/255);
-    editor_style->mid[0].pixel = (gulong)(150*65536 + 150*256 + 250);
-
-
-    editor_style->base[0].red = 250 * (65535/255);
-    editor_style->base[0].green = 250 *(65535/255);
-    editor_style->base[0].blue = 250 * (65535/255);
-    editor_style->base[0].pixel = (gulong)(250*65536 + 250*256 + 250);
-
-
-    /* notebook style */
-    notebook_style = gtk_style_new();
-    notebook_style->fg[0].red = 255 * (65535/255);
-    notebook_style->fg[0].green = 255 *(65535/255);
-    notebook_style->fg[0].blue = 0 * (65535/255);
-    notebook_style->fg[0].pixel = (gulong)(255*65536 + 255*256 + 0);
-
-    notebook_style->bg[0].red = 40 * (65535/255);
-    notebook_style->bg[0].green = 40 *(65535/255);
-    notebook_style->bg[0].blue = 40 * (65535/255);
-    notebook_style->bg[0].pixel = (gulong)(40*65536 + 40*256 + 40);
-
-    notebook_style->mid[0].red = 10 * (65535/255);
-    notebook_style->mid[0].green = 20 *(65535/255);
-    notebook_style->mid[0].blue = 230 * (65535/255);
-    notebook_style->mid[0].pixel = (gulong)(10*65536 + 20*256 + 230);
-
-    notebook_style->dark[0].red = 192 * (65535/255);
-    notebook_style->dark[0].green = 192 *(65535/255);
-    notebook_style->dark[0].blue = 0 * (65535/255);
-    notebook_style->dark[0].pixel = (gulong)(150*65536 + 150*256 + 250);
-
-    notebook_style->light[0].red = 255 * (65535/255);
-    notebook_style->light[0].green = 255 *(65535/255);
-    notebook_style->light[0].blue = 63 * (65535/255);
-    notebook_style->light[0].pixel = (gulong)(255*65536 + 255*256 + 63);
-
-    notebook_style->base[0].red = 0 * (65535/255);
-    notebook_style->base[0].green = 0 *(65535/255);
-    notebook_style->base[0].blue = 0 * (65535/255);
-    notebook_style->base[0].pixel = (gulong)(0*65536 + 0*256 + 0);
-
-
-    /* ruler style */
-    //TODO:JK: deprecated
-    //    ruler_style = gtk_style_new();
-    //    ruler_style->fg[0].red = 255 * (65535/255);
-    //    ruler_style->fg[0].green = 240 *(65535/255);
-    //    ruler_style->fg[0].blue = 200 * (65535/255);
-    //    ruler_style->fg[0].pixel = (gulong)(255*65536 + 240*256 + 200);
-
-    //    ruler_style->bg[0].red = 40 * (65535/255);
-    //    ruler_style->bg[0].green = 40 *(65535/255);
-    //    ruler_style->bg[0].blue = 40 * (65535/255);
-    //    ruler_style->bg[0].pixel = (gulong)(40*65536 + 40*256 + 40);
-
-    //    ruler_style->mid[0].red = 250 * (65535/255);
-    //    ruler_style->mid[0].green = 0 *(65535/255);
-    //    ruler_style->mid[0].blue = 250 * (65535/255);
-    //    ruler_style->mid[0].pixel = (gulong)(150*65536 + 150*256 + 250);
-
-    //    ruler_style->base[0].red = 250 * (65535/255);
-    //    ruler_style->base[0].green = 250 *(65535/255);
-    //    ruler_style->base[0].blue = 250 * (65535/255);
-    //    ruler_style->base[0].pixel = (gulong)(250*65536 + 250*256 + 250);
-
-
-    /* meter style */
-    meter_style = gtk_style_new();
-    meter_style->fg[0].red = 100 * (65535/255);
-    meter_style->fg[0].green = 200 *(65535/255);
-    meter_style->fg[0].blue = 255 * (65535/255);
-    meter_style->fg[0].pixel = (gulong)(100*65536 + 200*256 + 255);
-
-    meter_style->bg[0].red = 40 * (65535/255);
-    meter_style->bg[0].green = 40 *(65535/255);
-    meter_style->bg[0].blue = 40 * (65535/255);
-    meter_style->bg[0].pixel = (gulong)(40*65536 + 40*256 + 40);
-
-    meter_style->mid[0].red = 10 * (65535/255);
-    meter_style->mid[0].green = 20 *(65535/255);
-    meter_style->mid[0].blue = 230 * (65535/255);
-    meter_style->mid[0].pixel = (gulong)(10*65536 + 20*256 + 230);
-
-    meter_style->light[0].red = 0 * (65535/255);
-    meter_style->light[0].green = 150 *(65535/255);
-    meter_style->light[0].blue = 255 * (65535/255);
-    meter_style->light[0].pixel = (gulong)(150*65536 + 150*256 + 250);
-
-    meter_style->base[0].red = 0 * (65535/255);
-    meter_style->base[0].green = 0 *(65535/255);
-    meter_style->base[0].blue = 0 * (65535/255);
-    meter_style->base[0].pixel = (gulong)(0*65536 + 0*256 + 0);
-
-
-    /* note_edit style */
-    note_edit_style = gtk_style_new();
-    note_edit_style->fg[0].red = 255 * (65535/255);
-    note_edit_style->fg[0].green = 240 *(65535/255);
-    note_edit_style->fg[0].blue = 200 * (65535/255);
-    note_edit_style->fg[0].pixel = (gulong)(255*65536 + 240*256 + 200);
-
-    note_edit_style->bg[0].red = 255 * (65535/255);
-    note_edit_style->bg[0].green = 255 *(65535/255);
-    note_edit_style->bg[0].blue = 100 * (65535/255);
-    note_edit_style->bg[0].pixel = (gulong)(255*65536 + 255*256 + 255);
-
-    note_edit_style->mid[0].red = 250 * (65535/255);
-    note_edit_style->mid[0].green = 0 *(65535/255);
-    note_edit_style->mid[0].blue = 250 * (65535/255);
-    note_edit_style->mid[0].pixel = (gulong)(150*65536 + 150*256 + 250);
-
-
-    note_edit_style->base[0].red = 250 * (65535/255);
-    note_edit_style->base[0].green = 250 *(65535/255);
-    note_edit_style->base[0].blue = 250 * (65535/255);
-    note_edit_style->base[0].pixel = (gulong)(250*65536 + 250*256 + 250);
-  }
 }
 
 void
@@ -676,6 +463,7 @@ void
 ags_main_register_widget_type()
 {
   ags_dial_get_type();
+  gtk_button_get_type();
 }
 
 void
@@ -727,7 +515,12 @@ ags_main_register_thread_type()
 void
 ags_main_quit(AgsMain *ags_main)
 {
-  ags_thread_stop(AGS_AUDIO_LOOP(ags_main->main_loop)->gui_thread);
+  AgsThread *gui_thread;
+
+  gui_thread = ags_thread_find_type(ags_main->main_loop,
+				    AGS_TYPE_GUI_THREAD);
+
+  ags_thread_stop(gui_thread);
 }
 
 AgsMain*
@@ -786,7 +579,11 @@ main(int argc, char **argv)
 {
   AgsDevout *devout;
   AgsWindow *window;
-  AgsGuiThread *gui_thread;
+  AgsThread *gui_thread;
+  AgsThread *async_queue;
+  AgsThread *devout_thread;
+  AgsThread *export_thread;
+  
   GFile *autosave_file;
   struct sched_param param;
   struct rlimit rl;
@@ -874,6 +671,14 @@ main(int argc, char **argv)
   }
 #endif
 
+  /* parse gtkrc */
+  uid = getuid();
+  pw = getpwuid(uid);
+  
+  gtk_rc_parse(g_strdup_printf("%s/%s/ags.rc",
+			       pw->pw_dir,
+			       AGS_DEFAULT_DIRECTORY));
+  
   /**/
   LIBXML_TEST_VERSION;
 
@@ -918,8 +723,6 @@ main(int argc, char **argv)
   }
 
   config = ags_config_new();
-  uid = getuid();
-  pw = getpwuid(uid);
   
   wdir = g_strdup_printf("%s/%s\0",
 			 pw->pw_dir,
@@ -937,6 +740,7 @@ main(int argc, char **argv)
 
   if(filename != NULL){
     AgsFile *file;
+    AgsThread *thread;
 
     file = g_object_new(AGS_TYPE_FILE,
 			"filename\0", filename,
@@ -945,13 +749,6 @@ main(int argc, char **argv)
     ags_file_read(file);
 
     ags_main = AGS_MAIN(file->ags_main);
-    ags_file_close(file);
-
-    ags_thread_start(ags_main->main_loop);
-
-    /* complete thread pool */
-    ags_main->thread_pool->parent = AGS_THREAD(ags_main->main_loop);
-    ags_thread_pool_start(ags_main->thread_pool);
 
 #ifdef AGS_USE_TIMER
     /* Start the timer */
@@ -971,12 +768,15 @@ main(int argc, char **argv)
       exit(EXIT_FAILURE);
     }
 #endif
+
+    gui_thread = ags_thread_find_type(ags_main->main_loop,
+				      AGS_TYPE_GUI_THREAD);
     
 #ifdef _USE_PTH
-    pth_join(AGS_AUDIO_LOOP(ags_main->main_loop)->gui_thread->thread,
+    pth_join(gui_thread->thread,
 	     NULL);
 #else
-    pthread_join(*(AGS_AUDIO_LOOP(ags_main->main_loop)->gui_thread->thread),
+    pthread_join(*(gui_thread->thread),
 		 NULL);
 #endif
   }else{
@@ -1032,15 +832,59 @@ main(int argc, char **argv)
       ags_main->server = ags_server_new((GObject *) ags_main);
 
       /* AgsMainLoop */
-      ags_main->main_loop = AGS_MAIN_LOOP(ags_audio_loop_new((GObject *) devout, (GObject *) ags_main));
+      ags_main->main_loop = (AgsThread *) ags_audio_loop_new((GObject *) devout, (GObject *) ags_main);
+      ags_main->thread_pool->parent = ags_main->main_loop;
       g_object_ref(G_OBJECT(ags_main->main_loop));
       ags_connectable_connect(AGS_CONNECTABLE(G_OBJECT(ags_main->main_loop)));
 
+      /* AgsTaskThread */
+      async_queue = (AgsThread *) ags_task_thread_new(devout);
+      AGS_TASK_THREAD(async_queue)->thread_pool = ags_main->thread_pool;
+      ags_main_loop_set_async_queue(AGS_MAIN_LOOP(ags_main->main_loop),
+				    async_queue);
+      ags_thread_add_child_extended(ags_main->main_loop,
+				    async_queue,
+				    TRUE, TRUE);
+
+      /* AgsGuiThread */
+      gui_thread = (AgsThread *) ags_gui_thread_new();
+      ags_thread_add_child_extended(ags_main->main_loop,
+				    gui_thread,
+				    TRUE, TRUE);
+
+      /* AgsDevoutThread */
+      devout_thread = (AgsThread *) ags_devout_thread_new(devout);
+      ags_thread_add_child_extended(ags_main->main_loop,
+				    devout_thread,
+				    TRUE, TRUE);
+
+      /* AgsExportThread */
+      export_thread = (AgsThread *) ags_export_thread_new(devout, NULL);
+      ags_thread_add_child_extended(ags_main->main_loop,
+				    export_thread,
+				    TRUE, TRUE);
+
       /* start thread tree */
       ags_thread_start(ags_main->main_loop);
 
+      /* wait thread */
+      pthread_mutex_lock(AGS_THREAD(ags_main->main_loop)->start_mutex);
+
+      g_atomic_int_set(&(AGS_THREAD(ags_main->main_loop)->start_wait),
+		       TRUE);
+	
+      if(g_atomic_int_get(&(AGS_THREAD(ags_main->main_loop)->start_wait)) == TRUE &&
+	 g_atomic_int_get(&(AGS_THREAD(ags_main->main_loop)->start_done)) == FALSE){
+	while(g_atomic_int_get(&(AGS_THREAD(ags_main->main_loop)->start_wait)) == TRUE &&
+	      g_atomic_int_get(&(AGS_THREAD(ags_main->main_loop)->start_done)) == FALSE){
+	  pthread_cond_wait(AGS_THREAD(ags_main->main_loop)->start_cond,
+			    AGS_THREAD(ags_main->main_loop)->start_mutex);
+	}
+      }
+	
+      pthread_mutex_unlock(AGS_THREAD(ags_main->main_loop)->start_mutex);
+
       /* complete thread pool */
-      ags_main->thread_pool->parent = AGS_THREAD(ags_main->main_loop);
       ags_thread_pool_start(ags_main->thread_pool);
     }else{
       AgsSingleThread *single_thread;
@@ -1098,14 +942,17 @@ main(int argc, char **argv)
       exit(EXIT_FAILURE);
     }
 #endif
-    
+
+    gui_thread = ags_thread_find_type(ags_main->main_loop,
+				      AGS_TYPE_GUI_THREAD);
+
     if(!single_thread){
       /* join gui thread */
 #ifdef _USE_PTH
-      pth_join(AGS_AUDIO_LOOP(ags_main->main_loop)->gui_thread->thread,
+      pth_join(gui_thread->thread,
 	       NULL);
 #else
-      pthread_join(*(AGS_AUDIO_LOOP(ags_main->main_loop)->gui_thread->thread),
+      pthread_join(*(gui_thread->thread),
 		   NULL);
 #endif
     }
