@@ -57,6 +57,7 @@ void ags_audio_loop_get_property(GObject *gobject,
 				 GParamSpec *param_spec);
 void ags_audio_loop_connect(AgsConnectable *connectable);
 void ags_audio_loop_disconnect(AgsConnectable *connectable);
+pthread_mutex_t* ags_audio_loop_get_tree_lock(AgsMainLoop *main_loop);
 void ags_audio_loop_set_async_queue(AgsMainLoop *main_loop, GObject *async_queue);
 GObject* ags_audio_loop_get_async_queue(AgsMainLoop *main_loop);
 void ags_audio_loop_set_tic(AgsMainLoop *main_loop, guint tic);
@@ -89,10 +90,6 @@ void ags_audio_loop_sync_audio_super_threaded(AgsAudioLoop *audio_loop, AgsDevou
 
 enum{
   PROP_0,
-  PROP_TASK_THREAD,
-  PROP_GUI_THREAD,
-  PROP_DEVOUT_THREAD,
-  PROP_EXPORT_THREAD,
   PROP_PLAY_RECALL,
   PROP_PLAY_CHANNEL,
   PROP_PLAY_AUDIO,
@@ -169,70 +166,6 @@ ags_audio_loop_class_init(AgsAudioLoopClass *audio_loop)
   gobject->finalize = ags_audio_loop_finalize;
 
   /* properties */
-  /**
-   * AgsAudioLoop:task-thread:
-   *
-   * The assigned #AgsTaskThread.
-   * 
-   * Since: 0.4
-   */
-  param_spec = g_param_spec_object("task-thread\0",
-				   "task thread to run\0",
-				   "The task thread to run\0",
-				   AGS_TYPE_TASK_THREAD,
-				   G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_TASK_THREAD,
-				  param_spec);
-
-  /**
-   * AgsAudioLoop:devout-thread:
-   *
-   * The assigned #AgsDevoutThread.
-   * 
-   * Since: 0.4
-   */
-  param_spec = g_param_spec_object("devout-thread\0",
-				   "devout thread to run\0",
-				   "The devout thread to run\0",
-				   AGS_TYPE_DEVOUT_THREAD,
-				   G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_DEVOUT_THREAD,
-				  param_spec);
-
-  /**
-   * AgsAudioLoop:export-thread:
-   *
-   * The assigned #AgsExportThread.
-   * 
-   * Since: 0.4
-   */
-  param_spec = g_param_spec_object("export-thread\0",
-				   "export thread to run\0",
-				   "The export thread to run\0",
-				   AGS_TYPE_EXPORT_THREAD,
-				   G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_EXPORT_THREAD,
-				  param_spec);
-
-  /**
-   * AgsAudioLoop:gui-thread:
-   *
-   * The assigned #AgsGuiThread.
-   * 
-   * Since: 0.4
-   */
-  param_spec = g_param_spec_object("gui-thread\0",
-				   "gui thread to run\0",
-				   "The gui thread to run\0",
-				   AGS_TYPE_GUI_THREAD,
-				   G_PARAM_WRITABLE);
-  g_object_class_install_property(gobject,
-				  PROP_GUI_THREAD,
-				  param_spec);
-
   /**
    * AgsAudioLoop:play-recall:
    *
@@ -302,6 +235,7 @@ ags_audio_loop_connectable_interface_init(AgsConnectableInterface *connectable)
 void
 ags_audio_loop_main_loop_interface_init(AgsMainLoopInterface *main_loop)
 {
+  main_loop->get_tree_lock = ags_audio_loop_get_tree_lock;
   main_loop->set_async_queue = ags_audio_loop_set_async_queue;
   main_loop->get_async_queue = ags_audio_loop_get_async_queue;
   main_loop->set_tic = ags_audio_loop_set_tic;
@@ -314,13 +248,14 @@ void
 ags_audio_loop_init(AgsAudioLoop *audio_loop)
 {
   AgsThread *thread;
-  AgsGuiThread *gui_thread;
 
   gchar *str0, *str1;
-  
+
+  /* calculate frequency */
   thread = (AgsThread *) audio_loop;
 
   //  thread->flags |= AGS_THREAD_WAIT_FOR_CHILDREN;
+  pthread_mutex_lock(&(ags_application_mutex));
 
   str0 = ags_config_get(config,
 			AGS_CONFIG_DEVOUT,
@@ -329,6 +264,8 @@ ags_audio_loop_init(AgsAudioLoop *audio_loop)
 			AGS_CONFIG_DEVOUT,
 			"buffer_size\0");
 
+  pthread_mutex_unlock(&(ags_application_mutex));
+
   if(str0 == NULL || str1 == NULL){
     thread->freq = AGS_AUDIO_LOOP_DEFAULT_JIFFIE;
   }else{
@@ -355,24 +292,13 @@ ags_audio_loop_init(AgsAudioLoop *audio_loop)
 
   audio_loop->ags_main = NULL;
 
-  /* AgsTaskThread */  
-  audio_loop->async_queue = 
-    audio_loop->task_thread = (AgsThread *) ags_task_thread_new(NULL);
-  ags_thread_add_child(AGS_THREAD(audio_loop), audio_loop->task_thread);
-
-  /* AgsGuiThread */
-  audio_loop->gui_thread = (AgsThread *) ags_gui_thread_new();
-  gui_thread = (AgsGuiThread *) audio_loop->gui_thread;
-  ags_thread_add_child(AGS_THREAD(audio_loop), audio_loop->gui_thread);
-
-  /* AgsDevoutThread */
-  audio_loop->devout_thread = (AgsThread *) ags_devout_thread_new(NULL);
-  ags_thread_add_child(AGS_THREAD(audio_loop), audio_loop->devout_thread);
-
-  /* AgsExportThread */
-  audio_loop->export_thread = (AgsThread *) ags_export_thread_new(NULL, NULL);
-  ags_thread_add_child(AGS_THREAD(audio_loop), audio_loop->export_thread);
+  /* tree lock mutex */
+  pthread_mutexattr_init(&(audio_loop->tree_lock_mutexattr));
+  pthread_mutexattr_settype(&(audio_loop->tree_lock_mutexattr), PTHREAD_MUTEX_RECURSIVE);
 
+  audio_loop->tree_lock = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(audio_loop->tree_lock, &(audio_loop->tree_lock_mutexattr));
+  
   /* recall mutex */
   audio_loop->recall_mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
   pthread_mutex_init(audio_loop->recall_mutex, NULL);
@@ -402,74 +328,6 @@ ags_audio_loop_set_property(GObject *gobject,
   audio_loop = AGS_AUDIO_LOOP(gobject);
 
   switch(prop_id){
-  case PROP_TASK_THREAD:
-    {
-      AgsThread *thread;
-
-      thread = (AgsThread *) g_value_get_object(value);
-
-      if(audio_loop->task_thread != NULL){
-	g_object_unref(G_OBJECT(audio_loop->task_thread));
-      }
-
-      if(thread != NULL){
-	g_object_ref(G_OBJECT(thread));
-      }
-
-      audio_loop->task_thread = thread;
-    }
-    break;
-  case PROP_GUI_THREAD:
-    {
-      AgsThread *thread;
-
-      thread = (AgsThread *) g_value_get_object(value);
-
-      if(audio_loop->gui_thread != NULL){
-	g_object_unref(G_OBJECT(audio_loop->gui_thread));
-      }
-
-      if(thread != NULL){
-	g_object_ref(G_OBJECT(thread));
-      }
-
-      audio_loop->gui_thread = thread;
-    }
-    break;
-  case PROP_DEVOUT_THREAD:
-    {
-      AgsThread *thread;
-
-      thread = (AgsThread *) g_value_get_object(value);
-
-      if(audio_loop->devout_thread != NULL){
-	g_object_unref(G_OBJECT(audio_loop->devout_thread));
-      }
-
-      if(thread != NULL){
-	g_object_ref(G_OBJECT(thread));
-      }
-
-      audio_loop->devout_thread = thread;
-    }
-    break;
-  case PROP_EXPORT_THREAD:
-    {
-      AgsThread *thread;
-
-      thread = (AgsThread *) g_value_get_object(value);
-
-      if(audio_loop->export_thread != NULL){
-	g_object_unref(G_OBJECT(audio_loop->export_thread));
-      }
-
-      if(thread != NULL){
-	g_object_ref(G_OBJECT(thread));
-      }
-
-      audio_loop->export_thread = thread;
-    }
-    break;
   case PROP_PLAY_RECALL:
     {
       AgsRecall *recall;
@@ -523,26 +381,6 @@ ags_audio_loop_get_property(GObject *gobject,
   audio_loop = AGS_AUDIO_LOOP(gobject);
 
   switch(prop_id){
-  case PROP_TASK_THREAD:
-    {
-      g_value_set_object(value, audio_loop->task_thread);
-    }
-    break;
-  case PROP_GUI_THREAD:
-    {
-      g_value_set_object(value, audio_loop->gui_thread);
-    }
-    break;
-  case PROP_DEVOUT_THREAD:
-    {
-      g_value_set_object(value, audio_loop->devout_thread);
-    }
-    break;
-  case PROP_EXPORT_THREAD:
-    {
-      g_value_set_object(value, audio_loop->export_thread);
-    }
-    break;
   case PROP_PLAY_RECALL:
     {
       g_value_set_pointer(value, audio_loop->play_recall);
@@ -580,6 +418,12 @@ ags_audio_loop_disconnect(AgsConnectable *connectable)
   /* empty */
 }
 
+pthread_mutex_t*
+ags_audio_loop_get_tree_lock(AgsMainLoop *main_loop)
+{
+  return(AGS_AUDIO_LOOP(main_loop)->tree_lock);
+}
+
 void
 ags_audio_loop_set_async_queue(AgsMainLoop *main_loop, GObject *async_queue)
 {
@@ -648,10 +492,6 @@ ags_audio_loop_start(AgsThread *thread)
   if((AGS_THREAD_SINGLE_LOOP & (thread->flags)) == 0){
     /*  */
     AGS_THREAD_CLASS(ags_audio_loop_parent_class)->start(thread);
-
-    /*  */
-    ags_thread_start(audio_loop->task_thread);
-    ags_thread_start(audio_loop->gui_thread);
   }
 }
 
@@ -659,9 +499,81 @@ void
 ags_audio_loop_run(AgsThread *thread)
 {
   AgsAudioLoop *audio_loop;
-
+  AgsMutexManager *mutex_manager;
+  
   guint val;
+  guint audio_loop_flags;
+  guint play_recall_ref;
+  guint play_channel_ref;
+  guint play_audio_ref;
+  guint play_notation_ref;
+  
+  pthread_mutex_t *mutex;
 
+  /* lookup thread mutex */
+  pthread_mutex_lock(&(ags_application_mutex));
+    
+  mutex_manager = ags_mutex_manager_get_instance();
+
+  mutex = ags_mutex_manager_lookup(mutex_manager,
+				   thread);
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* start other threads */
+  if((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) != 0){
+    AgsThread *async_queue;
+    AgsThread *gui_thread;
+
+    /* get async queue and gui thread */
+    pthread_mutex_lock(mutex);
+    
+    async_queue = ags_main_loop_get_async_queue(AGS_MAIN_LOOP(thread));
+
+    pthread_mutex_unlock(mutex);
+
+    gui_thread = ags_thread_find_type(thread,
+				      AGS_TYPE_GUI_THREAD);
+    
+    /*  */    
+    g_atomic_int_set(&(async_queue->start_wait),
+		     TRUE);
+
+    g_atomic_int_set(&(gui_thread->start_wait),
+		     TRUE);
+
+    ags_thread_start(async_queue);
+    ags_thread_start(gui_thread);
+
+    /* wait thread */
+    pthread_mutex_lock(async_queue->start_mutex);
+	
+    if(g_atomic_int_get(&(async_queue->start_wait)) == TRUE &&
+       g_atomic_int_get(&(async_queue->start_done)) == FALSE){
+      while(g_atomic_int_get(&(async_queue->start_wait)) == TRUE &&
+	    g_atomic_int_get(&(async_queue->start_done)) == FALSE){
+	pthread_cond_wait(async_queue->start_cond,
+			  async_queue->start_mutex);
+      }
+    }
+	
+    pthread_mutex_unlock(async_queue->start_mutex);
+
+    /* wait thread */
+    pthread_mutex_lock(gui_thread->start_mutex);
+	
+    if(g_atomic_int_get(&(gui_thread->start_wait)) == TRUE &&
+       g_atomic_int_get(&(gui_thread->start_done)) == FALSE){
+      while(g_atomic_int_get(&(gui_thread->start_wait)) == TRUE &&
+	    g_atomic_int_get(&(gui_thread->start_done)) == FALSE){
+	pthread_cond_wait(gui_thread->start_cond,
+			  gui_thread->start_mutex);
+      }
+    }
+	
+    pthread_mutex_unlock(gui_thread->start_mutex);
+  }
+  
   if(!thread->rt_setup){
     struct sched_param param;
     
@@ -679,83 +591,87 @@ ags_audio_loop_run(AgsThread *thread)
   //  thread->freq = AGS_DEVOUT(thread->devout)->delay[AGS_DEVOUT(thread->devout)->tic_counter] / AGS_DEVOUT(thread->devout)->delay_factor;
   audio_loop = AGS_AUDIO_LOOP(thread);
 
+  pthread_mutex_lock(mutex);
+
+  audio_loop_flags = audio_loop->flags;
+  
+  pthread_mutex_unlock(mutex);
+  
   pthread_mutex_lock(audio_loop->recall_mutex);
 
   /* play recall */
-  if((AGS_AUDIO_LOOP_PLAY_RECALL & (audio_loop->flags)) != 0){
+  if((AGS_AUDIO_LOOP_PLAY_RECALL & (audio_loop_flags)) != 0){
     ags_audio_loop_play_recall(audio_loop);
 
+    pthread_mutex_lock(mutex);
+    
     if(audio_loop->play_recall_ref == 0){
       audio_loop->flags &= (~AGS_AUDIO_LOOP_PLAY_RECALL);
     }
+
+    pthread_mutex_unlock(mutex);
   }
 
   /* play channel */
-  if((AGS_AUDIO_LOOP_PLAY_CHANNEL & (audio_loop->flags)) != 0){
+  if((AGS_AUDIO_LOOP_PLAY_CHANNEL & (audio_loop_flags)) != 0){
     ags_audio_loop_play_channel(audio_loop);
 
+    pthread_mutex_lock(mutex);
+
     if(audio_loop->play_channel_ref == 0){
       audio_loop->flags &= (~AGS_AUDIO_LOOP_PLAY_CHANNEL);
     }
+
+    pthread_mutex_unlock(mutex);
   }
 
   /* play audio */
-  if((AGS_AUDIO_LOOP_PLAY_AUDIO & (audio_loop->flags)) != 0){
+  if((AGS_AUDIO_LOOP_PLAY_AUDIO & (audio_loop_flags)) != 0){
     ags_audio_loop_play_audio(audio_loop);
 
+    pthread_mutex_lock(mutex);
+
     if(audio_loop->play_audio_ref == 0){
       audio_loop->flags &= (~AGS_AUDIO_LOOP_PLAY_AUDIO);
     }
+
+    pthread_mutex_unlock(mutex);
   }
   
   pthread_mutex_unlock(audio_loop->recall_mutex);
 
   /* decide if we stop */
-  if(audio_loop->play_recall_ref == 0 &&
-     audio_loop->play_channel_ref == 0 &&
-     audio_loop->play_audio_ref == 0 &&
-     audio_loop->play_notation_ref == 0){
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(AGS_THREAD(audio_loop->devout_thread)->flags)))) != 0){
-      ags_thread_stop(AGS_THREAD(audio_loop->devout_thread));
+  pthread_mutex_lock(mutex);
 
-      if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(AGS_THREAD(audio_loop->export_thread)->flags)))) != 0){
-	ags_thread_stop(AGS_THREAD(audio_loop->export_thread));
-      }
+  play_recall_ref = audio_loop->play_recall_ref;
+  play_channel_ref = audio_loop->play_channel_ref;
+  play_audio_ref = audio_loop->play_audio_ref;
+  play_notation_ref = audio_loop->play_notation_ref;
+  
+  pthread_mutex_unlock(mutex);
+  
+  if(play_recall_ref == 0 &&
+     play_channel_ref == 0 &&
+     play_audio_ref == 0 &&
+     play_notation_ref == 0){
+    AgsThread *devout_thread;
+    AgsThread *export_thread;
+
+    devout_thread = ags_thread_find_type(thread,
+					 AGS_TYPE_DEVOUT_THREAD);
+
+    export_thread = ags_thread_find_type(thread,
+					 AGS_TYPE_EXPORT_THREAD);
+
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_thread->flags)))) != 0){
+      ags_thread_stop(devout_thread);
+    }
+
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(export_thread->flags)))) != 0){
+      ags_thread_stop(export_thread);
     }
   }
-
-  /* wait for task thread */
-  pthread_mutex_lock(audio_loop->task_thread->start_mutex);
-
-  val = g_atomic_int_get(&(AGS_THREAD(audio_loop->task_thread)->flags));
-
-  if((AGS_THREAD_INITIAL_RUN & val) != 0){
-    while((AGS_THREAD_INITIAL_RUN & val) != 0){
-      pthread_cond_wait(audio_loop->task_thread->start_cond,
-			audio_loop->task_thread->start_mutex);
-
-      val = g_atomic_int_get(&(AGS_THREAD(audio_loop->task_thread)->flags));
-    }
-  }
-
-  pthread_mutex_unlock(audio_loop->task_thread->start_mutex);
-
-  /* wait for gui thread */
-  pthread_mutex_lock(audio_loop->gui_thread->start_mutex);
-
-  val = g_atomic_int_get(&(AGS_THREAD(audio_loop->task_thread)->flags));
-
-  if((AGS_THREAD_INITIAL_RUN & val) != 0){
-    while((AGS_THREAD_INITIAL_RUN & val) != 0){
-      pthread_cond_wait(audio_loop->gui_thread->start_cond,
-			audio_loop->gui_thread->start_mutex);
-
-      val = g_atomic_int_get(&(AGS_THREAD(audio_loop->gui_thread)->flags));
-    }
-  }
-
-  pthread_mutex_unlock(audio_loop->gui_thread->start_mutex);
-
+  
   pango_fc_font_map_cache_clear(pango_cairo_font_map_get_default());
   pango_cairo_font_map_set_default(NULL);
   //  cairo_debug_reset_static_data();
@@ -783,9 +699,21 @@ ags_audio_loop_play_recall(AgsAudioLoop *audio_loop)
   GList *list, *list_next;
   guint stage;
 
+  pthread_mutex_t *mutex;
   pthread_mutex_t *devout_mutex, *audio_mutex;
 
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  mutex_manager = ags_mutex_manager_get_instance();
+
+  mutex = ags_mutex_manager_lookup(mutex_manager,
+				   (GObject *) audio_loop);
+
+  pthread_mutex_unlock(&(ags_application_mutex));
+
   /*  */
+  pthread_mutex_lock(mutex);
+
   audio_loop->flags |= AGS_AUDIO_LOOP_PLAYING_RECALL;
   stage = 0;
 
@@ -802,13 +730,10 @@ ags_audio_loop_play_recall(AgsAudioLoop *audio_loop)
     }
   }
 
+  //FIXME:JK: probably missing else
   audio_loop->flags &= (~AGS_AUDIO_LOOP_PLAY_RECALL_TERMINATING);
-
-  pthread_mutex_lock(&(ags_application_mutex));
-
-  mutex_manager = ags_mutex_manager_get_instance();
-
-  pthread_mutex_unlock(&(ags_application_mutex));
+  
+  pthread_mutex_unlock(mutex);
 
   while(list != NULL){
     AgsAudio *audio;
@@ -837,50 +762,63 @@ ags_audio_loop_play_recall(AgsAudioLoop *audio_loop)
       audio = (AgsAudio *) source->audio;
     }
     
-    /*  */
-    pthread_mutex_lock(&(ags_application_mutex));
+    /*  */    
+    if(audio != NULL){
+      pthread_mutex_lock(&(ags_application_mutex));
 
-    devout_mutex = ags_mutex_manager_lookup(mutex_manager,
+      devout_mutex = ags_mutex_manager_lookup(mutex_manager,
 					    (GObject *) recall->devout);
-    
-    if(audio != NULL){      
+      
       audio_mutex = ags_mutex_manager_lookup(mutex_manager,
 					     (GObject *) audio);
-    }
 
-    //    pthread_mutex_lock(devout_mutex);
-
-    if(audio != NULL){      
-      //      pthread_mutex_lock(audio_mutex);
-    }
+      pthread_mutex_unlock(&(ags_application_mutex));
+      
+      pthread_mutex_lock(devout_mutex);
+      pthread_mutex_lock(audio_mutex);
     
-    pthread_mutex_unlock(&(ags_application_mutex));
-    
-    //    recall_id = devout_play->recall_id;
-    list_next = list->next;
+      //    recall_id = devout_play->recall_id;
+      list_next = list->next;
 
-    if((AGS_RECALL_REMOVE & (recall->flags)) == 0){
-      if((AGS_RECALL_HIDE & (recall->flags)) == 0){
-	if(stage == 0){
-	  ags_recall_run_pre(recall);
-	}else if(stage == 1){
-	  ags_recall_run_inter(recall);
-	}else{
-	  ags_recall_run_post(recall);
+      if((AGS_RECALL_REMOVE & (recall->flags)) == 0){
+	if((AGS_RECALL_HIDE & (recall->flags)) == 0){
+	  if(stage == 0){
+	    ags_recall_run_pre(recall);
+	  }else if(stage == 1){
+	    ags_recall_run_inter(recall);
+	  }else{
+	    ags_recall_run_post(recall);
+	  }
 	}
+      }else{
+	ags_audio_loop_remove_recall(audio_loop, devout_play);
+	g_object_unref(recall);
+	free(devout_play);
       }
+      
+      pthread_mutex_unlock(audio_mutex);
+      pthread_mutex_unlock(devout_mutex);
     }else{
-      ags_audio_loop_remove_recall(audio_loop, devout_play);
-      g_object_unref(recall);
-      free(devout_play);
-    }
-    
-    if(audio != NULL){
-      //      pthread_mutex_unlock(audio_mutex);
-    }
-    
-    //    pthread_mutex_unlock(devout_mutex);
+      /* lock free */
+      list_next = list->next;
 
+      if((AGS_RECALL_REMOVE & (recall->flags)) == 0){
+	if((AGS_RECALL_HIDE & (recall->flags)) == 0){
+	  if(stage == 0){
+	    ags_recall_run_pre(recall);
+	  }else if(stage == 1){
+	    ags_recall_run_inter(recall);
+	  }else{
+	    ags_recall_run_post(recall);
+	  }
+	}
+      }else{
+	ags_audio_loop_remove_recall(audio_loop, devout_play);
+	g_object_unref(recall);
+	free(devout_play);
+      }
+    }
+      
     list = list_next;
   }
 
@@ -1027,90 +965,100 @@ ags_audio_loop_play_channel(AgsAudioLoop *audio_loop)
 void
 ags_audio_loop_play_channel_super_threaded(AgsAudioLoop *audio_loop, AgsDevoutPlay *devout_play)
 {
+  AgsThread *thread;
+  AgsChannelThread *channel_thread;
+  
   /* playback */
-  if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[0]->flags)))) == 0){
+  thread = devout_play->channel_thread[0];
+  channel_thread = (AgsChannelThread *) thread;
+  
+  if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
     guint val;
 
-    ags_thread_start(devout_play->channel_thread[0]);
-  
+    g_atomic_int_set(&(AGS_THREAD(thread)->start_wait),
+		     TRUE);
+
+    ags_thread_start(thread);
+
     /* wait child */
-    pthread_mutex_lock(devout_play->channel_thread[0]->start_mutex);
-
-    val = g_atomic_int_get(&(devout_play->channel_thread[0]->flags));
-
-    if((AGS_THREAD_INITIAL_RUN & val) != 0){
-      while((AGS_THREAD_INITIAL_RUN & val) != 0){
-	pthread_cond_wait(devout_play->channel_thread[0]->start_cond,
-			  devout_play->channel_thread[0]->start_mutex);
-
-	val = g_atomic_int_get(&(devout_play->channel_thread[0]->flags));
+    pthread_mutex_lock(thread->start_mutex);
+	
+    if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+       g_atomic_int_get(&(thread->start_done)) == FALSE){
+      while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	    g_atomic_int_get(&(thread->start_done)) == FALSE){
+	pthread_cond_wait(thread->start_cond,
+			  thread->start_mutex);
       }
     }
-
-    pthread_mutex_unlock(devout_play->channel_thread[0]->start_mutex);
+	
+    pthread_mutex_unlock(thread->start_mutex);
   }
 
   /* wakeup wait */
-  pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[0])->wakeup_mutex);
+  pthread_mutex_lock(channel_thread->wakeup_mutex);
 
-  g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[0])->flags),
-		  AGS_CHANNEL_THREAD_WAKEUP);
+  g_atomic_int_and(&(channel_thread->flags),
+		   (~AGS_CHANNEL_THREAD_WAIT));
 	    
-  if((AGS_CHANNEL_THREAD_WAITING & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[0])->flags)))) != 0){
-    pthread_cond_signal(AGS_CHANNEL_THREAD(devout_play->channel_thread[0])->wakeup_cond);
+  if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(channel_thread->flags)))) == 0){
+    pthread_cond_signal(channel_thread->wakeup_cond);
   }
 	    
-  pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[0])->wakeup_mutex);
+  pthread_mutex_unlock(channel_thread->wakeup_mutex);
 }
 
 void
 ags_audio_loop_sync_channel_super_threaded(AgsAudioLoop *audio_loop, AgsDevoutPlay *devout_play)
 {
+  AgsThread *thread;
+  AgsChannelThread *channel_thread;
   gboolean playback;
   
+  /* playback */
+  thread = devout_play->channel_thread[0];
+  channel_thread = (AgsChannelThread *) thread;
+  
   playback = FALSE;
 	
-  /* playback */
   if((AGS_DEVOUT_PLAY_PLAYBACK & (g_atomic_int_get(&(devout_play->flags)))) != 0){
     playback = TRUE;
 
-    pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+    pthread_mutex_lock(channel_thread->done_mutex);
   
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) != 0 &&
-       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_0 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_1 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_2 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0){
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) != 0 &&
+       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) == 0){
 
-      if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags)))) == 0){
-	g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-			AGS_CHANNEL_THREAD_NOTIFY);
-    
-	while((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags)))) == 0){
-	  pthread_cond_wait(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_cond,
-			    AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+      g_atomic_int_and(&(channel_thread->flags),
+		       (~AGS_CHANNEL_THREAD_DONE_SYNC));
+      
+      if((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	 (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+	while((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	      (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+	  pthread_cond_wait(channel_thread->done_cond,
+			    channel_thread->done_mutex);
 	}
       }
     }
 
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-		     (~AGS_CHANNEL_THREAD_NOTIFY));
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-		     (~AGS_CHANNEL_THREAD_DONE));
+    g_atomic_int_or(&(channel_thread->flags),
+		    (AGS_CHANNEL_THREAD_WAIT_SYNC |
+		     AGS_CHANNEL_THREAD_DONE_SYNC));
 
-    pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+    pthread_mutex_unlock(channel_thread->done_mutex);
   }
 
   /*  */
   if(playback && devout_play->recall_id[0] == NULL){
-    ags_thread_stop(devout_play->channel_thread[0]);
+    ags_thread_stop(thread);
     g_atomic_int_and(&(devout_play->flags),
 		     (~AGS_DEVOUT_PLAY_PLAYBACK));
   }
 
   /* remove from playback */
   if(!playback){
-    ags_thread_stop(devout_play->channel_thread[0]);
+    ags_thread_stop(thread);
     ags_audio_loop_remove_channel(audio_loop,
 				  devout_play->source);
   }
@@ -1168,7 +1116,7 @@ ags_audio_loop_play_audio(AgsAudioLoop *audio_loop)
     list_next_play_domain = list_play_domain->next;
     
     play_domain = (AgsDevoutPlayDomain *) list_play_domain->data;
-    audio = AGS_AUDIO(play_domain->domain);
+    audio = (AgsAudio *) play_domain->domain;
     
     /*  */
     pthread_mutex_lock(&(ags_application_mutex));
@@ -1218,11 +1166,15 @@ ags_audio_loop_play_audio(AgsAudioLoop *audio_loop)
 	  gboolean remove_domain;
 
 	  /* not super threaded */
-	  devout_mutex = ags_mutex_manager_lookup(mutex_manager,
-						  (GObject *) audio->devout);
-	  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) audio);
+	  pthread_mutex_lock(&(ags_application_mutex));
 	  
+	  devout_mutex = ags_mutex_manager_lookup(mutex_manager,
+	  					  (GObject *) audio->devout);
+	  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
+	  					 (GObject *) audio);
+
+	  pthread_mutex_unlock(&(ags_application_mutex));
+
 	  //	  pthread_mutex_lock(audio_mutex);
 	  //	  pthread_mutex_lock(devout_mutex);    
 
@@ -1345,80 +1297,94 @@ ags_audio_loop_play_audio_super_threaded(AgsAudioLoop *audio_loop, AgsDevoutPlay
 
   /* sequencer */
   if((AGS_DEVOUT_PLAY_DOMAIN_SEQUENCER & (g_atomic_int_get(&(devout_play_domain->flags)))) != 0){
+    AgsThread *thread;
+    AgsAudioThread *audio_thread;
+
+    thread = devout_play_domain->audio_thread[1];
+    audio_thread = (AgsAudioThread *) thread;
+    
     sequencer = TRUE;
 
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0){
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
       guint val;
-	  
-      ags_thread_start(devout_play_domain->audio_thread[1]);
-  
+
+      g_atomic_int_set(&(thread->start_wait),
+		       TRUE);
+      
+      ags_thread_start(thread);
+
       /* wait child */
-      pthread_mutex_lock(devout_play_domain->audio_thread[1]->start_mutex);
+      pthread_mutex_lock(thread->start_mutex);
 
-      val = g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags));
-
-      if((AGS_THREAD_INITIAL_RUN & val) != 0){
-	while((AGS_THREAD_INITIAL_RUN & val) != 0){
-	  pthread_cond_wait(devout_play_domain->audio_thread[1]->start_cond,
-			    devout_play_domain->audio_thread[1]->start_mutex);
-
-	  val = g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags));
+      if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	 g_atomic_int_get(&(thread->start_done)) == FALSE){
+	while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	      g_atomic_int_get(&(thread->start_done)) == FALSE){
+	  pthread_cond_wait(thread->start_cond,
+			    thread->start_mutex);
 	}
       }
-
-      pthread_mutex_unlock(devout_play_domain->audio_thread[1]->start_mutex);
+	
+      pthread_mutex_unlock(thread->start_mutex);
     }
 
     /* wakeup wait */
-    pthread_mutex_lock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->wakeup_mutex);
+    pthread_mutex_lock(audio_thread->wakeup_mutex);
 
-    g_atomic_int_or(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags),
-		    AGS_AUDIO_THREAD_WAKEUP);
+    g_atomic_int_and(&(audio_thread->flags),
+		     (~AGS_AUDIO_THREAD_WAIT));
 	    
-    if((AGS_AUDIO_THREAD_WAITING & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags)))) != 0){
-      pthread_cond_signal(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->wakeup_cond);
+    if((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
+      pthread_cond_signal(audio_thread->wakeup_cond);
     }
 	    
-    pthread_mutex_unlock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->wakeup_mutex);
+    pthread_mutex_unlock(audio_thread->wakeup_mutex);
   }
 
   /* notation */
   if((AGS_DEVOUT_PLAY_DOMAIN_NOTATION & (g_atomic_int_get(&(devout_play_domain->flags)))) != 0){
+    AgsThread *thread;
+    AgsAudioThread *audio_thread;
+
+    thread = devout_play_domain->audio_thread[2];
+    audio_thread = (AgsAudioThread *) thread;
+
     notation = TRUE;
 
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0){
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
       guint val;
 
-      ags_thread_start(devout_play_domain->audio_thread[2]);
+      g_atomic_int_set(&(thread->start_wait),
+		       TRUE);
+	
+      ags_thread_start(thread);
 
       /* wait child */
-      pthread_mutex_lock(devout_play_domain->audio_thread[2]->start_mutex);
-
-      val = g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags));
-
-      if((AGS_THREAD_INITIAL_RUN & val) != 0){
-	while((AGS_THREAD_INITIAL_RUN & val) != 0){
-	  pthread_cond_wait(devout_play_domain->audio_thread[2]->start_cond,
-			    devout_play_domain->audio_thread[2]->start_mutex);
-
-	  val = g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags));
+      pthread_mutex_lock(thread->start_mutex);
+      
+      if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	 g_atomic_int_get(&(thread->start_done)) == FALSE){
+	while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	      g_atomic_int_get(&(thread->start_done)) == FALSE){
+	  pthread_cond_wait(thread->start_cond,
+			    thread->start_mutex);
 	}
       }
-
-      pthread_mutex_unlock(devout_play_domain->audio_thread[2]->start_mutex);
+	
+      pthread_mutex_unlock(thread->start_mutex);
     }
 
     /* wakeup wait */
-    pthread_mutex_lock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->wakeup_mutex);
+    pthread_mutex_lock(audio_thread->wakeup_mutex);
 	  
-    g_atomic_int_or(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags),
-		    AGS_AUDIO_THREAD_WAKEUP);
+    g_atomic_int_and(&(audio_thread->flags),
+		     (~AGS_AUDIO_THREAD_WAIT));
 	  
-    if((AGS_AUDIO_THREAD_WAITING & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags)))) != 0){
-      pthread_cond_signal(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->wakeup_cond);
+    if((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
+      pthread_cond_signal(audio_thread->wakeup_cond);
     }
 	  
-    pthread_mutex_unlock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->wakeup_mutex);
+    pthread_mutex_unlock(audio_thread->wakeup_mutex);
   }
 }
 
@@ -1443,64 +1409,66 @@ ags_audio_loop_sync_audio_super_threaded(AgsAudioLoop *audio_loop, AgsDevoutPlay
   
   /* sequencer */
   if((AGS_DEVOUT_PLAY_DOMAIN_SEQUENCER & (g_atomic_int_get(&(devout_play_domain->flags)))) != 0){
+    AgsAudioThread *audio_thread;
+
+    audio_thread = AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1]);
     sequencer = TRUE;
 
-    pthread_mutex_lock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->done_mutex);
+    pthread_mutex_lock(audio_thread->done_mutex);
 
     if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) != 0 &&
-       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_0 & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_1 & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_2 & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0){
+       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play_domain->audio_thread[1]->flags)))) == 0){
 
-      if((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags)))) == 0){
-	g_atomic_int_or(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags),
-			AGS_AUDIO_THREAD_NOTIFY);
-    
-	while((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags)))) == 0){
-	  pthread_cond_wait(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->done_cond,
-			    AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->done_mutex);
+      g_atomic_int_and(&(audio_thread->flags),
+		       (~AGS_AUDIO_THREAD_DONE_SYNC));
+      
+      if((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0 &&
+	 (AGS_AUDIO_THREAD_WAIT_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
+	while((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0 &&
+	      (AGS_AUDIO_THREAD_WAIT_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
+	  pthread_cond_wait(audio_thread->done_cond,
+			    audio_thread->done_mutex);
 	}
       }
     }
     
-    g_atomic_int_and(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags),
-		     (~AGS_AUDIO_THREAD_NOTIFY));
-    g_atomic_int_and(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->flags),
-		     (~AGS_AUDIO_THREAD_DONE));
+    g_atomic_int_or(&(audio_thread->flags),
+		    (AGS_AUDIO_THREAD_WAIT_SYNC |
+		     AGS_AUDIO_THREAD_DONE_SYNC));
       	  
-    pthread_mutex_unlock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[1])->done_mutex);
+    pthread_mutex_unlock(audio_thread->done_mutex);
   }
 
   /* notation */
   if((AGS_DEVOUT_PLAY_DOMAIN_NOTATION & (g_atomic_int_get(&(devout_play_domain->flags)))) != 0){
+    AgsAudioThread *audio_thread;
+
+    audio_thread = AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2]);
     notation = TRUE;
 
-    pthread_mutex_lock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->done_mutex);
+    pthread_mutex_lock(audio_thread->done_mutex);
 
     if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) != 0 &&
-       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_0 & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_1 & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_2 & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0){
-      
-      if((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags)))) == 0){
-	g_atomic_int_or(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags),
-			AGS_AUDIO_THREAD_NOTIFY);
-    
-	while((AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags)))) == 0){
-	  pthread_cond_wait(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->done_cond,
-			    AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->done_mutex);
+       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play_domain->audio_thread[2]->flags)))) == 0){
+
+      g_atomic_int_and(&(audio_thread->flags),
+		       (~AGS_AUDIO_THREAD_DONE_SYNC));
+
+      if((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0 &&
+	 (AGS_AUDIO_THREAD_WAIT_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
+	while((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0 &&
+	      (AGS_AUDIO_THREAD_WAIT_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
+	  pthread_cond_wait(audio_thread->done_cond,
+			    audio_thread->done_mutex);
 	}
       }
     }
       
-    g_atomic_int_and(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags),
-		     (~AGS_AUDIO_THREAD_NOTIFY));
-    g_atomic_int_and(&(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->flags),
-		     (~AGS_AUDIO_THREAD_DONE));
-      
-    pthread_mutex_unlock(AGS_AUDIO_THREAD(devout_play_domain->audio_thread[2])->done_mutex);
+    g_atomic_int_or(&(audio_thread->flags),
+		    (AGS_AUDIO_THREAD_WAIT_SYNC |
+		     AGS_AUDIO_THREAD_DONE_SYNC));
+    
+    pthread_mutex_unlock(audio_thread->done_mutex);
   }
 
   /* check if flags are still valid */
@@ -1521,14 +1489,15 @@ ags_audio_loop_sync_audio_super_threaded(AgsAudioLoop *audio_loop, AgsDevoutPlay
     devout_play = devout_play->next;
   }
 
-  if(sequencer && !found_sequencer){
+  if(!found_sequencer){
     ags_thread_stop(devout_play_domain->audio_thread[1]);
     g_atomic_int_and(&(devout_play_domain->flags),
 		     (~AGS_DEVOUT_PLAY_DOMAIN_SEQUENCER));
   }
 
-  if(notation && !found_notation){
+  if(!found_notation){
     ags_thread_stop(devout_play_domain->audio_thread[2]);
+
     g_atomic_int_and(&(devout_play_domain->flags),
 		     (~AGS_DEVOUT_PLAY_DOMAIN_NOTATION));
   }
