@@ -309,11 +309,11 @@ void
 ags_audio_thread_start(AgsThread *thread)
 {
   /* reset status */
-  g_atomic_int_and(&(AGS_AUDIO_THREAD(thread)->flags),
-		   (~(AGS_AUDIO_THREAD_WAKEUP |
-		      AGS_AUDIO_THREAD_WAITING |
-		      AGS_AUDIO_THREAD_NOTIFY |
-  		      AGS_AUDIO_THREAD_DONE)));
+  g_atomic_int_or(&(AGS_AUDIO_THREAD(thread)->flags),
+		   (AGS_AUDIO_THREAD_WAIT |
+		    AGS_AUDIO_THREAD_DONE |
+		    AGS_AUDIO_THREAD_WAIT_SYNC |
+		    AGS_AUDIO_THREAD_DONE_SYNC));
 
   AGS_THREAD_CLASS(ags_audio_thread_parent_class)->start(thread);
 }
@@ -325,6 +325,7 @@ ags_audio_thread_run(AgsThread *thread)
   AgsChannel *output;
   AgsDevoutPlayDomain *devout_play_domain;
   AgsDevoutPlay *devout_play;
+  AgsRecallID *recall_id[3];
   
   AgsMutexManager *mutex_manager;
   AgsAudioThread *audio_thread;
@@ -332,6 +333,7 @@ ags_audio_thread_run(AgsThread *thread)
   gint stage;
   
   pthread_mutex_t *audio_mutex;
+  pthread_mutex_t *output_mutex;
 
   if(!thread->rt_setup){
     struct sched_param param;
@@ -346,9 +348,29 @@ ags_audio_thread_run(AgsThread *thread)
     thread->rt_setup = TRUE;
   }
 
+  audio_thread = AGS_AUDIO_THREAD(thread);
+
+  if((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) != 0){
+    pthread_mutex_lock(audio_thread->done_mutex);
+
+    g_atomic_int_and(&(audio_thread->flags),
+		     (~AGS_AUDIO_THREAD_WAIT_SYNC));
+
+    pthread_mutex_unlock(audio_thread->done_mutex);
+
+    pthread_mutex_lock(audio_thread->wakeup_mutex);
+    
+    g_atomic_int_or(&(audio_thread->flags),
+		    (AGS_AUDIO_THREAD_WAIT |
+		   AGS_AUDIO_THREAD_DONE));
+  
+    pthread_mutex_unlock(audio_thread->wakeup_mutex);
+    
+    return;
+  }
+  
   //  thread->freq = AGS_DEVOUT(thread->devout)->delay[AGS_DEVOUT(thread->devout)->tic_counter] / AGS_DEVOUT(thread->devout)->delay_factor;
   
-  audio_thread = AGS_AUDIO_THREAD(thread);
   audio = audio_thread->audio;
   devout_play_domain = audio->devout_play_domain;
   
@@ -357,52 +379,77 @@ ags_audio_thread_run(AgsThread *thread)
   /* start - wait until signaled */
   pthread_mutex_lock(audio_thread->wakeup_mutex);
 
-  if((AGS_AUDIO_THREAD_WAKEUP & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
-    g_atomic_int_or(&(audio_thread->flags),
-		    AGS_AUDIO_THREAD_WAITING);
-    
-    while((AGS_AUDIO_THREAD_WAKEUP & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
+  g_atomic_int_and(&(audio_thread->flags),
+		   (~AGS_AUDIO_THREAD_DONE));
+  
+  if((AGS_AUDIO_THREAD_WAIT & (g_atomic_int_get(&(audio_thread->flags)))) != 0 &&
+     (AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
+    while((AGS_AUDIO_THREAD_WAIT & (g_atomic_int_get(&(audio_thread->flags)))) != 0 &&
+	  (AGS_AUDIO_THREAD_DONE & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
       pthread_cond_wait(audio_thread->wakeup_cond,
 			audio_thread->wakeup_mutex);
     }
   }
   
-  g_atomic_int_and(&(audio_thread->flags),
-		   (~AGS_AUDIO_THREAD_WAITING));
-  g_atomic_int_and(&(audio_thread->flags),
-		   (~AGS_AUDIO_THREAD_WAKEUP));
+  g_atomic_int_or(&(audio_thread->flags),
+		  (AGS_AUDIO_THREAD_WAIT |
+		   AGS_AUDIO_THREAD_DONE));
   
   pthread_mutex_unlock(audio_thread->wakeup_mutex);
 
-  /* audio mutex */
+  /* lookup audio mutex */
   pthread_mutex_lock(&(ags_application_mutex));
 
   mutex_manager = ags_mutex_manager_get_instance();
+
   audio_mutex = ags_mutex_manager_lookup(mutex_manager,
 					 (GObject *) audio);
       
   pthread_mutex_unlock(&(ags_application_mutex));
 
   /* do audio processing */
+  pthread_mutex_lock(audio_mutex);
+  
   output = audio->output;
 
+  pthread_mutex_unlock(audio_mutex);
+  
   while(output != NULL){
-    devout_play = output->devout_play;
+    /* lookup output mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    mutex_manager = ags_mutex_manager_get_instance();
+
+    output_mutex = ags_mutex_manager_lookup(mutex_manager,
+					    (GObject *) output);
       
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get devout play */
+    pthread_mutex_lock(output_mutex);
+    
+    devout_play = output->devout_play;
+
+    recall_id[0] = devout_play->recall_id[0];
+    recall_id[1] = devout_play->recall_id[1];
+    recall_id[2] = devout_play->recall_id[2];
+    
+    pthread_mutex_unlock(output_mutex);
+
     if((AGS_DEVOUT_PLAY_SUPER_THREADED_CHANNEL & (g_atomic_int_get(&(devout_play->flags)))) != 0){
       ags_audio_thread_play_channel_super_threaded(audio_thread, devout_play);
     }else{
       for(stage = 0; stage < 3; stage++){
 	if(audio_thread == devout_play_domain->audio_thread[0]){
-	  ags_channel_recursive_play(output, devout_play->recall_id[0], stage);
+	  ags_channel_recursive_play(output, recall_id[0], stage);
 	}
 
 	if(audio_thread == devout_play_domain->audio_thread[1]){
-	  ags_channel_recursive_play(output, devout_play->recall_id[1], stage);
+	  ags_channel_recursive_play(output, recall_id[1], stage);
 	}
 
 	if(audio_thread == devout_play_domain->audio_thread[2]){
-	  ags_channel_recursive_play(output, devout_play->recall_id[2], stage);
+	  ags_channel_recursive_play(output, recall_id[2], stage);
 	}
       }
     }
@@ -412,11 +459,30 @@ ags_audio_thread_run(AgsThread *thread)
 
 
   /* sync - wait the 3 stages */
+  pthread_mutex_lock(audio_mutex);
+  
   output = audio->output;
 
+  pthread_mutex_unlock(audio_mutex);
+
   while(output != NULL){
-    devout_play = output->devout_play;
+    /* lookup output mutex */
+    pthread_mutex_lock(&(ags_application_mutex));
+
+    mutex_manager = ags_mutex_manager_get_instance();
+
+    output_mutex = ags_mutex_manager_lookup(mutex_manager,
+					    (GObject *) output);
+      
+    pthread_mutex_unlock(&(ags_application_mutex));
+
+    /* get devout play */
+    pthread_mutex_lock(output_mutex);
     
+    devout_play = output->devout_play;
+      
+    pthread_mutex_unlock(output_mutex);
+
     if((AGS_DEVOUT_PLAY_SUPER_THREADED_CHANNEL & (g_atomic_int_get(&(devout_play->flags)))) != 0){
       ags_audio_thread_sync_channel_super_threaded(audio_thread, devout_play);
     }
@@ -427,10 +493,10 @@ ags_audio_thread_run(AgsThread *thread)
   /* sync */
   pthread_mutex_lock(audio_thread->done_mutex);
 
-  g_atomic_int_or(&(audio_thread->flags),
-		  AGS_AUDIO_THREAD_DONE);
+  g_atomic_int_and(&(audio_thread->flags),
+		   (~AGS_AUDIO_THREAD_WAIT_SYNC));
 	    
-  if((AGS_AUDIO_THREAD_NOTIFY & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
+  if((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
     pthread_cond_signal(audio_thread->done_cond);
   }
 	    
@@ -445,6 +511,10 @@ ags_audio_thread_stop(AgsThread *thread)
   
   audio_thread = AGS_AUDIO_THREAD(thread);
 
+  /* call parent */
+  AGS_THREAD_CLASS(ags_audio_thread_parent_class)->stop(thread);
+
+  /* stop channel */
   child = thread->children;
 
   while(child != NULL){
@@ -452,20 +522,18 @@ ags_audio_thread_stop(AgsThread *thread)
     
     child = child->next;
   }
-  
-  pthread_mutex_lock(audio_thread->wakeup_mutex);
-  
-  g_atomic_int_or(&(audio_thread->flags),
-		  (AGS_AUDIO_THREAD_WAKEUP |
-		   AGS_AUDIO_THREAD_DONE));
 
-  if((AGS_AUDIO_THREAD_WAITING & (g_atomic_int_get(&(audio_thread->flags)))) != 0){
-    pthread_cond_signal(audio_thread->wakeup_cond);
+  /* ensure synced */
+  pthread_mutex_lock(audio_thread->done_mutex);
+  
+  g_atomic_int_and(&(audio_thread->flags),
+		   (~AGS_AUDIO_THREAD_WAIT_SYNC));
+
+  if((AGS_AUDIO_THREAD_DONE_SYNC & (g_atomic_int_get(&(audio_thread->flags)))) == 0){
+    pthread_cond_signal(audio_thread->done_cond);
   }
   
-  pthread_mutex_unlock(audio_thread->wakeup_mutex);
-  
-  AGS_THREAD_CLASS(ags_audio_thread_parent_class)->stop(thread);
+  pthread_mutex_unlock(audio_thread->done_mutex);
 }
 
 void
@@ -485,80 +553,94 @@ ags_audio_thread_play_channel_super_threaded(AgsAudioThread *audio_thread, AgsDe
 
   /* sequencer */
   if(audio_thread == devout_play_domain->audio_thread[1]){
+    AgsThread *thread;
+    AgsChannelThread *channel_thread;
+
+    thread = devout_play->channel_thread[1];
+    channel_thread = (AgsThread *) thread;
+    
     sequencer = TRUE;
 
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0){
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
       guint val;
 
-      ags_thread_start(devout_play->channel_thread[1]);
+      ags_thread_start(thread);
   
       /* wait child */
-      pthread_mutex_lock(devout_play->channel_thread[1]->start_mutex);
+      pthread_mutex_lock(thread->start_mutex);
 
-      val = g_atomic_int_get(&(devout_play->channel_thread[1]->flags));
-
-      if((AGS_THREAD_INITIAL_RUN & val) != 0){
-	while((AGS_THREAD_INITIAL_RUN & val) != 0){
-	  pthread_cond_wait(devout_play->channel_thread[1]->start_cond,
-			    devout_play->channel_thread[1]->start_mutex);
-
-	  val = g_atomic_int_get(&(devout_play->channel_thread[1]->flags));
+      g_atomic_int_set(&(thread->start_wait),
+		       TRUE);
+	
+      if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	 g_atomic_int_get(&(thread->start_done)) == FALSE){
+	while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	      g_atomic_int_get(&(thread->start_done)) == FALSE){
+	  pthread_cond_wait(thread->start_cond,
+			    thread->start_mutex);
 	}
       }
-
-      pthread_mutex_unlock(devout_play->channel_thread[1]->start_mutex);
+	
+      pthread_mutex_unlock(thread->start_mutex);
     }
-
+    
     /* wakeup wait */
-    pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->wakeup_mutex);
+    pthread_mutex_lock(channel_thread->wakeup_mutex);
 
-    g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-		    AGS_CHANNEL_THREAD_WAKEUP);
+    g_atomic_int_and(&(channel_thread->flags),
+		     (~AGS_CHANNEL_THREAD_WAIT));
 	    
-    if((AGS_CHANNEL_THREAD_WAITING & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags)))) != 0){
-      pthread_cond_signal(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->wakeup_cond);
+    if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(channel_thread->flags)))) == 0){
+      pthread_cond_signal(channel_thread->wakeup_cond);
     }
 	    
-    pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->wakeup_mutex);
+    pthread_mutex_unlock(channel_thread->wakeup_mutex);
   }
 
   /* notation */
   if(audio_thread == devout_play_domain->audio_thread[2]){
+    AgsThread *thread;
+    AgsChannelThread *channel_thread;
+
+    thread = devout_play->channel_thread[2];
+    channel_thread = (AgsThread *) thread;
+
     notation = TRUE;
 
-    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0){
+    if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
       guint val;
 
-      ags_thread_start(devout_play->channel_thread[2]);
+      ags_thread_start(thread);
 
       /* wait child */
-      pthread_mutex_lock(devout_play->channel_thread[2]->start_mutex);
+      pthread_mutex_lock(thread->start_mutex);
 
-      val = g_atomic_int_get(&(devout_play->channel_thread[2]->flags));
-
-      if((AGS_THREAD_INITIAL_RUN & val) != 0){
-	while((AGS_THREAD_INITIAL_RUN & val) != 0){
-	  pthread_cond_wait(devout_play->channel_thread[2]->start_cond,
-			    devout_play->channel_thread[2]->start_mutex);
-
-	  val = g_atomic_int_get(&(devout_play->channel_thread[2]->flags));
+      g_atomic_int_set(&(thread->start_wait),
+		       TRUE);
+	
+      if(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	 g_atomic_int_get(&(thread->start_done)) == FALSE){
+	while(g_atomic_int_get(&(thread->start_wait)) == TRUE &&
+	      g_atomic_int_get(&(thread->start_done)) == FALSE){
+	  pthread_cond_wait(thread->start_cond,
+			    thread->start_mutex);
 	}
       }
-
-      pthread_mutex_unlock(devout_play->channel_thread[2]->start_mutex);
+	
+      pthread_mutex_unlock(thread->start_mutex);
     }
 
     /* wakeup wait */
-    pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->wakeup_mutex);
+    pthread_mutex_lock(channel_thread->wakeup_mutex);
 	  
-    g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags),
-		    AGS_CHANNEL_THREAD_WAKEUP);
+    g_atomic_int_and(&(channel_thread->flags),
+		     (~AGS_CHANNEL_THREAD_WAIT));
 	  
-    if((AGS_CHANNEL_THREAD_WAITING & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags)))) != 0){
-      pthread_cond_signal(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->wakeup_cond);
+    if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(channel_thread->flags)))) == 0){
+      pthread_cond_signal(channel_thread->wakeup_cond);
     }
 	  
-    pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->wakeup_mutex);
+    pthread_mutex_unlock(channel_thread->wakeup_mutex);
   }
 }
 
@@ -578,64 +660,67 @@ ags_audio_thread_sync_channel_super_threaded(AgsAudioThread *audio_thread, AgsDe
 
   /* sequencer */
   if(audio_thread == devout_play_domain->audio_thread[1]){
+    AgsChannelThread *channel_thread;
+
+    channel_thread = (AgsChannelThread *) devout_play->channel_thread[1];
     sequencer = TRUE;
 
-    pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+    pthread_mutex_lock(channel_thread->done_mutex);
   
     if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) != 0 &&
-       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_0 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_1 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_2 & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0){
+       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play->channel_thread[1]->flags)))) == 0){
 
-      if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags)))) == 0){
-	g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-			AGS_CHANNEL_THREAD_NOTIFY);
-    
-	while((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags)))) == 0){
-	  pthread_cond_wait(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_cond,
-			    AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+      g_atomic_int_and(&(channel_thread->flags),
+		       (~AGS_CHANNEL_THREAD_DONE_SYNC));
+
+      if((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	 (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+	while((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	      (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+	  pthread_cond_wait(channel_thread->done_cond,
+			    channel_thread->done_mutex);
 	}
       }
     }
 
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-		     (~AGS_CHANNEL_THREAD_NOTIFY));
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->flags),
-		     (~AGS_CHANNEL_THREAD_DONE));
+    g_atomic_int_or(&(channel_thread->flags),
+		    (AGS_CHANNEL_THREAD_WAIT_SYNC |
+		     AGS_CHANNEL_THREAD_DONE_SYNC));
 
-    pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[1])->done_mutex);
+    pthread_mutex_unlock(channel_thread->done_mutex);
   }
 
   /* notation */
   if(audio_thread == devout_play_domain->audio_thread[2]){
+    AgsChannelThread *channel_thread;
+    
+    channel_thread = (AgsChannelThread *) devout_play->channel_thread[2];
     notation = TRUE;
 
-    pthread_mutex_lock(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->done_mutex);
+    pthread_mutex_lock(channel_thread->done_mutex);
 
     if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) != 0 &&
-       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_0 & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_1 & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0 &&
-       (AGS_THREAD_WAIT_2 & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0){
+       (AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(devout_play->channel_thread[2]->flags)))) == 0){
 
-      if((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags)))) == 0){
-	g_atomic_int_or(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags),
-			AGS_CHANNEL_THREAD_NOTIFY);
+      g_atomic_int_and(&(channel_thread->flags),
+		       (~AGS_CHANNEL_THREAD_DONE_SYNC));
     
-	while((AGS_CHANNEL_THREAD_DONE & (g_atomic_int_get(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags)))) == 0){
-	  pthread_cond_wait(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->done_cond,
-			    AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->done_mutex);
+      if((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	 (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+
+	while((AGS_CHANNEL_THREAD_DONE_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) == 0 &&
+	      (AGS_CHANNEL_THREAD_WAIT_SYNC & (g_atomic_int_get(&(channel_thread->flags)))) != 0){
+	  pthread_cond_wait(channel_thread->done_cond,
+			    channel_thread->done_mutex);
 	}
       }
     }
 
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags),
-		     (~AGS_CHANNEL_THREAD_NOTIFY));
-    g_atomic_int_and(&(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->flags),
-		     (~AGS_CHANNEL_THREAD_DONE));
+    g_atomic_int_or(&(channel_thread->flags),
+		     (AGS_CHANNEL_THREAD_WAIT_SYNC |
+		      AGS_CHANNEL_THREAD_DONE_SYNC));
       
-    pthread_mutex_unlock(AGS_CHANNEL_THREAD(devout_play->channel_thread[2])->done_mutex);
+    pthread_mutex_unlock(channel_thread->done_mutex);
   }
 }
 
