@@ -24,6 +24,7 @@
 #include <ags/object/ags_main_loop.h>
 #include <ags/object/ags_async_queue.h>
 
+#include <ags/thread/ags_mutex_manager.h>
 #include <ags/thread/ags_audio_loop.h>
 #include <ags/thread/ags_export_thread.h>
 #include <ags/thread/ags_devout_thread.h>
@@ -92,6 +93,8 @@ enum{
 static gpointer ags_thread_parent_class = NULL;
 static guint thread_signals[LAST_SIGNAL];
 
+extern pthread_mutex_t ags_application_mutex;
+
 __thread AgsThread *ags_thread_self = NULL;
 
 GType
@@ -299,6 +302,32 @@ ags_thread_connectable_interface_init(AgsConnectableInterface *connectable)
 void
 ags_thread_init(AgsThread *thread)
 {
+  AgsMutexManager *mutex_manager;
+
+  pthread_mutex_t *mutex;
+  pthread_mutexattr_t attr;
+
+  /* insert audio loop mutex */
+  //FIXME:JK: memory leak
+  pthread_mutexattr_init(&attr);
+  pthread_mutexattr_settype(&attr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+  mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(mutex,
+		     &attr);
+
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  mutex_manager = ags_mutex_manager_get_instance();
+
+  ags_mutex_manager_insert(mutex_manager,
+			   (GObject *) thread,
+			   mutex);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  /* fields */
   g_atomic_int_set(&(thread->flags),
 		   0);
   thread->rt_setup = FALSE;
@@ -326,6 +355,11 @@ ags_thread_init(AgsThread *thread)
   thread->start_cond = (pthread_cond_t *) malloc(sizeof(pthread_cond_t));
   pthread_cond_init(thread->start_cond, NULL);
 
+  g_atomic_int_set(&(thread->start_wait),
+		   FALSE);
+  g_atomic_int_set(&(thread->start_done),
+		   FALSE);
+  
   thread->barrier = (pthread_barrier_t **) malloc(2 * sizeof(pthread_barrier_t *));
   thread->barrier[0] = (pthread_barrier_t *) malloc(sizeof(pthread_barrier_t));
   thread->barrier[1] = (pthread_barrier_t *) malloc(sizeof(pthread_barrier_t));
@@ -369,10 +403,15 @@ ags_thread_init(AgsThread *thread)
   thread->timer_cond = (pthread_cond_t *) malloc(sizeof(pthread_cond_t));
   pthread_cond_init(thread->timer_cond, NULL);
 
-  thread->parent = NULL;
-  thread->next = NULL;
-  thread->prev = NULL;
-  thread->children = NULL;
+  g_atomic_pointer_set(&(thread->parent),
+		       NULL);
+  g_atomic_pointer_set(&(thread->children),
+		       NULL);
+  
+  g_atomic_pointer_set(&(thread->next),
+		       NULL);
+  g_atomic_pointer_set(&(thread->prev),
+		       NULL);
 
   thread->data = NULL;
 }
@@ -405,14 +444,14 @@ ags_thread_set_property(GObject *gobject,
 
       thread->devout = G_OBJECT(devout);
 
-      current = thread->children;
+      current = g_atomic_pointer_get(&(thread->children));
 
       while(current != NULL){
 	g_object_set(G_OBJECT(current),
 		     "devout\0", devout,
 		     NULL);
 
-	current = current->next;
+	current = g_atomic_pointer_get(&(current->next));
       }
     }
     break;
@@ -477,12 +516,12 @@ ags_thread_connect(AgsConnectable *connectable)
   g_message("connecting thread\0");
 #endif
 
-  child = thread->children;
+  child = g_atomic_pointer_get(&(thread->children));
 
   while(child != NULL){
     ags_connectable_connect(AGS_CONNECTABLE(child));
 
-    child = child->next;
+    child = g_atomic_pointer_get(&(child->next));
   }
 }
 
@@ -700,7 +739,7 @@ ags_thread_set_sync(AgsThread *thread, guint tic)
 void
 ags_thread_set_sync_all(AgsThread *thread, guint tic)
 {
-  AgsThread *toplevel;
+  AgsThread *main_loop;
 
   auto void ags_thread_set_sync_all_recursive(AgsThread *thread, guint tic);
 
@@ -709,22 +748,22 @@ ags_thread_set_sync_all(AgsThread *thread, guint tic)
 
     ags_thread_set_sync(thread, tic);
 
-    child = thread->children;
+    child = g_atomic_pointer_get(&(thread->children));
 
     while(child != NULL){
       ags_thread_set_sync_all_recursive(child, tic);
       
-      child = child->next;
+      child = g_atomic_pointer_get(&(child->next));
     }
   }
 
-  toplevel = ags_thread_get_toplevel(thread);
+  main_loop = ags_thread_get_toplevel(thread);
 
-  ags_thread_lock(toplevel);
+  pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
-  ags_thread_set_sync_all_recursive(toplevel, tic);
+  ags_thread_set_sync_all_recursive(main_loop, tic);
 
-  ags_thread_unlock(toplevel);
+  pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 }
 
 /**
@@ -746,16 +785,22 @@ ags_thread_lock(AgsThread *thread)
   
   main_loop = ags_thread_get_toplevel(thread);
 
+  if(AGS_IS_MAIN_LOOP(main_loop)){
+    pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
+  }
+  
   if(main_loop == thread){
     pthread_mutex_lock(thread->mutex);
     g_atomic_int_or(&(thread->flags),
 		     (AGS_THREAD_LOCKED));
   }else{
-    pthread_mutex_lock(main_loop->mutex);
     pthread_mutex_lock(thread->mutex);
     g_atomic_int_or(&(thread->flags),
 		     (AGS_THREAD_LOCKED));
-    pthread_mutex_unlock(main_loop->mutex);
+  }
+
+  if(AGS_IS_MAIN_LOOP(main_loop)){
+    pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
   }
 }
 
@@ -781,28 +826,39 @@ ags_thread_trylock(AgsThread *thread)
     
   main_loop = ags_thread_get_toplevel(thread);
 
+  if(AGS_IS_MAIN_LOOP(main_loop)){
+    pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
+  }
+  
   if(main_loop == thread){
     if(pthread_mutex_trylock(thread->mutex) != 0){
+      if(AGS_IS_MAIN_LOOP(main_loop)){
+	pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
+      }
+      
       return(FALSE);
     }
 
     g_atomic_int_or(&(thread->flags),
 		     (AGS_THREAD_LOCKED));
   }else{
-    if(pthread_mutex_trylock(main_loop->mutex) != 0){
-      return(FALSE);
-    }
 
     if(pthread_mutex_trylock(thread->mutex) != 0){
-      pthread_mutex_unlock(main_loop->mutex);
+      if(AGS_IS_MAIN_LOOP(main_loop)){
+	pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
+      }
+      
       return(FALSE);
     }
 
     g_atomic_int_or(&(thread->flags),
 		     (AGS_THREAD_LOCKED));
-    pthread_mutex_unlock(main_loop->mutex);
   }
 
+  if(AGS_IS_MAIN_LOOP(main_loop)){
+    pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
+  }
+  
   return(TRUE);
 }
 
@@ -844,8 +900,8 @@ ags_thread_get_toplevel(AgsThread *thread)
     return(NULL);
   }
 
-  while(thread->parent != NULL){
-    thread = thread->parent;
+  while(g_atomic_pointer_get(&(thread->parent)) != NULL){
+    thread = g_atomic_pointer_get(&(thread->parent));
   }
 
   return(thread);
@@ -868,8 +924,8 @@ ags_thread_first(AgsThread *thread)
     return(NULL);
   }
 
-  while(thread->prev != NULL){
-    thread = thread->prev;
+  while(g_atomic_pointer_get(&(thread->prev)) != NULL){
+    thread = g_atomic_pointer_get(&(thread->prev));
   }
 
   return(thread);
@@ -892,8 +948,8 @@ ags_thread_last(AgsThread *thread)
     return(NULL);
   }
 
-  while(thread->next != NULL){
-    thread = thread->next;
+  while(g_atomic_pointer_get(&(thread->next)) != NULL){
+    thread = g_atomic_pointer_get(&(thread->next));
   }
 
   return(thread);
@@ -912,32 +968,39 @@ void
 ags_thread_remove_child(AgsThread *thread, AgsThread *child)
 {
   AgsThread *main_loop;
-
+  AgsThread *prev, *next;
+  
   if(thread == NULL || child == NULL){
     return;
   }
 
   main_loop = ags_thread_get_toplevel(thread);
 
-  ags_thread_lock(main_loop);
+  //  pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
-  if(thread->children == child){
-    thread->children = child->next;
+  if(g_atomic_pointer_get(&(thread->children)) == child){
+    g_atomic_pointer_set(&(thread->children),
+			 g_atomic_pointer_get(&(child->next)));
   }
 
-  if(child->prev != NULL){
-    child->prev->next = child->next;
+  if((prev = g_atomic_pointer_get(&(child->prev))) != NULL){
+    g_atomic_pointer_set(&(prev->next),
+			 g_atomic_pointer_get(&(child->next)));
   }
 
-  if(child->next != NULL){
-    child->next->prev = child->prev;
+  if((next = g_atomic_pointer_get(&(child->next))) != NULL){
+    g_atomic_pointer_set(&(next->prev),
+			 g_atomic_pointer_get(&(child->prev)));
   }
 
-  child->parent = NULL;
-  child->prev = NULL;
-  child->next = NULL;
+  g_atomic_pointer_set(&(child->parent),
+		       NULL);
+  g_atomic_pointer_set(&(child->prev),
+		       NULL);
+  g_atomic_pointer_set(&(child->next),
+		       NULL);
 
-  ags_thread_unlock(main_loop);
+  //  pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 }
 
 /**
@@ -971,36 +1034,45 @@ void
 ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
 			      gboolean no_start, gboolean no_wait)
 {
-  AgsThread *main_loop;
+  AgsThread *toplevel;
 
   if(thread == NULL || child == NULL){
     return;
   }
 
-  main_loop = ags_thread_get_toplevel(thread);
+  toplevel = ags_thread_get_toplevel(thread);
 
-  if(child->parent != NULL){
+  if(g_atomic_pointer_get(&(child->parent)) != NULL){
     ags_thread_remove_child(child->parent, child);
   }
 
   /*  */
-  ags_thread_lock(main_loop);
-  
-  if(thread->children == NULL){
-    thread->children = child;
-    child->parent = thread;
+  if(AGS_IS_MAIN_LOOP(toplevel)){
+    pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(toplevel)));
+  }
+
+  if(g_atomic_pointer_get(&(thread->children)) == NULL){
+    g_atomic_pointer_set(&(thread->children),
+			 child);
+    g_atomic_pointer_set(&(child->parent),
+			 thread);
   }else{
     AgsThread *sibling;
 
-    sibling = ags_thread_last(thread->children);
+    sibling = ags_thread_last(g_atomic_pointer_get(&(thread->children)));
 
-    sibling->next = child;
-    child->prev = sibling;
-    child->parent = thread;
+    g_atomic_pointer_set(&(sibling->next),
+			 child);
+    g_atomic_pointer_set(&(child->prev),
+			 sibling);
+    g_atomic_pointer_set(&(child->parent),
+			 thread);
   }
-
-  ags_thread_unlock(main_loop);
     
+  if(AGS_IS_MAIN_LOOP(toplevel)){
+    pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(toplevel)));
+  }
+  
   if(!no_start){
     if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) != 0){
       /* start child */
@@ -1012,14 +1084,15 @@ ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
 	/* wait child */
 	pthread_mutex_lock(child->start_mutex);
 
-	val = g_atomic_int_get(&(child->flags));
-
-	if((AGS_THREAD_INITIAL_RUN & val) != 0){
-	  while((AGS_THREAD_INITIAL_RUN & val) != 0){
+	g_atomic_int_set(&(child->start_wait),
+			 TRUE);
+	
+	if(g_atomic_int_get(&(child->start_wait)) == TRUE &&
+	   g_atomic_int_get(&(child->start_done)) == FALSE){
+	  while(g_atomic_int_get(&(child->start_wait)) == TRUE &&
+		g_atomic_int_get(&(child->start_done)) == FALSE){
 	    pthread_cond_wait(child->start_cond,
 			      child->start_mutex);
-
-	    val = g_atomic_int_get(&(child->flags));
 	  }
 	}
 	
@@ -1049,7 +1122,7 @@ ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
     return(FALSE);
   }
 
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
 
   while(current != parent){
     if((AGS_THREAD_LOCKED & (g_atomic_int_get(&(current->flags)))) != 0){
@@ -1057,7 +1130,7 @@ ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
       return(TRUE);
     }
 
-    current = current->parent;
+    current = g_atomic_pointer_get(&(current->parent));
   }
 
   return(FALSE);
@@ -1082,12 +1155,12 @@ ags_thread_sibling_is_locked(AgsThread *thread)
 
   thread = ags_thread_first(thread);
 
-  while(thread->next != NULL){
+  while(g_atomic_pointer_get(&(thread->next)) != NULL){
     if((AGS_THREAD_LOCKED & (g_atomic_int_get(&(thread->flags)))) != 0){
       return(TRUE);
     }
 
-    thread = thread->next;
+    thread = g_atomic_pointer_get(&(thread->next));
   }
 
   return(FALSE);
@@ -1120,14 +1193,14 @@ ags_thread_children_is_locked(AgsThread *thread)
       return(TRUE);
     }
 
-    child = thread->children;
+    child = g_atomic_pointer_get(&(thread->children));
 
     while(child != NULL){
       if(ags_thread_children_is_locked_recursive(child)){
 	return(TRUE);
       }
 
-      child = child->next;
+      child = g_atomic_pointer_get(&(child->next));
     }
 
     return(FALSE);
@@ -1280,7 +1353,7 @@ ags_thread_is_tree_ready(AgsThread *thread,
   gboolean ags_thread_is_tree_ready_recursive(AgsThread *current){
     AgsThread *children;
 
-    children = current->children;
+    children = g_atomic_pointer_get(&(current->children));
 
     if(!ags_thread_is_tree_ready_current_tic(current)){
       return(FALSE);
@@ -1291,7 +1364,7 @@ ags_thread_is_tree_ready(AgsThread *thread,
 	return(FALSE);
       }
 
-      children = children->next;
+      children = g_atomic_pointer_get(&(children->next));
     }
 
     return(TRUE);
@@ -1318,14 +1391,14 @@ ags_thread_next_parent_locked(AgsThread *thread, AgsThread *parent)
 {
   AgsThread *current;
 
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
 
   while(current != parent){
     if((AGS_THREAD_WAITING_FOR_CHILDREN & (g_atomic_int_get(&(current->flags)))) != 0){
       return(current);
     }
 
-    current = current->parent;
+    current = g_atomic_pointer_get(&(current->parent));
   }
 
   return(NULL);
@@ -1348,7 +1421,7 @@ ags_thread_next_sibling_locked(AgsThread *thread)
 
   while(current != NULL){
     if(current == thread){
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
       
       continue;
     }
@@ -1357,7 +1430,7 @@ ags_thread_next_sibling_locked(AgsThread *thread)
       return(current);
     }
 
-    current = current->next;
+    current = g_atomic_pointer_get(&(current->next));
   }
 
   return(NULL);
@@ -1382,19 +1455,19 @@ ags_thread_next_children_locked(AgsThread *thread)
     current = ags_thread_last(child);
 
     while(current != NULL){
-      ags_thread_next_children_locked_recursive(current->children);
+      ags_thread_next_children_locked_recursive(g_atomic_pointer_get(&(current->children)));
 
       if((AGS_THREAD_WAITING_FOR_PARENT & (g_atomic_int_get(&(current->flags)))) != 0){
 	return(current);
       }
 
-      current = current->prev;
+      current = g_atomic_pointer_get(&(current->prev));
     }
 
     return(NULL);
   }
 
-  return(ags_thread_next_children_locked(thread->children));
+  return(ags_thread_next_children_locked(g_atomic_pointer_get(&(thread->children))));
 }
 
 /**
@@ -1417,14 +1490,14 @@ ags_thread_lock_parent(AgsThread *thread, AgsThread *parent)
 
   ags_thread_lock(thread);
 
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
 
   while(current != parent){
     ags_thread_lock(current);
     g_atomic_int_or(&(current->flags),
 		    AGS_THREAD_WAITING_FOR_CHILDREN);
 
-    current = current->parent;
+    current = g_atomic_pointer_get(&(current->parent));
   }
 }
 
@@ -1451,7 +1524,7 @@ ags_thread_lock_sibling(AgsThread *thread)
 
   while(current != NULL){
     if(current == thread){
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
     
       continue;
     }
@@ -1460,7 +1533,7 @@ ags_thread_lock_sibling(AgsThread *thread)
     g_atomic_int_or(&(current->flags),
 		    AGS_THREAD_WAITING_FOR_SIBLING);
 
-    current = current->next;
+    current = g_atomic_pointer_get(&(current->next));
   }
 }
 
@@ -1483,19 +1556,19 @@ ags_thread_lock_children(AgsThread *thread)
     current = ags_thread_last(child);
 
     while(current != NULL){
-      ags_thread_lock_children_recursive(current->children);
+      ags_thread_lock_children_recursive(g_atomic_pointer_get(&(current->children)));
 
       ags_thread_lock(current);
       g_atomic_int_or(&(current->flags),
 		      AGS_THREAD_WAITING_FOR_PARENT);
       
-      current = current->prev;
+      current = g_atomic_pointer_get(&(current->prev));
     }
   }
 
   ags_thread_lock(thread);
   
-  ags_thread_lock_children_recursive(thread->children);
+  ags_thread_lock_children_recursive(g_atomic_pointer_get(&(thread->children)));
 }
 
 void
@@ -1524,7 +1597,7 @@ ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
     return;
   }
 
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
 
   while(current != parent){
     g_atomic_int_and(&(current->flags),
@@ -1538,7 +1611,7 @@ ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
 
     ags_thread_unlock(current);
 
-    current = current->parent;
+    current = g_atomic_pointer_get(&(current->parent));
   }
 }
 
@@ -1563,7 +1636,7 @@ ags_thread_unlock_sibling(AgsThread *thread)
 
   while(current != NULL){
     if(current == thread){
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
     
       continue;
     }
@@ -1579,7 +1652,7 @@ ags_thread_unlock_sibling(AgsThread *thread)
 
     ags_thread_unlock(current);
 
-    current = current->next;
+    current = g_atomic_pointer_get(&(current->next));
   }
 }
 
@@ -1606,7 +1679,7 @@ ags_thread_unlock_children(AgsThread *thread)
     current = ags_thread_last(child);
 
     while(current != NULL){
-      ags_thread_unlock_children_recursive(current->children);
+      ags_thread_unlock_children_recursive(g_atomic_pointer_get(&(current->children)));
 
       g_atomic_int_and(&(current->flags),
 		       (~AGS_THREAD_WAITING_FOR_PARENT));
@@ -1623,11 +1696,11 @@ ags_thread_unlock_children(AgsThread *thread)
 
       ags_thread_unlock(current);
 
-      current = current->prev;
+      current = g_atomic_pointer_get(&(current->prev));
     }
   }
   
-  ags_thread_unlock_children_recursive(thread->children);
+  ags_thread_unlock_children_recursive(g_atomic_pointer_get(&(thread->children)));
 }
 
 void
@@ -1657,18 +1730,18 @@ ags_thread_wait_parent(AgsThread *thread, AgsThread *parent)
   }
 
   /* wait parent */
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
     
   while((current != NULL && current != parent) &&
 	(((AGS_THREAD_IDLE & (g_atomic_int_get(&(current->flags)))) != 0 ||
 	  (AGS_THREAD_WAITING_FOR_CHILDREN & (g_atomic_int_get(&(current->flags)))) == 0) ||
-	 current->parent != parent)){
+	 g_atomic_pointer_get(&(current->parent)) != parent)){
     pthread_cond_wait(current->cond,
 		      current->mutex);
     
     if(!((AGS_THREAD_IDLE & (g_atomic_int_get(&(current->flags)))) != 0 ||
 	 (AGS_THREAD_WAITING_FOR_CHILDREN & (g_atomic_int_get(&(current->flags)))) == 0)){
-      current = current->parent;
+      current = g_atomic_pointer_get(&(current->parent));
     }
   }
 
@@ -1700,9 +1773,9 @@ ags_thread_wait_sibling(AgsThread *thread)
   while(current != NULL &&
 	(((AGS_THREAD_IDLE & (g_atomic_int_get(&(current->flags)))) != 0 ||
 	  (AGS_THREAD_WAITING_FOR_SIBLING & (g_atomic_int_get(&(current->flags)))) == 0) ||
-	 current->next != NULL)){
+	 g_atomic_pointer_get(&(current->next)) != NULL)){
     if(current == thread){
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
       
       continue;
     }
@@ -1712,7 +1785,7 @@ ags_thread_wait_sibling(AgsThread *thread)
     
     if(!((AGS_THREAD_IDLE & (g_atomic_int_get(&(current->flags)))) != 0 ||
 	 (AGS_THREAD_WAITING_FOR_SIBLING & (g_atomic_int_get(&(current->flags)))) == 0)){
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
     }
   }
 
@@ -1746,9 +1819,9 @@ ags_thread_wait_children(AgsThread *thread)
     while(child != NULL &&
 	  (((AGS_THREAD_IDLE & (g_atomic_int_get(&(child->flags)))) != 0 ||
 	    (AGS_THREAD_WAITING_FOR_PARENT & (g_atomic_int_get(&(child->flags)))) == 0) ||
-	   child->next != NULL)){
+	   g_atomic_pointer_get(&(child->next)) != NULL)){
       if(initial_run){
-	ags_thread_wait_children_recursive(child->children);
+	ags_thread_wait_children_recursive(g_atomic_pointer_get(&(child->children)));
 
 	initial_run = FALSE;
       }
@@ -1758,7 +1831,7 @@ ags_thread_wait_children(AgsThread *thread)
      
       if(!((AGS_THREAD_IDLE & (g_atomic_int_get(&(child->flags)))) != 0 ||
 	   (AGS_THREAD_WAITING_FOR_PARENT & (g_atomic_int_get(&(child->flags)))) == 0)){
-	child = child->next;
+	child = g_atomic_pointer_get(&(child->next));
 
 	initial_run = TRUE;
       }
@@ -1770,7 +1843,7 @@ ags_thread_wait_children(AgsThread *thread)
   }
 
   /* wait children */
-  ags_thread_wait_children_recursive(thread->children);
+  ags_thread_wait_children_recursive(g_atomic_pointer_get(&(thread->children)));
 
   /* unset flags */
   g_atomic_int_and(&(thread->flags),
@@ -1792,7 +1865,7 @@ ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
 {
   AgsThread *current;
 
-  current = thread->parent;
+  current = g_atomic_pointer_get(&(thread->parent));
 
   while(current != NULL){
     if((AGS_THREAD_WAIT_FOR_CHILDREN & (g_atomic_int_get(&(current->flags)))) != 0){
@@ -1803,7 +1876,7 @@ ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
       }
     }
 
-    current = current->parent;
+    current = g_atomic_pointer_get(&(current->parent));
   }
 }
 
@@ -1831,6 +1904,8 @@ ags_thread_signal_sibling(AgsThread *thread, gboolean broadcast)
 	pthread_cond_broadcast(current->cond);
       }
     }
+
+    current = g_atomic_pointer_get(&(current->next));
   }
 }
 
@@ -1868,11 +1943,11 @@ ags_thread_signal_children(AgsThread *thread, gboolean broadcast)
       
       ags_thread_signal_children_recursive(current, broadcast);
 
-      current = current->next;
+      current = g_atomic_pointer_get(&(current->next));
     }
   }
 
-  ags_thread_signal_children(thread->children, broadcast);
+  ags_thread_signal_children(g_atomic_pointer_get(&(thread->children)), broadcast);
 }
 
 void
@@ -1969,7 +2044,7 @@ ags_thread_loop(void *ptr)
       tic = 2;
     }
 
-    pthread_mutex_lock(main_loop->mutex);
+    pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
     switch(current_tic){
     case 0:
@@ -1998,13 +2073,13 @@ ags_thread_loop(void *ptr)
     
       while(!ags_thread_is_current_ready(thread,
 					 current_tic)){
-	pthread_mutex_unlock(main_loop->mutex);
+	pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 	pthread_cond_wait(thread->cond,
 			  thread->mutex);
-	pthread_mutex_lock(main_loop->mutex);
+	pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
       }
 
-      pthread_mutex_unlock(main_loop->mutex);
+      pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
       ags_main_loop_set_last_sync(AGS_MAIN_LOOP(main_loop), current_tic);
       ags_main_loop_set_tic(AGS_MAIN_LOOP(main_loop), next_tic);
@@ -2015,7 +2090,7 @@ ags_thread_loop(void *ptr)
 
       /* thread tree */
       ags_thread_set_sync_all(main_loop, current_tic);
-      pthread_mutex_unlock(main_loop->mutex);
+      pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
       
       ags_main_loop_set_last_sync(AGS_MAIN_LOOP(main_loop), current_tic);
       ags_main_loop_set_tic(AGS_MAIN_LOOP(main_loop), next_tic);
@@ -2073,7 +2148,7 @@ ags_thread_loop(void *ptr)
   
   /*  */
 #ifndef AGS_USE_TIMER
-  if(thread->parent == NULL){
+  if(g_atomic_pointer_get(&(thread->parent)) == NULL){
     clock_gettime(CLOCK_MONOTONIC, &time_prev);
   }
 #endif
@@ -2093,7 +2168,7 @@ ags_thread_loop(void *ptr)
     running = g_atomic_int_get(&(thread->flags));
 
     /* idle */
-    if(thread->parent == NULL){
+    if(g_atomic_pointer_get(&(thread->parent)) == NULL){
 #ifdef AGS_USE_TIMER
       pthread_mutex_lock(thread->timer_mutex);
 
@@ -2152,14 +2227,19 @@ ags_thread_loop(void *ptr)
 	if((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) != 0){
 	  /* unset initial run */
 	  /* signal AgsAudioLoop */
-	  pthread_mutex_lock(thread->mutex);
+	  pthread_mutex_lock(thread->start_mutex);
 
 	  g_atomic_int_and(&(thread->flags),
 			   (~AGS_THREAD_INITIAL_RUN));
-
-	  pthread_cond_broadcast(thread->start_cond);
 	  
-	  pthread_mutex_unlock(thread->mutex);
+	  g_atomic_int_set(&(thread->start_done),
+			   TRUE);
+
+	  if(g_atomic_int_get(&(thread->start_wait)) == TRUE){
+	    pthread_cond_broadcast(thread->start_cond);
+	  }
+	  
+	  pthread_mutex_unlock(thread->start_mutex);
 	}else{
 	  /* run in hierarchy */
 	  pthread_mutex_lock(thread->mutex);
@@ -2188,7 +2268,19 @@ ags_thread_loop(void *ptr)
 			   (~AGS_THREAD_WAIT_0));
 
 	  /* signal AgsAudioLoop */
-	  pthread_cond_broadcast(thread->start_cond);
+	  pthread_mutex_lock(thread->start_mutex);
+
+	  g_atomic_int_and(&(thread->flags),
+			   (~AGS_THREAD_INITIAL_RUN));
+	  
+	  g_atomic_int_set(&(thread->start_done),
+			   TRUE);
+
+	  if(g_atomic_int_get(&(thread->start_wait)) == TRUE){
+	    pthread_cond_broadcast(thread->start_cond);
+	  }
+	  
+	  pthread_mutex_unlock(thread->start_mutex);
 	}else{
 	  /* run in hierarchy */
 	  pthread_mutex_lock(thread->mutex);
@@ -2414,9 +2506,6 @@ ags_thread_loop(void *ptr)
 	}
       }
 
-      /**/
-      ags_thread_lock(thread);
-
       /* unset idle flag */
       g_atomic_int_and(&(thread->flags),
 		       (~AGS_THREAD_IDLE));
@@ -2445,13 +2534,21 @@ ags_thread_loop(void *ptr)
 			   (~AGS_THREAD_WAIT_0));
 
 	  /* signal AgsAudioLoop */
-	  if(AGS_IS_TASK_THREAD(thread)){
-	    pthread_cond_signal(thread->start_cond);
+	  pthread_mutex_lock(thread->start_mutex);
+
+	  g_atomic_int_and(&(thread->flags),
+			   (~AGS_THREAD_INITIAL_RUN));
+	  
+	  g_atomic_int_set(&(thread->start_done),
+			   TRUE);
+
+	  if(g_atomic_int_get(&(thread->start_wait)) == TRUE){
+	    pthread_cond_broadcast(thread->start_cond);
 	  }
+	  
+	  pthread_mutex_unlock(thread->start_mutex);
 	}
       }
-
-      ags_thread_unlock(thread);
     }
 
     pthread_yield();
@@ -2473,10 +2570,10 @@ ags_thread_loop(void *ptr)
     tic = 2;
   }
 
-  pthread_mutex_lock(main_loop->mutex);
+  pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
   if((AGS_THREAD_UNREF_ON_EXIT & (g_atomic_int_get(&(thread->flags)))) != 0){
-    ags_thread_remove_child(thread->parent,
+    ags_thread_remove_child(g_atomic_pointer_get(&(thread->parent)),
     			    thread);
     g_object_unref(thread);
   }
@@ -2487,12 +2584,12 @@ ags_thread_loop(void *ptr)
 
     ags_thread_set_sync_all(main_loop, current_tic);
 
-    pthread_mutex_unlock(main_loop->mutex);
+    pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
     ags_main_loop_set_last_sync(AGS_MAIN_LOOP(main_loop), current_tic);
     ags_main_loop_set_tic(AGS_MAIN_LOOP(main_loop), next_tic);
   }else{
-    pthread_mutex_unlock(main_loop->mutex);
+    pthread_mutex_unlock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
   }
 
 #ifdef AGS_DEBUG
@@ -2798,7 +2895,7 @@ ags_thread_hangcheck(AgsThread *thread)
     while(child != NULL){
       ags_thread_hangcheck_recursive(child);
 
-      child = child->next;
+      child = g_atomic_pointer_get(&(child->next));
     }
   }
   void ags_thread_hangcheck_unsync_all(AgsThread *thread, gboolean broadcast){
@@ -2836,12 +2933,12 @@ ags_thread_hangcheck(AgsThread *thread)
     }
 
     /* iterate tree */
-    child = thread->children;
+    child = g_atomic_pointer_get(&(thread->children));
 
     while(child != NULL){
       ags_thread_hangcheck_unsync_all(child, broadcast);
 
-      child = child->next;
+      child = g_atomic_pointer_get(&(child->next));
     }
   }
 
@@ -2868,23 +2965,43 @@ AgsThread*
 ags_thread_find_type(AgsThread *thread, GType type)
 {
   AgsThread *current, *retval;
+  AgsMutexManager *mutex_manager;
+    
+  GType current_type;
 
+  pthread_mutex_t *current_mutex;
+  
   if(thread == NULL || type == G_TYPE_NONE){
     return(NULL);
   }
 
-  if(g_type_is_a(G_OBJECT_TYPE(thread), type)){
+  pthread_mutex_lock(&(ags_application_mutex));
+
+  mutex_manager = ags_mutex_manager_get_instance();
+
+  current_mutex = ags_mutex_manager_lookup(mutex_manager,
+					   (GObject *) thread);
+  
+  pthread_mutex_unlock(&(ags_application_mutex));
+
+  pthread_mutex_lock(current_mutex);
+  
+  current_type = G_OBJECT_TYPE(thread);
+
+  pthread_mutex_unlock(current_mutex);
+  
+  if(g_type_is_a(current_type, type)){
     return(thread);
   }
   
-  current = thread->children;
+  current = g_atomic_pointer_get(&(thread->children));
 
   while(current != NULL){
     if((retval = ags_thread_find_type(current, type)) != NULL){
       return(retval);
     }
     
-    current = current->next;
+    current = g_atomic_pointer_get(&(current->next));
   }
 
   
